@@ -1,6 +1,6 @@
 #!/usr/local/bin/wish -f
 
-#charles vidal 1997 <cvidal@newlog.com>
+#charles vidal 1997 <vidalc@club-internet.fr>
 
 #function find in demo: mkStyles.tcl
 # The procedure below inserts text into a given text widget and
@@ -16,10 +16,11 @@ set ftname ""
 set mftname ""
 set usernom ""
 set passmot ""
+set validstr ""
+set invalidstr ""
 set prgname ""
 set geometrie ""
 set icogeometrie ""
-set passstring ""
 set infostring ""
 proc insertWithTags {w text args} {
     set start [$w index insert]
@@ -31,7 +32,29 @@ proc insertWithTags {w text args} {
         $w tag add $i $start insert
     }
 }
+# Function for the help
+proc mkHelpCheck { w args } {
+  set nbf 0
+  foreach i $args {
+	set nbf [ expr $nbf +1 ]
+	$w insert insert  "\n"
+	checkbutton $w.c$nbf -variable [lindex $i 0] -text [lindex $i 0]
+	$w window create {end lineend} -window $w.c$nbf
+	$w insert insert  " [lindex $i 1] "
+	} 
+}
+proc mkHelpEntry { w args } {
+  set nbf 0
 
+  foreach i $args {
+	set nbf [ expr $nbf +1 ]
+	insertWithTags  $w "\n [lindex $i 0] " underline
+	entry $w.e$nbf -textvariable [lindex $i 1]
+	$w window create {end lineend} -window $w.e$nbf
+	$w insert insert  "\n[lindex $i 2] "
+	} 
+}
+#
 proc whichcolor { which } {
 global fgcolor
 global bgcolor
@@ -45,6 +68,9 @@ proc mkColor { what } {
 	wm title .color "Color"
     frame .color.frame -borderwidth 10
     frame .color.frame2 -borderwidth 10
+    set w .color.frame
+    label $w.msg0   -text "Color Options" 
+    pack $w.msg0 -side top
 foreach i {/usr/local/lib/X11/rgb.txt /usr/lib/X11/rgb.txt
         /X11/R5/lib/X11/rgb.txt /X11/R4/lib/rgb/rgb.txt} {
     if ![file readable $i] {
@@ -82,18 +108,20 @@ foreach i {/usr/local/lib/X11/rgb.txt /usr/lib/X11/rgb.txt
 proc Helpxlock {} {
      toplevel .help
 	wm title .help "Help About Xlock"
-scrollbar .help.s -orient vertical -command {.help.t yview}
-pack .help.s -side right -fill y
-text .help.t -yscrollcommand {.help.s set} -wrap word -width 60 -height 30 \
+frame .help.f 
+scrollbar .help.f.s -orient vertical -command {.help.f.t yview}
+pack .help.f
+pack .help.f.s -side right -fill y
+text .help.f.t -yscrollcommand {.help.f.s set} -wrap word -width 60 -height 30 \
         -setgrid 1
-pack .help.t -expand y -fill both
-.help.t tag configure big -font -Adobe-Courier-Bold-R-Normal-*-140-*
-.help.t tag configure verybig -font -Adobe-Courier-Bold-R-Normal-*-240-*
-.help.t tag configure color2 -foreground red
-.help.t tag configure underline -underline on
-insertWithTags .help.t  "Xlock Help\n"  verybig
-insertWithTags .help.t "\n"
-.help.t insert end { Locks the X server still the user enters their pass\
+pack .help.f.t -expand y -fill both
+.help.f.t tag configure big -font -Adobe-Courier-Bold-R-Normal-*-140-*
+.help.f.t tag configure verybig -font -Adobe-Courier-Bold-R-Normal-*-240-*
+.help.f.t tag configure color2 -foreground red
+.help.f.t tag configure underline -underline on
+insertWithTags .help.f.t  "Xlock Help\n"  verybig
+insertWithTags .help.f.t "\n"
+.help.f.t insert end { Locks the X server still the user enters their pass\
 word at the keyboard.  While xlock  is  running,  all  new\
 server  connections are refused.  The screen saver is dis\
 abled.  The mouse cursor is turned  off.   The  screen  is\
@@ -106,89 +134,67 @@ password  Control-U  and  Control-H are active as kill and\
 erase respectively.  To return to the locked screen, click\
 in the small icon version of the changing pattern.
 }
-.help.t insert end "\n"
-insertWithTags .help.t  "OPTIONS\n"  big
-insertWithTags .help.t  "\n\n-display "  underline
-insertWithTags .help.t {The option sets  the  X11  display  to  lock.\
+.help.f.t insert end "\n"
+insertWithTags .help.f.t  "OPTIONS\n"  big
+insertWithTags .help.f.t  "\n\n-display "  underline
+insertWithTags .help.f.t {The option sets  the  X11  display  to  lock.\
             xlock  locks all available screens on a given server,\
             and restricts you to locking only a local server such\
             as  unix::00,,  localhost::00,,  or  ::00  unless you set the\
      -remote option.}
-.help.t insert end "\n"
-insertWithTags .help.t  "\n\n-name "  underline
-insertWithTags .help.t  { is used instead of XLock  when  looking\
+.help.f.t insert end "\n"
+insertWithTags .help.f.t  "\n\n-name "  underline
+insertWithTags .help.f.t  { is used instead of XLock  when  looking\
             for resources to configure xlock.}
-.help.t insert end "\n"
-insertWithTags .help.t  "\n\n-mode "  underline
-insertWithTags .help.t {As  of  this  writing there are 48 display modes sup\
+.help.f.t insert end "\n"
+insertWithTags .help.f.t  "\n\n-mode "  underline
+insertWithTags .help.f.t {As  of  this  writing there are 48 display modes sup\
             ported (plus one more for random selection of one  of\
             the 48).}
-insertWithTags .help.t  "\n\n-delay "  underline
-insertWithTags .help.t {It simply sets the number  of  microseconds to  delay  between  batches  of animations.  In blank mode, it is important to set this to some small  number  of  microseconds,  because the keyboard and mouse are only checked after each delay, so you cannot set  the delay  too high, but a delay of zero would needlessly consume cpu checking for mouse and keyboard input  in a tight loop, since blank mode has no work to do.}
-insertWithTags .help.t  "\n\n-saturation"  underline
-insertWithTags .help.t { This option sets saturation  of  the  color
+insertWithTags .help.f.t  "\n\n-delay "  underline
+insertWithTags .help.f.t {It simply sets the number  of  microseconds to  delay  between  batches  of animations.  In blank mode, it is important to set this to some small  number  of  microseconds,  because the keyboard and mouse are only checked after each delay, so you cannot set  the delay  too high, but a delay of zero would needlessly consume cpu checking for mouse and keyboard input  in a tight loop, since blank mode has no work to do.}
+insertWithTags .help.f.t  "\n\n-saturation"  underline
+insertWithTags .help.f.t { This option sets saturation  of  the  color
  ramp .  0 is grayscale and 1 is very rich color. 0.4 is a nice pastel.}
-insertWithTags .help.t  "\n\n-nice"  underline
-insertWithTags .help.t { This option sets system nicelevel  of  the  xlock .}
-insertWithTags .help.t  "\n\n-timeout "  underline
-insertWithTags .help.t { This option sets the number of second before the password screen will time out.}
-insertWithTags .help.t  "\n\n-lockdelay "  underline
-insertWithTags .help.t { This  option  sets  the  number  of second before  the  screen  needs a password to be unlocked.  Good for use with an autolocking mechanism like xautolock(1).}
-insertWithTags .help.t  "\n\n-font "  underline
-insertWithTags .help.t { Ths option  sets  the  font  to be used on the prompt screen.}
-insertWithTags .help.t  "\n\n-fg "  underline
-insertWithTags .help.t { This option sets the color of the text on the password screen.}
-insertWithTags .help.t  "\n\n-bg "  underline
-insertWithTags .help.t { This option sets the color of the background on the password screen.}
-insertWithTags .help.t  "\n\n-usename "  underline
-insertWithTags .help.t { usename is shown in front of user  name,  defaults to "Name: ".}
-insertWithTags .help.t  "\n\n-password "  underline
-insertWithTags .help.t { the password prompt string, defaults to "Password: ".}
-insertWithTags .help.t  "\n\n-info "  underline
-insertWithTags .help.t { the informational message  to  tell  the user  what  to  do,  defaults  to  "Enter password to
- unlock; select icon to lock.".}
-insertWithTags .help.t  "\n\n-validate "  underline
-insertWithTags .help.t { the message shown  while  validating  the password, defaults to "Validating login..."}
-insertWithTags .help.t  "\n\n-invalid"  underline
-insertWithTags .help.t {
-the  message  shown  when  password  is invalid, defaults to "Invalid login."}
-insertWithTags .help.t  "\n\n-geometry "  underline
-insertWithTags .help.t { This option sets the size and offset  of the  lock  window  (normally the entire screen).  The entire screen format is still used for  entering  the password.   The  purpose  is  to  see the screen even though it is locked.  This should be used  with  caution since many of the modes will fail if the windows are far from square or are too small  (size  must  be greater  than  0x0).   This  should also be used with esaver to protect screen from phosphor burn.}
-insertWithTags .help.t  "\n\n-icongeometry "  underline
-insertWithTags .help.t { this option sets  the  size  of  the iconic screen (normally 64x64) seen when entering the password.  This should be  used  with  caution  since many  of  the  modes will fail if the windows are far from square or are too small (size  must  be  greater than  0x0).   The  greatest  size  is 256x256.  There should be some limit  so  users  could  see  who  has locked  the  screen.  Position information of icon is ignored.}
-insertWithTags .help.t  "\n\n-forceLogout "  underline
-insertWithTags .help.t { This option sets the  auto-logout.  This  might not be enforced depending how your system is configured.}
-insertWithTags .help.t  "\n\nOPTIONS BOOLEAN\n"  big
-    insertWithTags .help.t "\n-/+mono " underline
-    insertWithTags .help.t " turn on/off monochrome override"
-    insertWithTags .help.t "\n-/+nolock" underline    
-     insertWithTags .help.t " turn on/off no password required mode"
-    insertWithTags .help.t "\n-/+remote"  underline           
-insertWithTags .help.t " turn on/off remote host access"
-    insertWithTags .help.t "\n-/+allowroot" underline      
-     insertWithTags .help.t " turn on/off allow root password mode (ignored)"
-    insertWithTags .help.t "\n-/+enablesaver"  underline      
-  insertWithTags .help.t " turn on/off enable X server screen saver"
-    insertWithTags .help.t "\n-/+allowaccess"  underline    
-    insertWithTags .help.t " turn on/off allow new clients to connect"
-    insertWithTags .help.t "\n-/+grabmouse  "  underline      
-  insertWithTags .help.t " turn on/off grabbing of mouse and keyboard"
-    insertWithTags .help.t "\n-/+echokeys "  underline      
-    insertWithTags .help.t " turn on/off echo '?' for each password key"
-    insertWithTags .help.t "\n-/+usefirst"  underline        
-   insertWithTags .help.t " turn on/off using the first char typed in password"
-    insertWithTags .help.t "\n-/+v  "   underline         
-      insertWithTags .help.t " turn on/off verbose mode"
-    insertWithTags .help.t "\n-/+inwindow"  underline       
-    insertWithTags .help.t " turn on/off making xlock run in a window"
-    insertWithTags .help.t "\n-/+mono "   underline           
-  insertWithTags .help.t " turn on/off monochrome override"
-    insertWithTags .help.t "\n-/+inroot"  underline        
-     insertWithTags .help.t " turn on/off making xlock run in the root window"
-    insertWithTags .help.t "\n-/+timeelapsed "  underline      
- insertWithTags .help.t " turn on/off clock"
-    insertWithTags .help.t "\n-/+install " underline        
- insertWithTags .help.t " whether to use private colormap if needed (yes/no)"
+insertWithTags .help.f.t  "\n\n-nice"  underline
+insertWithTags .help.f.t { This option sets system nicelevel  of  the  xlock .}
+insertWithTags .help.f.t  "\n\n-timeout "  underline
+insertWithTags .help.f.t { This option sets the number of second before the password screen will time out.}
+insertWithTags .help.f.t  "\n\n-lockdelay "  underline
+insertWithTags .help.f.t { This  option  sets  the  number  of second before  the  screen  needs a password to be unlocked.  Good for use with an autolocking mechanism like xautolock(1).}
+insertWithTags .help.f.t  "\n\n-font "  underline
+#    button .help.f.t.b1 -text "font" -command "mkFont FONT"
+#    .help.f.t window create {end lineend} -window .help.f.t.b1
+insertWithTags .help.f.t { Ths option  sets  the  font  to be used on the prompt screen.}
+insertWithTags .help.f.t  "\n\n-fg "  underline
+insertWithTags .help.f.t { This option sets the color of the text on the password screen.}
+insertWithTags .help.f.t  "\n\n-bg "  underline
+insertWithTags .help.f.t { This option sets the color of the background on the password screen.}
+mkHelpEntry .help.f.t {-username usernom {text string to use for Name prompt}}\
+	{-password passmot {text string to use for Password prompt}}\
+	{-info infostring {text string to use for instructions}}\
+	{-validate validstr {the message shown  while  validating  the password, defaults to \"Validating login...\"}}\
+	{-invalid invalidstr {the  message  shown  when  password  is invalid, defaults to \"Invalid login.\"}}\
+	{-geometry geometrie {This option sets the size and offset  of the  lock  window  (normally the entire screen).  The entire screen format is still used for  entering  the password.   The  purpose  is  to  see the screen even though it is locked.  This should be used  with  caution since many of the modes will fail if the windows are far from square or are too small  (size  must  be greater  than  0x0).   This  should also be used with esaver to protect screen from phosphor burn.}}\
+{-icongeometry icongeometrie {this option sets  the  size  of  the iconic screen (normally 64x64) seen when entering the password.  This should be  used  with  caution  since many  of  the  modes will fail if the windows are far from square or are too small (size  must  be  greater than  0x0).   The  greatest  size  is 256x256.  There should be some limit  so  users  could  see  who  has locked  the  screen.  Position information of icon is ignored.}}
+insertWithTags .help.f.t  "\n\n-forceLogout "  underline
+insertWithTags .help.f.t { This option sets the  auto-logout.  This  might not be enforced depending how your system is configured.}
+insertWithTags .help.f.t  "\n\nOPTIONS BOOLEAN\n"  big
+  mkHelpCheck .help.f.t {mono {turn on/off monochrome override}}\
+		{nolock {trun on/off no password required mode}}\
+		{remote {turn on/off remote host access}}\
+		{allowroot {turn on/off allow root password mode (ignored)}}\
+		{enablesaver {turn on/off enable X server screen saver}}\
+		{allowaccess {turn on/off access of the terminal X}}\
+		{grabmouse {turn on/off grabbing of mouse and keyboard}}\
+		{echokeys {turn on/off echo \'?\' for each password key}}\
+		{usefirst {turn on/off using the first char typed in password}}\
+		{verbose {turn on/off verbose mode}}\
+		{inwindow {turn on/off making xlock run in a window}}\
+		{inroot {turn on/off making xlock run in the root window}}\
+		{timeelapsed {turn on/off clock}}\
+		{install {whether to use private colormap if needed (yes/no)}}\
+		{sound {whether to use sound if configured for it (yes/no}}
 
         button .help.ok -text OK -command "destroy .help"
 	pack  .help.ok
@@ -200,12 +206,13 @@ proc mkAuthor {} {
     frame .author.frame -borderwidth 10
     set w .author.frame
 
-    label $w.msg1   -text "Maintained by: David A. Bagley (bagleyd@hertz.njit.edu)"
+    label $w.msg0   -text "Author and Maintener of xlock" 
+    label $w.msg1   -text "Maintained by: David A. Bagley (bagleyd@bigfoot.com)"
 label $w.msg2   -text "Original Author: Patrick J. Naughton (naughton@eng.sun.com)" 
 label $w.msg3   -text "Mailstop 21-14 Sun Microsystems Laboratories,"
  label $w.msg4   -text "Inc.  Mountain View, CA  94043 15//336-1080"
 label $w.msg5   -text "with many additional contributors"
-        pack $w.msg1 $w.msg2 $w.msg3 $w.msg4 $w.msg5 -side top
+        pack $w.msg0 $w.msg1 $w.msg2 $w.msg3 $w.msg4 $w.msg5 -side top
 
         label $w.msg6   -text "xlock.tcl\n created by charles VIDAL\n (author of flag mode and xmlock launcher )"
         pack $w.msg6 -side top
@@ -213,57 +220,51 @@ label $w.msg5   -text "with many additional contributors"
     button .author.ok -text OK -command "destroy .author"
     pack $w .author.ok 
 }
+proc mkDialog { nom titre args } {
+  toplevel .$nom
+  wm title .$nom "$titre"
+  frame .$nom.frame -borderwidth 10
+  frame .$nom.frame2 -borderwidth 10
+  set w .$nom.frame
+  set w2 .$nom.frame2
+  set nbf 0
 
-#no influence with xlock
+  label $w.msg0   -text "$titre" 
+  pack $w.msg0 -side top
+  foreach i $args {
+	set nbf [ expr $nbf +1 ]
+	frame $w.f$nbf
+	set wf $w.f$nbf
+	label $wf.l$nbf -text [lindex $i 0]
+	entry $wf.e$nbf -textvariable [lindex $i 1]
+	pack $wf.l$nbf  -side left
+	pack $wf.e$nbf  -side right
+	pack $wf
+	} 
+    button $w2.ok -text OK -command "destroy .$nom"
+    button $w2.cancel -text Cancel -command "destroy .$nom"
+    pack $w -side top 
+    pack $w2.ok $w2.cancel -side left -fill x
+	pack $w2  -side bottom
+}
 proc mkMessage {} {
-global passstring 
+global passmot
+global validstr
+global invalidstr
 global infostring 
-     toplevel .message
-	wm title .message "Message Options"
-    frame .message.frame -borderwidth 10
-    frame .message.frame2 -borderwidth 10
-    set w .message.frame
-
-    frame $w.messagename
-    label $w.messagename.l1 -text "password string" 
-    entry $w.messagename.e1 -relief sunken -textvariable passstring
-    frame $w.specmessage
-    label $w.specmessage.l2 -text "info string" 
-    entry $w.specmessage.e2 -relief sunken -textvariable infostring
-    pack $w.messagename $w.specmessage 
-    pack $w.messagename.l1 -side left 
-    pack $w.specmessage.l2 -side left 
-    pack $w.messagename.e1 $w.specmessage.e2  -side top -pady 5 -fill x
-    button .message.frame.ok -text OK -command "destroy .message"
-    button .message.frame.cancel -text Cancel -command "destroy .message"
-    pack $w .message.frame.ok .message.frame.cancel -side left -fill x
-	pack .message.frame2  -side bottom
-
+	mkDialog message {Message Options} \
+	{"message password" passmot} \
+	{"validate string" validstr} \
+	{"invalid string" invalidstr} \
+	{"info string" infostring} 
 }
 #no influence with xlock
 proc mkGeometry {} {
 	global geometrie
 	global icogeometrie
-     toplevel .geometry
-	wm title .geometry "Geometry  Options"
-    frame .geometry.frame -borderwidth 10
-    frame .geometry.frame2 -borderwidth 10
-    set w .geometry.frame
-
-    frame $w.geometryname
-    label $w.geometryname.l1 -text "geometry" 
-    entry $w.geometryname.e1 -relief sunken -textvariable geometrie
-    frame $w.specgeometry
-    label $w.specgeometry.l2 -text "icon geometry" 
-    entry $w.specgeometry.e2 -relief sunken -textvariable icogeometrie
-    pack $w.geometryname $w.specgeometry 
-    pack $w.geometryname.l1 -side left 
-    pack $w.specgeometry.l2 -side left 
-    pack $w.geometryname.e1 $w.specgeometry.e2  -side top -pady 5 -fill x
-    button .geometry.frame2.ok -text OK -command "destroy .geometry"
-    button .geometry.frame2.cancel -text Cancel -command "destroy .geometry"
-    pack $w .geometry.frame2.ok .geometry.frame2.cancel -side left -fill x 
-	pack .geometry.frame2 -side bottom
+	mkDialog geometry {Geometry Options} \
+	{"geometry" geometrie} \
+	{"icon geometry" icogeometrie}
 }
 
 proc whichfont { which } {
@@ -278,10 +279,12 @@ global mftname
 proc mkFont { What } {
      toplevel .font 
 	wm title .font "Font Options"
-	label  .font.label -text "ABCDEFGHIJKabedfghijkmnopq" 
+	label  .font.label -text "ABCDEFGH\nIJKabedfg\nhijkmnopq" 
     frame .font.frame -borderwidth 10 
     frame .font.frame2 -borderwidth 10
     set w .font.frame
+    label $w.msg0   -text "Font Options" 
+    pack $w.msg0 -side top
     eval exec "xlsfonts \> /tmp/xlsfont.tmp"
     set f [open "/tmp/xlsfont.tmp"]
     listbox .font.frame.names -yscrollcommand ".font.frame.scroll set" \
@@ -310,7 +313,7 @@ proc mkFont { What } {
     button .font.frame2.cancel -text Cancel -command "destroy .font"
     button .font.frame2.reset -text Reset -command "whichfont RESET$What;destroy .font"
     button .font.frame2.ok -text OK -command "whichfont $What;destroy .font"
-	pack .font.frame2.ok .font.frame2.cancel .font.frame2.reset -side left -fill x
+	pack .font.frame2.ok .font.frame2.cancel .font.frame2.reset -side left -fill both
 	pack .font.frame2 -fill both
 
     #frame $w.fontname
@@ -329,36 +332,12 @@ proc mkFont { What } {
 	#pack .font.frame2 -side bottom
 }
 
-#no influence with xlock
 proc mkEntry {} {
 global usernom
-global passmot
 global prgname
-     toplevel .option
-	wm title .option "Entry options"
-    frame .option.frame -borderwidth 10
-    frame .option.frame2 -borderwidth 10
-    set w .option.frame
-
-    frame $w.username
-    label $w.username.l1 -text "user name" 
-    entry $w.username.e1 -relief sunken  -textvariable usernom
-    frame $w.password
-    label $w.password.l2 -text "password" 
-    entry $w.password.e2 -relief sunken -textvariable passmot
-    frame $w.info
-    label $w.info.l3 -text "program name" 
-    entry $w.info.e3 -relief sunken -textvariable prgname
-    pack $w.username $w.password $w.info 
-    pack $w.username.l1 -side left 
-    pack $w.password.l2 -side left 
-    pack $w.info.l3 -side left 
-    pack $w.username.e1 $w.password.e2 $w.info.e3 -side top -pady 5 -fill x
-    button .option.frame2.ok -text OK -command "destroy .option"
-    button .option.frame2.cancel -text Cancel -command "destroy .option"
-    pack $w .option.frame2.ok .option.frame2.cancel -side left -fill x 
-
-	pack .option.frame2 -side bottom
+	mkDialog option {User Options} \
+	{"user name" usernom} \
+	{"program name" prgname}
 }
 
 proc Affopts { device } {
@@ -373,8 +352,8 @@ global allowaccess
 global grabmouse
 global echokeys
 global usefirst
-
-
+global install
+global sound
 
 global fgcolor
 global bgcolor
@@ -383,10 +362,11 @@ global mftname
 
 global usernom 
 global passmot 
+global validstr 
+global invalidstr 
 global prgname
 global geometrie 
 global icogeometrie 
-global passstring 
 global infostring 
 
 set linecommand "xlock "
@@ -396,14 +376,19 @@ if {$bgcolor!=""} {append linecommand "-bg $bgcolor "}
 if {$fgcolor!=""} {append linecommand "-fg $fgcolor "}
 if {$ftname!=""} {append linecommand "-font $ftname "}
 if {$mftname!=""} {append linecommand "-mfont $mftname "}
+#entry action
 if {$usernom!=""} {append linecommand "-username $usernom "}
 if {$passmot!=""} {append linecommand "-password $passmot "}
+if {$validstr!=""} {append linecommand "-validate $validstr "}
+if {$invalidstr!=""} {append linecommand "-invalid $invalidstr "}
 if {$prgname!=""} {append linecommand "-program $prgname "}
 if {$geometrie!=""} {append linecommand "-geometry $geometrie "}
 if {$icogeometrie!=""} {append linecommand "-icongeometry $icogeometrie "}
-if {$passstring!=""} {append linecommand "-password $passstring "}
 if {$infostring!=""} {append linecommand "-info $infostring "}
+#check actions
 if { $mono == 1 } {append linecommand "-mono "}
+if { $install == 1 } {append linecommand "+install "}
+if { $sound == 1 } {append linecommand "+sound "}
 if { $nolock == 1 } {append linecommand "-nolock "}
 if { $remote == 1 } {append linecommand "-remote "}
 if { $allowroot == 1 } {append linecommand "-allowroot "}
@@ -427,6 +412,8 @@ menubutton .menu.button -text "switches" -menu .menu.button.check
 pack .menu -side top -fill x 
 
 global mono
+global sound
+global install
 global nolock
 global remote
 global allowroot
@@ -440,7 +427,6 @@ global usernom
 global passmot
 global geometrie
 global icogeometrie
-global passstring 
 global infostring 
 
 # Creation of GUI 
@@ -460,6 +446,8 @@ $CHECK add check -label "allowaccess" -variable allowaccess
 $CHECK add check -label "grabmouse" -variable grabmouse 
 $CHECK add check -label "echokeys" -variable echokeys
 $CHECK add check -label "usefirst" -variable usefirst
+$CHECK add check -label "install" -variable install
+$CHECK add check -label "sound" -variable sound
 
 menubutton .menu.button2 -text "options" -menu .menu.button2.options
 menu .menu.button2.options
@@ -500,20 +488,20 @@ pack $LISTSCROL -fill both
 
 $LISTSCROL.list  insert 0 \
 ant ball bat blot \
-bouboule bounce braid bug \
-cartoon clock coral crystal \
-daisy dclock deco demon dilemma drift escher eyes \
+bouboule bounce braid bubble bug \
+cage cartoon clock coral crystal \
+daisy dclock deco demon dilemma drift eyes \
 fadeplot flag flame forest \
 galaxy gears geometry grav \
 helix hop hyper \
 ico ifs image julia kaleid \
 laser life life1d life3d \
 lightning lisa lissie loop \
-mandelbrot marquee maze morph3d mountain munch nose \
+mandelbrot marquee maze moebius morph3d mountain munch nose \
 pacman penrose petal pipes puzzle pyro \
 qix roll rotor rubik \
 shape sierpinski slip \
-sphere spiral spline sproingies \
+sphere spiral spline sproingies stairs \
 star strange superquadrics swarm swirl \
 triangle tube turtle vines voters \
 wator wire world worm \
