@@ -1,4 +1,4 @@
-// charles vidal <cvidal@newlog.com>
+// charles vidal <vidalc@club-internet.fr>
 // the GUI definition and the handler event
 import java.applet.*;
 import java.awt.*;
@@ -79,7 +79,9 @@ lst.addItem("blot");
 lst.addItem("bouboule");
 lst.addItem("bounce");
 lst.addItem("braid");
+lst.addItem("bubble");
 lst.addItem("bug");
+lst.addItem("cage");
 lst.addItem("cartoon");
 lst.addItem("clock");
 lst.addItem("coral");
@@ -90,7 +92,6 @@ lst.addItem("deco");
 lst.addItem("demon");
 lst.addItem("dilemma");
 lst.addItem("drift");
-lst.addItem("escher");
 lst.addItem("eyes");
 lst.addItem("fadeplot");
 lst.addItem("flag");
@@ -118,6 +119,7 @@ lst.addItem("loop");
 lst.addItem("marquee");
 lst.addItem("mandelbrot");
 lst.addItem("maze");
+lst.addItem("moebius");
 lst.addItem("morph3d");
 lst.addItem("mountain");
 lst.addItem("munch");
@@ -139,6 +141,7 @@ lst.addItem("sphere");
 lst.addItem("spiral");
 lst.addItem("spline");
 lst.addItem("sproingies");
+lst.addItem("stairs");
 lst.addItem("star");
 lst.addItem("strange");
 lst.addItem("superquadrics");
