@@ -18,6 +18,7 @@ XCOMM  #define GLLibrary
 XCOMM  #define DtSaverLibrary
 XCOMM  #define RplayLibrary
 XCOMM  #define NasLibrary
+XCOMM  #define Modules
 XCOMM  #define Check
 
 N =
@@ -139,6 +140,10 @@ XCOMM On other OpenGL
 XCOMM  GLLIB = -lGL -lGLU
 XCOMM On Sun with OGL1.1
 XCOMM  GLDEF += -DSUN_OGL_NO_VERTEX_MACROS
+XCOMM 3Dfx card
+XCOMM  GLDEF += -DFX
+XCOMM Threads
+XCOMM  GLLIB += -lpthread
 #endif
 
 XCOMM   *** END MESAGL AND OPENGL CONFIG SECTION ***
@@ -215,12 +220,48 @@ RANDDEF = $(SRANDDEF) $(LRANDDEF) $(MAXRANDDEF)
 
 XCOMM      *** END RNG CONFIG SECTION ***
 
+XCOMM    *** BEGIN MODULES CONFIG SECTION **
+
+XCOMM This is a HIGHLY experimental section.
+XCOMM To use modules, you must have an ELF setup installed, and
+XCOMM libdl.  This was originally written for Linux.
+XCOMM /usr/include/dlfcn.h is no longer found there at least on 2.0.0
+XCOMM Anybody have any ideas?  So I will try to get this running on
+XCOMM Solaris-2.6 with gcc and gmake first.  I don't know if
+XCOMM other Unices support this kind of dynamic loading.
+XCOMM Un-commenting Modules will compile xlock so that all of the modes
+XCOMM are in separate modules which are loaded at runtime.  This means
+XCOMM you can add new modes simply by putting them in your modules
+XCOMM directory.
+XCOMM  #define Modules
+
+#ifdef Modules
+
+XCOMM #define UseElfFormat YES
+
+XCOMM This is the default directory to look in for modules. It can be
+XCOMM a colon separated list of directories.
+DEF_MODULEPATH = $(USRLIBDIR)/X11/xlock_modules
+
+XCOMM Flag to compile the main program so that global variables will be
+XCOMM available to modules
+MODULEFLAGS = -rdynamic
+
+MODULEDEF = -DDEF_MODULEPATH=\"$(DEF_MODULEPATH)\" -DUSE_MODULES
+
+XCOMM Library necessary to use dynamic loading
+MODULELIB = -ldl
+
+#endif
+
+XCOMM    *** END MODULES CONFIG SECTION ***
+
 XCOMM      *** BEGIN DEBUG CHECK SECTION ***
  
 XCOMM #define Check
  
 #ifdef Check
-XCOMM Very esperimental
+XCOMM Very experimental
 CHECKDEF = -DDEBUG
 #endif
  
@@ -251,6 +292,10 @@ XCOMM  PASSWDDEF = -DDCE_PASSWD
 XCOMM  PASSWDINC = -I/usr/include/reentrant
 XCOMM  PASSWDLIB = -ldce -lc_r
 
+XCOMM PAM (tested only on Linux)
+XCOMM  PASSWDDEF = -DUSE_PAM
+XCOMM  PASSWDLIB = -lpam -ldl
+
 #ifdef SVR4ix86Architecture
 PASSWDDEF = -DHAVE_SHADOW
 #endif
@@ -367,30 +412,30 @@ PIXMAPTYPE = x11
 
 XLOCKINC = -I$(top_srcdir) -I. -I$(UTILSDIR)
 
-XCOMM -DUSE_VROOT          Allows xlock to run in root window (some
-XCOMM                      window managers have problems)
-XCOMM -DALWAYS_ALLOW_ROOT  Users can not turn off allowroot
-XCOMM -DUSE_SYSLOG         Paranoid administrator option (a check is also
-XCOMM                      done to see if you have it)
-XCOMM -DUSE_MULTIPLE_ROOT  Multiple root users ... security?
-XCOMM -DUSE_MOUSE_MOTION   Password screen displayed with mouse motion
-XCOMM -DUSE_OLD_EVENT_LOOP Some machines may still need this (fd_set
-XCOMM                      errors may be a sign)
-XCOMM -DUSE_VMSUTILS       This patches up old __VMS_VER < 70000000
+XCOMM -DUSE_VROOT              Allows xlock to run in root window (some
+XCOMM                          window managers have problems)
+XCOMM -DALWAYS_ALLOW_ROOT      Users can not turn off allowroot
+XCOMM -DUSE_SYSLOG             Paranoid administrator option (a check is also
+XCOMM                          done to see if you have it)
+XCOMM -DUSE_MULTIPLE_ROOT      Multiple root users ... security?
+XCOMM -DUSE_WINDOW_VISIBILITY  Allow a window to be placed over xlock
+XCOMM -DUSE_OLD_EVENT_LOOP     Some machines may still need this (fd_set
+XCOMM                          errors may be a sign)
+XCOMM -DUSE_VMSUTILS           This patches up old __VMS_VER < 70000000
 XCOMM
 XCOMM For personal use you may want to consider:
-XCOMM -DUSE_XLOCKRC        paranoid admin or unknown shadow passwd alg
+XCOMM -DUSE_XLOCKRC            paranoid admin or unknown shadow passwd alg
 XCOMM
 XCOMM For labs you may want to consider:
-XCOMM -DUSE_AUTO_LOGOUT=240  Enable auto-logout and set deadline (minutes)
+XCOMM -DUSE_AUTO_LOGOUT=240     Enable auto-logout and set deadline (minutes)
 XCOMM -DDEF_AUTO_LOGOUT=\"120\" Set default auto-logout deadline (minutes)
-XCOMM -DUSE_BUTTON_LOGOUT=10 Enable logout button and set when to appear (min) 
-XCOMM -DDEF_BUTTON_LOGOUT=\"5\" Set default logout button (min) 
-XCOMM -DUSE_BOMB           Enable automatic logout mode (does not come up
-XCOMM                      in random mode)
-XCOMM -DCLOSEDOWN_LOGOUT   Use with USE_AUTO_LOGOUT, USE_BUTTON_LOGOUT,
-XCOMM                      USE_BOMB if using xdm
-XCOMM -DSESSION_LOGOUT     Alternate of above
+XCOMM -DUSE_BUTTON_LOGOUT=10    Enable logout button and set appear time (min) 
+XCOMM -DDEF_BUTTON_LOGOUT=\"5\" Set default logout button (minutes) 
+XCOMM -DUSE_BOMB                Enable automatic logout mode (does not come up
+XCOMM                           in random mode)
+XCOMM -DCLOSEDOWN_LOGOUT        Use with USE_AUTO_LOGOUT, USE_BUTTON_LOGOUT,
+XCOMM                           USE_BOMB if using xdm
+XCOMM -DSESSION_LOGOUT          Alternate of above
 XCOMM -DSTAFF_FILE=\"/etc/xlock.staff\"  File of staff who are exempt
 XCOMM -DSTAFF_NETGROUP=\"/etc/xlock.netgroup\"  Netgroup that is exempt
 
@@ -399,7 +444,6 @@ OPTDEF = -DUSE_VROOT -DALWAYS_ALLOW_ROOT -DUSE_BOMB
 XCOMM  OPTDEF += -DUSE_SYSLOG -DSYSLOG_FACILITY=LOG_AUTH
 XCOMM  OPTDEF += -DSYSLOG_WARNING=LOG_WARNING
 XCOMM  OPTDEF += -DSYSLOG_NOTICE=LOG_NOTICE -DSYSLOG_INFO=LOG_INFO
-XCOMM  OPTDEF += -DUSE_MOUSE_MOTION
 XCOMM  OPTDEF += -DUSE_MULTIPLE_ROOT
 XCOMM  OPTDEF += -DUSE_OLD_EVENT_LOOP
 XCOMM  OPTDEF += -DUSE_XLOCKRC
@@ -413,13 +457,14 @@ XCOMM  OPTDEF += -DSTAFF_FILE=\"/etc/xlock.staff\"
 XCOMM  OPTDEF += -DSTAFF_NETGROUP=\"/etc/xlock.netgroup\"
 
 DEFINES = -DDEF_FILESEARCHPATH=\"$(LIBDIR)/%T/%N%S\" \
-$(SYSTEMDEF) $(EDITRESDEF) $(SLEEPDEF) $(OPTDEF) $(RANDDEF) \
+$(SYSTEMDEF) $(EDITRESDEF) $(SLEEPDEF) $(OPTDEF) $(RANDDEF) $(MODULEDEF) \
 $(PASSWDDEF) $(XMINC) $(XAWINC) $(XPMDEF) $(GLDEF) $(DTSAVERDEF) $(SOUNDDEF) \
 $(PASSWDINC) $(XPMINC) $(GLINC) $(DTSAVERINC) $(SOUNDINC) $(XLOCKINC)
 
 DEPLIBS = $(DEPXLIB)
-LOCAL_LIBRARIES = $(XLIB) $(XPMLIB) $(GLLIB) $(DTSAVERLIB) $(SOUNDLIB)
-MLIBS = $(XMLIB) $(EDITRESLIB) -lXt $(XLIB) $(SMLIB) $(ICELIB)
+LOCAL_LIBRARIES = $(MODULELIB) $(XLIB) \
+$(XPMLIB) $(GLLIB) $(DTSAVERLIB) $(SOUNDLIB)
+MLIBS = $(XPMLIB) $(XMLIB) $(EDITRESLIB) -lXt $(XLIB) $(SMLIB) $(ICELIB)
 ALIBS = $(XAWLIB) -lXt $(XLIB) $(SMLIB) $(ICELIB)
 LINTLIBS = $(LINTXLIB)
 #if HasLibCrypt
