@@ -2,7 +2,7 @@
 /* bat --- bouncing bats */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)bat.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)bat.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -69,12 +69,20 @@ static const char sccsid[] = "@(#)bat.c	4.04 97/07/28 xlockmore";
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
-
 #endif /* STANDALONE */
 
 ModeSpecOpt bat_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   bat_description =
+{"bat", "init_bat", "draw_bat", "release_bat",
+ "refresh_bat", "init_bat", NULL, &bat_opts,
+ 100000, -8, 1, 0, 64, 1.0, "",
+ "Shows bouncing flying bats", 0, NULL};
+
+#endif
+
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 #if USE_XPMINC
 #include <xpm.h>
@@ -137,7 +145,7 @@ typedef struct {
 	int         pixelmode;
 	GC          backGC;
 	Colormap    cmap;
-	unsigned long black, white;
+	unsigned long black;
 	batstruct  *bats;
 	XImage     *images[ORIENTS / 2 + 1];
 } bouncestruct;
@@ -231,7 +239,7 @@ drawabat(ModeInfo * mi, batstruct * bat)
 		XSetForeground(display, bp->backGC, bat->color);
 /* PURIFY 4.0.1 on SunOS4 and on Solaris 2 reports a 15985 byte memory leak on
    * the next line. */
-		XPutImage(display, window, bp->backGC,
+		(void) XPutImage(display, window, bp->backGC,
 			  bp->images[(bat->orient > ORIENTS / 2) ?
 				     ORIENTS - bat->orient : bat->orient],
 			  0, 0, bat->x, bat->y, bp->xs, bp->ys);
@@ -318,20 +326,21 @@ init_stuff(ModeInfo * mi)
 	bouncestruct *bp = &bounces[MI_SCREEN(mi)];
 
 	if (!bp->cmap && !bp->pixelmode) {
-		int         i, total = 0;
+		int         i;
 
 #if defined( USE_XPM ) || defined( USE_XPMINC )
+		int         total = 0;
 		XpmAttributes attrib;
 
 		if (!fixedColors(mi)) {
 			bp->cmap = XCreateColormap(display, window, MI_VISUAL(mi), AllocNone);
 			attrib.colormap = bp->cmap;
-			reserveColors(mi, bp->cmap, &bp->black, &bp->white);
+			reserveColors(mi, bp->cmap, &bp->black);
 		} else
-			attrib.colormap = MI_WIN_COLORMAP(mi);
+			attrib.colormap = MI_COLORMAP(mi);
 
 		attrib.visual = MI_VISUAL(mi);
-		attrib.depth = MI_WIN_DEPTH(mi);
+		attrib.depth = MI_DEPTH(mi);
 		attrib.valuemask = XpmVisual | XpmColormap | XpmDepth;
 
 		if (bp->graphics_format == IS_NONE
@@ -350,7 +359,7 @@ init_stuff(ModeInfo * mi)
 				if (MI_WIN_IS_VERBOSE(mi))
 					(void) fprintf(stderr, "Full color images could not be loaded.\n");
 				for (i = 0; i < total; i++) {
-					XDestroyImage(bp->images[i]);
+					(void) XDestroyImage(bp->images[i]);
 					bp->images[i] = None;
 				}
 				bp->images[total] = None;
@@ -385,7 +394,7 @@ init_stuff(ModeInfo * mi)
 			bp->backGC = XCreateGC(display, window, GCBackground, &xgcv);
 		}
 	} else {
-		bp->black = MI_WIN_BLACK_PIXEL(mi);
+		bp->black = MI_BLACK_PIXEL(mi);
 		bp->backGC = MI_GC(mi);
 	}
 
@@ -400,7 +409,7 @@ free_stuff(Display * display, bouncestruct * bp)
 	if (bp->graphics_format == IS_XPM) {
 		for (i = 0; i <= ORIENTS / 2; i++) {
 			if (bp->images[i]) {
-				XDestroyImage(bp->images[i]);
+				(void) XDestroyImage(bp->images[i]);
 				bp->images[i] = None;
 			}
 		}
@@ -421,7 +430,6 @@ free_stuff(Display * display, bouncestruct * bp)
 void
 init_bat(ModeInfo * mi)
 {
-	Display    *display = MI_DISPLAY(mi);
 	bouncestruct *bp;
 	int         size = MI_SIZE(mi);
 	int         i, tryagain = 0;
@@ -434,7 +442,7 @@ init_bat(ModeInfo * mi)
 	bp = &bounces[MI_SCREEN(mi)];
 
 #if defined( USE_XPM ) || defined( USE_XPMINC )
-	free_stuff(display, bp);
+	free_stuff(MI_DISPLAY(mi), bp);
 #endif
 
 	bp->width = MI_WIN_WIDTH(mi);
@@ -494,7 +502,7 @@ init_bat(ModeInfo * mi)
 			if (MI_NPIXELS(mi) > 2)
 				bp->bats[i].color = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
 			else
-				bp->bats[i].color = MI_WIN_WHITE_PIXEL(mi);
+				bp->bats[i].color = MI_WHITE_PIXEL(mi);
 			bp->bats[i].xlast = -1;
 			bp->bats[i].ylast = 0;
 			bp->bats[i].spincount = 1;
@@ -507,9 +515,7 @@ init_bat(ModeInfo * mi)
 		} else
 			tryagain++;
 	}
-	XSetForeground(display, bp->backGC, bp->black);
-	XFillRectangle(display, MI_WINDOW(mi), bp->backGC,
-		       0, 0, bp->width, bp->height);
+	MI_CLEARWINDOWCOLORMAP(mi, bp->backGC, bp->black);
 }
 
 void
