@@ -2,7 +2,7 @@
 /* blank --- blank screen */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)blank.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)blank.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -42,18 +42,26 @@ static const char sccsid[] = "@(#)blank.c	4.04 97/07/28 xlockmore";
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
-
 #endif /* STANDALONE */
 
 ModeSpecOpt blank_opts =
 {0, NULL, 0, NULL, NULL};
 
-extern      enablesaver;
+#ifdef USE_MODULES
+ModStruct   blank_description =
+{"blank", "init_blank", "draw_blank", "release_blank",
+ "refresh_blank", "init_blank", NULL, &blank_opts,
+ 3000000, 1, 1, 1, 2, 1.0, "",
+ "Shows nothing but a black screen", 0, NULL};
+
+#endif
+
+extern Bool enablesaver;
 
 void
 init_blank(ModeInfo * mi)
 {
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 	/* Must set PreferBlanking, or XForceScreenSaver won't work */
 	if (!MI_WIN_IS_INWINDOW(mi) && !MI_WIN_IS_INROOT(mi) && enablesaver)
 		XSetScreenSaver(MI_DISPLAY(mi), 0, 0, PreferBlanking, 0);
