@@ -2,7 +2,7 @@
 /* bomb --- temporary screen */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)bomb.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)bomb.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -53,10 +53,18 @@ static const char sccsid[] = "@(#)bomb.c	4.04 97/07/28 xlockmore";
 #endif /* STANDALONE */
 
 #ifdef USE_BOMB
-
 ModeSpecOpt bomb_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   bomb_description =
+{"bomb", "init_bomb", "draw_bomb", "release_bomb",
+ "refresh_bomb", "change_bomb", NULL, &bomb_opts,
+ 100000, 10, 20, 1, 64, 1.0, "",
+ "Shows a bomb and will autologout after a time", 0, NULL};
+
+#endif
+
 #include <sys/signal.h>
 
 #if 0
@@ -96,61 +104,17 @@ static bombstruct *bombs = NULL;
 
 static XFontStruct *mode_font = None;
 
-	/*
-	 *  Game Over
-	 */
-static void
-explode(ModeInfo * mi)
-{
-	bombstruct *bp = &bombs[MI_SCREEN(mi)];
-	char        buff[NDIGITS + 2];
-	extern void logoutUser(void);
-
-#if defined(__cplusplus) || defined(c_plusplus)
-	extern int  kill(int, int);
-
-#endif
-
-	/*
-	 *  ToDo:
-	 *      Improve the graphics - some sort of explosion?
-	 *      (Will need to involve the main X event loop)
-	 */
-#ifdef SIMPLE_COUNTDOWN
-	(void) sprintf(buff, "%.*s", NDIGITS, "*********");
-#else
-	(void) sprintf(buff, "%.*s:**", NDIGITS - 2, "*******");
-#endif
-	XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_PIXEL(mi, 1));
-	XDrawString(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-		    bp->x, bp->y, buff, NDIGITS);
-
-	(void) fprintf(stderr, "BOOM!!!!\n");
-#ifndef DEBUG
-	if (MI_WIN_IS_INWINDOW(mi) || MI_WIN_IS_INROOT(mi) ||
-	    MI_WIN_IS_NOLOCK(mi) || MI_WIN_IS_DEBUG(mi))
-		(void) kill(getpid(), SIGTERM);
-	else if (getuid() == 0) {	/* Do not try to logout root! */
-		bp->countdown = 0;
-		init_bomb(mi);
-	} else
-		logoutUser();
-#else
-	(void) kill(getpid(), SIGTERM);
-#endif
-}
-
 static void
 rivet(ModeInfo * mi, int x, int y)
 {
 	Display    *display = MI_DISPLAY(mi);
 	GC          gc = MI_GC(mi);
 
-	XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+	XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	if (MI_NPIXELS(mi) > 2) {
 		XDrawArc(display, MI_WINDOW(mi), gc, x, y,
 		      2 * RIVET_RADIUS, 2 * RIVET_RADIUS, 270 * 64, 90 * 64);
-		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 		XDrawArc(display, MI_WINDOW(mi), gc, x, y,
 		      2 * RIVET_RADIUS, 2 * RIVET_RADIUS, 70 * 64, 130 * 64);
 	} else
@@ -173,8 +137,8 @@ detonator(ModeInfo * mi, int draw)
 			XSetForeground(MI_DISPLAY(mi), MI_GC(mi),
 				       MI_PIXEL(mi, bp->color));
 		else
-			XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
-		/*XSetForeground(display, gc, allocPixel(display, MI_WIN_COLORMAP(mi),
+			XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WHITE_PIXEL(mi));
+		/*XSetForeground(display, gc, allocPixel(display, MI_COLORMAP(mi),
 		   "grey", "white")); */
 		XFillRectangle(display, MI_WINDOW(mi), gc,
 			       bp->loc.x, bp->loc.y, b_width, b_height);
@@ -193,7 +157,7 @@ detonator(ModeInfo * mi, int draw)
 			      bp->loc.y + b_height - 3 * RIVET_RADIUS);
 		}
 	} else {
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_BLACK_PIXEL(mi));
 		XFillRectangle(display, MI_WINDOW(mi), gc,
 			       bp->loc.x, bp->loc.y, b_width, b_height);
 	}
@@ -229,20 +193,20 @@ init_bomb(ModeInfo * mi)
 		mode_font = XLoadQueryFont(display, ICON_COUNT_FONT);
 		bp->delta = 2;
 	}
-  if (mode_font == None)
-    mode_font = getFont(display);
-  if (mode_font != None)
-	  XSetFont(display, gc, mode_font->fid);
+	if (mode_font == None)
+		mode_font = getFont(display);
+	if (mode_font != None)
+		XSetFont(display, gc, mode_font->fid);
 
 #ifdef SIMPLE_COUNTDOWN
 	(void) sprintf(number, "%0*d", NDIGITS, 0);
 #else
 	(void) sprintf(number, "%.*s:**", NDIGITS - 2, "*******");
 #endif
-  if (mode_font != None)
-	  bp->text_width = XTextWidth(mode_font, number, NDIGITS + 1);
-  else
-	  bp->text_width = 8;
+	if (mode_font != None)
+		bp->text_width = XTextWidth(mode_font, number, NDIGITS + 1);
+	else
+		bp->text_width = 8;
 	bp->text_ascent = mode_font->max_bounds.ascent;
 	bp->text_descent = mode_font->max_bounds.descent;
 
@@ -259,7 +223,9 @@ init_bomb(ModeInfo * mi)
 	if (bp->countdown == 0)	/* <--Stricter if uncommented */
 		bp->countdown = (time((time_t *) NULL) + bp->startcountdown);
 	/* Detonator Primed */
-	XClearWindow(display, MI_WINDOW(mi));
+
+	MI_CLEARWINDOW(mi);
+
 	/*
 	 *  Draw the graphics
 	 *      Very simple - detonator box with countdown
@@ -277,6 +243,50 @@ init_bomb(ModeInfo * mi)
 	bp->moveok = 0;
 }
 
+	/*
+	 *  Game Over
+	 */
+static void
+explode(ModeInfo * mi)
+{
+	bombstruct *bp = &bombs[MI_SCREEN(mi)];
+	char        buff[NDIGITS + 2];
+	extern void logoutUser(Display * display);
+
+#if defined(__cplusplus) || defined(c_plusplus)
+	extern int  kill(int, int);
+
+#endif
+
+	/*
+	 *  ToDo:
+	 *      Improve the graphics - some sort of explosion?
+	 *      (Will need to involve the main X event loop)
+	 */
+#ifdef SIMPLE_COUNTDOWN
+	(void) sprintf(buff, "%.*s", NDIGITS, "*********");
+#else
+	(void) sprintf(buff, "%.*s:**", NDIGITS - 2, "*******");
+#endif
+	XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_PIXEL(mi, 1));
+	(void) XDrawString(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
+		    bp->x, bp->y, buff, NDIGITS);
+
+	(void) fprintf(stderr, "BOOM!!!!\n");
+#ifndef DEBUG
+	if (MI_WIN_IS_INWINDOW(mi) || MI_WIN_IS_INROOT(mi) ||
+	    MI_WIN_IS_NOLOCK(mi) || MI_WIN_IS_DEBUG(mi))
+		(void) kill(getpid(), SIGTERM);
+	else if (getuid() == 0) {	/* Do not try to logout root! */
+		bp->countdown = 0;
+		init_bomb(mi);
+	} else
+		logoutUser(MI_DISPLAY(mi));
+#else
+	(void) kill(getpid(), SIGTERM);
+#endif
+}
+
 void
 draw_bomb(ModeInfo * mi)
 {
@@ -299,7 +309,7 @@ draw_bomb(ModeInfo * mi)
 #endif
 
 		/* Blank out the previous number .... */
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		XFillRectangle(display, MI_WINDOW(mi), gc,
 			       bp->x - bp->delta,
 			       (bp->y - bp->text_ascent) - bp->delta,
@@ -308,7 +318,7 @@ draw_bomb(ModeInfo * mi)
 
 		/* ... and count down */
 		if (MI_NPIXELS(mi) <= 2)
-			crayon = MI_WIN_WHITE_PIXEL(mi);
+			crayon = MI_WHITE_PIXEL(mi);
 		else
 #ifdef COLOUR_CHANGE		/* Blue to red */
 			crayon = MI_PIXEL(mi, countleft * MI_NPIXELS(mi) / bp->startcountdown);
@@ -331,7 +341,7 @@ draw_bomb(ModeInfo * mi)
 			bp->moveok = 1;
 		}
 		XSetForeground(display, gc, crayon);
-		XDrawString(display, MI_WINDOW(mi), gc, bp->x, bp->y,
+		(void) XDrawString(display, MI_WINDOW(mi), gc, bp->x, bp->y,
 			    number, strlen(number));
 	}
 }
