@@ -5,7 +5,7 @@
  */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)braid.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)braid.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -49,6 +49,15 @@ static const char sccsid[] = "@(#)braid.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt braid_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   braid_description =
+{"braid", "init_braid", "draw_braid", "release_braid",
+ "refresh_braid", "init_braid", NULL, &braid_opts,
+ 1000, 15, 100, 1, 64, 1.0, "",
+ "Shows random braids and knots", 0, NULL};
+
+#endif
+
 #if defined( COLORROUND ) && defined( COLORCOMP )
 #undef COLORROUND
 #undef COLORCOMP
@@ -149,7 +158,6 @@ applywordbackto(braidtype * braid, int string, int position)
 void
 init_braid(ModeInfo * mi)
 {
-	Display    *display = MI_DISPLAY(mi);
 	braidtype  *braid;
 	int         used[MAXSTRANDS];
 	int         i, count, comp, c;
@@ -169,7 +177,7 @@ init_braid(ModeInfo * mi)
 	/* jwz: go in the other direction sometimes. */
 	braid->color_direction = ((LRAND() & 1) ? 1 : -1);
 
-	XClearWindow(display, MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 
 	min_length = (braid->center_x > braid->center_y) ?
 		braid->center_y : braid->center_x;
@@ -251,20 +259,20 @@ draw_braid(ModeInfo * mi)
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	braidtype  *braid = &braids[MI_SCREEN(mi)];
-	float       num_points, t_inc;
+	int         num_points = 500;
+	float       t_inc;
 	float       theta, psi;
 	float       t, r_diff;
 	int         i, s;
 	float       x_1, y_1, x_2, y_2, r1, r2;
 	float       color, color_use = 0.0, color_inc;
 
-	num_points = 500.0;
 	theta = (2.0 * M_PI) / (float) (braid->braidlength);
-	t_inc = (2.0 * M_PI) / num_points;
-	color_inc = (float) MI_NPIXELS(mi) / num_points;
-	color_inc *= braid->color_direction;
+	t_inc = (2.0 * M_PI) / (float) num_points;
+	color_inc = (float) MI_NPIXELS(mi) * braid->color_direction /
+    (float) num_points;
 	braid->startcolor += SPINRATE * color_inc;
-	if (braid->startcolor >= MI_NPIXELS(mi))
+	if (((int) braid->startcolor) >= MI_NPIXELS(mi))
 		braid->startcolor = 0.0;
 
 	r_diff = (braid->max_radius - braid->min_radius) / (float) (braid->nstrands);
@@ -276,7 +284,7 @@ draw_braid(ModeInfo * mi)
 		for (t = 0.0; t < theta; t += t_inc) {
 #ifdef COLORROUND
 			color += color_inc;
-			if (color >= (float) (MI_NPIXELS(mi)))
+			if (((int) color) >= MI_NPIXELS(mi))
 				color = 0.0;
 			color_use = color;
 #endif
@@ -290,16 +298,16 @@ draw_braid(ModeInfo * mi)
 						color_use = color + SPINRATE *
 							braid->components[applywordbackto(braid, s, i)] +
 							(psi + t) / 2.0 / M_PI * (float) MI_NPIXELS(mi);
-						while (color_use >= (float) MI_NPIXELS(mi))
+						while (((int) color_use) >= MI_NPIXELS(mi))
 							color_use -= (float) MI_NPIXELS(mi);
-						while (color_use < 0.0)
+						while (((int) color_use) < 0)
 							color_use += (float) MI_NPIXELS(mi);
 					}
 #endif
 #ifdef COLORROUND
 					if (MI_NPIXELS(mi) > 2) {
 						color_use += SPINRATE * color_inc;
-						while (color_use >= (float) (MI_NPIXELS(mi)))
+						while (((int) color_use) >= MI_NPIXELS(mi))
 							color_use -= (float) MI_NPIXELS(mi);
 					}
 #endif
@@ -322,7 +330,7 @@ draw_braid(ModeInfo * mi)
 						if (MI_NPIXELS(mi) > 2)
 							XSetForeground(display, MI_GC(mi), MI_PIXEL(mi, (int) color_use));
 						else
-							XSetForeground(display, MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
+							XSetForeground(display, MI_GC(mi), MI_WHITE_PIXEL(mi));
 
 						XDrawLine(display, window, MI_GC(mi),
 							  (int) (x_1), (int) (y_1), (int) (x_2), (int) (y_2));
@@ -332,9 +340,9 @@ draw_braid(ModeInfo * mi)
 						color_use = color + SPINRATE *
 							braid->components[applywordbackto(braid, s + 1, i)] +
 							(psi + t) / 2.0 / M_PI * (float) MI_NPIXELS(mi);
-						while (color_use >= (float) MI_NPIXELS(mi))
+						while (((int) color_use) >= MI_NPIXELS(mi))
 							color_use -= (float) MI_NPIXELS(mi);
-						while (color_use < 0.0)
+						while (((int) color_use) < 0)
 							color_use += (float) MI_NPIXELS(mi);
 					}
 #endif
@@ -355,7 +363,7 @@ draw_braid(ModeInfo * mi)
 						if (MI_NPIXELS(mi) > 2)
 							XSetForeground(display, MI_GC(mi), MI_PIXEL(mi, (int) color_use));
 						else
-							XSetForeground(display, MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
+							XSetForeground(display, MI_GC(mi), MI_WHITE_PIXEL(mi));
 
 						XDrawLine(display, window, MI_GC(mi),
 							  (int) (x_1), (int) (y_1), (int) (x_2), (int) (y_2));
@@ -367,16 +375,16 @@ draw_braid(ModeInfo * mi)
 						color_use = color + SPINRATE *
 							braid->components[applywordbackto(braid, s, i)] +
 							(psi + t) / 2.0 / M_PI * (float) MI_NPIXELS(mi);
-						while (color_use >= (float) MI_NPIXELS(mi))
+						while (((int) color_use) >= MI_NPIXELS(mi))
 							color_use -= (float) MI_NPIXELS(mi);
-						while (color_use < 0.0)
+						while (((int) color_use) < 0)
 							color_use += (float) MI_NPIXELS(mi);
 					}
 #endif
 #ifdef COLORROUND
 					if (MI_NPIXELS(mi) > 2) {
 						color_use += SPINRATE * color_inc;
-						while (color_use >= (float) MI_NPIXELS(mi))
+						while (((int) color_use) >= MI_NPIXELS(mi))
 							color_use -= (float) MI_NPIXELS(mi);
 					}
 #endif
@@ -388,7 +396,7 @@ draw_braid(ModeInfo * mi)
 					if (MI_NPIXELS(mi) > 2)
 						XSetForeground(display, MI_GC(mi), MI_PIXEL(mi, (int) color_use));
 					else
-						XSetForeground(display, MI_GC(mi), MI_WIN_WHITE_PIXEL(mi));
+						XSetForeground(display, MI_GC(mi), MI_WHITE_PIXEL(mi));
 
 					XDrawLine(display, window, MI_GC(mi),
 						  (int) (x_1), (int) (y_1), (int) (x_2), (int) (y_2));
