@@ -2,7 +2,7 @@
 /* deco --- art as ugly as sin */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)deco.c 4.06 97/10/29 xlockmore";
+static const char sccsid[] = "@(#)deco.c 4.07 97/11/24 xlockmore";
 
 #endif
 /* 
@@ -59,6 +59,15 @@ static const char sccsid[] = "@(#)deco.c 4.06 97/10/29 xlockmore";
 ModeSpecOpt deco_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   deco_description =
+{"deco", "init_deco", "draw_deco", "release_deco",
+ "init_deco", "init_deco", NULL, &deco_opts,
+ 1000000, -30, 2, -10, 64, 0.8, "",
+ "Shows art as ugly as sin", 0, NULL};
+
+#endif
+
 #define MINSIZE 2
 #define MINDEPTH 1
 
@@ -81,24 +90,24 @@ deco(ModeInfo * mi, int x, int y, int w, int h, int depth)
 	GC          gc = MI_GC(mi);
 	decostruct *dp = &decos[MI_SCREEN(mi)];
 
-	if ((NRAND(dp->max_depth) < depth) ||
+	if ((NRAND(dp->max_depth) + 1 < depth) ||
 	    (w < dp->min_width) || (h < dp->min_height)) {
-		if (MI_NPIXELS(mi) > 2) {
-			if (++dp->colorindex >= MI_NPIXELS(mi))
-				dp->colorindex = 0;
-			XSetForeground(display, gc, MI_PIXEL(mi, dp->colorindex));
-		} else
-			XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
-		XFillRectangle(display, window, gc, x, y, w, h);
-		XSetForeground(display, gc, dp->bordercolor);
-		XDrawRectangle(display, window, gc, x, y, w - 1, h - 1);
+		if (w > 2 && h > 2) {
+			if (MI_NPIXELS(mi) > 2) {
+				XSetForeground(display, gc, MI_PIXEL(mi, dp->colorindex));
+				if (++dp->colorindex >= MI_NPIXELS(mi))
+					dp->colorindex = 0;
+			} else
+				XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
+			XFillRectangle(display, window, gc, x + 1, y + 1, w - 2, h - 2);
+		}
 	} else {
 		if (LRAND() & 1) {
 			deco(mi, x, y, w / 2, h, depth + 1);
-			deco(mi, x + w / 2, y, w / 2, h, depth + 1);
+			deco(mi, x + w / 2, y, w - w / 2, h, depth + 1);
 		} else {
 			deco(mi, x, y, w, h / 2, depth + 1);
-			deco(mi, x, y + h / 2, w, h / 2, depth + 1);
+			deco(mi, x, y + h / 2, w, h - h / 2, depth + 1);
 		}
 	}
 }
@@ -117,11 +126,11 @@ init_deco(ModeInfo * mi)
 	}
 	dp = &decos[MI_SCREEN(mi)];
 
-	if (MI_NPIXELS(mi) > 2)
+	if (MI_NPIXELS(mi) > 2) {
 		dp->bordercolor = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
-	else
-		dp->bordercolor = MI_WIN_BLACK_PIXEL(mi);
-	dp->colorindex = NRAND(MI_NPIXELS(mi));
+		dp->colorindex = NRAND(MI_NPIXELS(mi));
+	} else
+		dp->bordercolor = MI_BLACK_PIXEL(mi);
 	if (depth < -MINDEPTH)
 		dp->max_depth = NRAND(-depth - MINDEPTH + 1) + MINDEPTH;
 	else if (depth < MINDEPTH)
@@ -141,17 +150,11 @@ init_deco(ModeInfo * mi)
 void
 draw_deco(ModeInfo * mi)
 {
-	Display    *display = MI_DISPLAY(mi);
-	Window      window = MI_WINDOW(mi);
-	GC          gc = MI_GC(mi);
 	decostruct *dp = &decos[MI_SCREEN(mi)];
 
 	if (dp->time == 0) {
 		/* This fills up holes */
-		XSetForeground(display, gc, dp->bordercolor);
-		XFillRectangle(display, window, gc, 0, 0,
-			       MI_WIN_WIDTH(mi), MI_WIN_HEIGHT(mi));
-
+		MI_CLEARWINDOWCOLOR(mi, dp->bordercolor);
 		deco(mi, 0, 0, MI_WIN_WIDTH(mi), MI_WIN_HEIGHT(mi), 0);
 	}
 	if (++dp->time > MI_CYCLES(mi))
