@@ -2,7 +2,7 @@
 /* dilemma --- Lloyd's Prisoner's Dilemma Simulation */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)dilemma.c	4.06 97/10/20 xlockmore";
+static const char sccsid[] = "@(#)dilemma.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -66,11 +66,11 @@ static const char sccsid[] = "@(#)dilemma.c	4.06 97/10/20 xlockmore";
 #define HACK_INIT init_dilemma
 #define HACK_DRAW draw_dilemma
 #define dilemma_opts xlockmore_opts
-#define DEFAULTS "*delay: 100000 \n" \
+#define DEFAULTS "*delay: 200000 \n" \
  "*batchcount: -2 \n" \
  "*cycles: 1000 \n" \
  "*size: 0 \n" \
- "*ncolors: 200 \n"
+ "*ncolors: 6 \n"
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
@@ -100,7 +100,16 @@ static OptionStruct desc[] =
 };
 
 ModeSpecOpt dilemma_opts =
-{3, opts, 2, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   dilemma_description =
+{"dilemma", "init_dilemma", "draw_dilemma", "release_dilemma",
+ "refresh_dilemma", "init_dilemma", NULL, &dilemma_opts,
+ 200000, -2, 1000, 0, 6, 64, 1.0, "",
+ "Shows Lloyd's Prisoner's Dilemma simulation", 0, NULL};
+
+#endif
 
 extern int  neighbors;
 
@@ -111,6 +120,10 @@ extern int  neighbors;
 #define DEFECTOR 0
 #define COOPERATOR 1
 #define COLORS 4
+#define BLUE (45 * MI_NPIXELS(mi) / 64)	/* COOPERATING, was cooperating */
+#define GREEN (23 * MI_NPIXELS(mi) / 64) /* COOPERATING, was defecting */
+#define YELLOW (MI_NPIXELS(mi) / 6) /* DEFECTING, was cooperating */
+#define RED	0	/* DEFECTING, was defecting */
 #define MINDEFECT 1
 #define BITMAPS 2
 #define MINGRIDSIZE 16
@@ -152,11 +165,13 @@ typedef struct {
 	int         npositions;
 	int         width, height;
 	int         neighbors;
-	XPoint      hexagonList[6];
-	XPoint      triangleList[2][3];
+	union {
+		XPoint      hexagon[6];
+		XPoint      triangle[2][3];
+	} shape;
 } dilemmastruct;
 
-static int  initVal[NEIGHBORKINDS] =
+static char plots[NEIGHBORKINDS] =
 {
 	3, 4, 6, 8, 9, 12	/* Neighborhoods */
 };
@@ -172,32 +187,32 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 	Window      window = MI_WINDOW(mi);
 	GC          gc = MI_GC(mi);
 	dilemmastruct *dp = &dilemmas[MI_SCREEN(mi)];
-	unsigned long colour = (MI_NPIXELS(mi) > 2) ?
-	color : MI_WIN_WHITE_PIXEL(mi);
+	unsigned long colour = (MI_NPIXELS(mi) >= COLORS) ?
+	color : MI_WHITE_PIXEL(mi);
 
 	XSetForeground(display, gc, colour);
 	if (dp->neighbors == 6) {
 		int         ccol = 2 * col + !(row & 1), crow = 2 * row;
 
-		dp->hexagonList[0].x = dp->xb + ccol * dp->xs;
-		dp->hexagonList[0].y = dp->yb + crow * dp->ys;
+		dp->shape.hexagon[0].x = dp->xb + ccol * dp->xs;
+		dp->shape.hexagon[0].y = dp->yb + crow * dp->ys;
 		if (dp->xs == 1 && dp->ys == 1)
 			XFillRectangle(display, window, gc,
-			   dp->hexagonList[0].x, dp->hexagonList[0].y, 1, 1);
+			dp->shape.hexagon[0].x, dp->shape.hexagon[0].y, 1, 1);
 		else if (bitmap == BITMAPS - 1)
 			XFillPolygon(display, window, gc,
-			      dp->hexagonList, 6, Convex, CoordModePrevious);
+			    dp->shape.hexagon, 6, Convex, CoordModePrevious);
 		else {
 			if (firstChange) {
-				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 				XFillPolygon(display, window, gc,
-					     dp->hexagonList, 6, Convex, CoordModePrevious);
+					     dp->shape.hexagon, 6, Convex, CoordModePrevious);
 				XSetForeground(display, gc, colour);
 			}
 			if (dp->xs <= 6 || dp->ys <= 2)
 				XFillRectangle(display, window, gc,
-				       dp->hexagonList[0].x - 3 * dp->xs / 4,
-					       dp->hexagonList[0].y + dp->ys / 4, dp->xs, dp->ys);
+				     dp->shape.hexagon[0].x - 3 * dp->xs / 4,
+					       dp->shape.hexagon[0].y + dp->ys / 4, dp->xs, dp->ys);
 			else
 				XFillArc(display, window, gc,
 				     dp->xb + dp->xs * ccol - 3 * dp->xs / 4,
@@ -213,7 +228,7 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 					       dp->xs, dp->ys);
 			else {
 				if (firstChange) {
-					XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+					XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 					XFillRectangle(display, window, gc,
 						       dp->xb + dp->xs * col, dp->yb + dp->ys * row,
 						       dp->xs, dp->ys);
@@ -224,32 +239,32 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 					 0, 23040);
 			}
 		} else
-			XPutImage(display, window, gc,
+			(void) XPutImage(display, window, gc,
 				  &logo[bitmap], 0, 0,
 				dp->xb + dp->xs * col, dp->yb + dp->ys * row,
 				  icon_width, icon_height);
 	} else {		/* TRI */
 		int         orient = (col + row) % 2;	/* O left 1 right */
 
-		dp->triangleList[orient][0].x = dp->xb + col * dp->xs;
-		dp->triangleList[orient][0].y = dp->yb + row * dp->ys;
+		dp->shape.triangle[orient][0].x = dp->xb + col * dp->xs;
+		dp->shape.triangle[orient][0].y = dp->yb + row * dp->ys;
 		if (dp->xs <= 3 || dp->ys <= 3)
 			XFillRectangle(display, window, gc,
-			 ((orient) ? -1 : 1) + dp->triangleList[orient][0].x,
-				       dp->triangleList[orient][0].y, 1, 1);
+			((orient) ? -1 : 1) + dp->shape.triangle[orient][0].x,
+				       dp->shape.triangle[orient][0].y, 1, 1);
 		else {
 			if (orient)
-				dp->triangleList[orient][0].x += (dp->xs / 2 - 1);
+				dp->shape.triangle[orient][0].x += (dp->xs / 2 - 1);
 			else
-				dp->triangleList[orient][0].x -= (dp->xs / 2 - 1);
+				dp->shape.triangle[orient][0].x -= (dp->xs / 2 - 1);
 			if (bitmap == BITMAPS - 1)
 				XFillPolygon(display, window, gc,
-					     dp->triangleList[orient], 3, Convex, CoordModePrevious);
+					     dp->shape.triangle[orient], 3, Convex, CoordModePrevious);
 			else {
 				if (firstChange) {
-					XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+					XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 					XFillPolygon(display, window, gc,
-						     dp->triangleList[orient], 3, Convex, CoordModePrevious);
+						     dp->shape.triangle[orient], 3, Convex, CoordModePrevious);
 					XSetForeground(display, gc, colour);
 				}
 				XFillArc(display, window, gc,
@@ -613,13 +628,13 @@ init_dilemma(ModeInfo * mi)
 	dp->height = MI_WIN_HEIGHT(mi);
 
 	for (i = 0; i < NEIGHBORKINDS; i++) {
-		if (neighbors == initVal[i]) {
+		if (neighbors == plots[i]) {
 			dp->neighbors = neighbors;
 			break;
 		}
 		if (i == NEIGHBORKINDS - 1) {
 #if 0
-			dp->neighbors = initVal[NRAND(NEIGHBORKINDS)];
+			dp->neighbors = plots[NRAND(NEIGHBORKINDS)];
 			dp->neighbors = (LRAND() & 1) ? 4 : 8;
 #else
 			dp->neighbors = 8;
@@ -655,8 +670,8 @@ init_dilemma(ModeInfo * mi)
 		dp->xb = (dp->width - dp->xs * nccols) / 2 + dp->xs / 2;
 		dp->yb = (dp->height - dp->ys * (ncrows / 2) * 2) / 2 + dp->ys;
 		for (i = 0; i < 6; i++) {
-			dp->hexagonList[i].x = (dp->xs - 1) * hexagonUnit[i].x;
-			dp->hexagonList[i].y = ((dp->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
+			dp->shape.hexagon[i].x = (dp->xs - 1) * hexagonUnit[i].x;
+			dp->shape.hexagon[i].y = ((dp->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
 		}
 	} else if (dp->neighbors == 4 || dp->neighbors == 8) {
 		if (dp->width < 2)
@@ -717,9 +732,9 @@ init_dilemma(ModeInfo * mi)
 		dp->yb = (dp->height - dp->ys * dp->nrows) / 2 + dp->ys / 2;
 		for (orient = 0; orient < 2; orient++) {
 			for (i = 0; i < 3; i++) {
-				dp->triangleList[orient][i].x =
+				dp->shape.triangle[orient][i].x =
 					(dp->xs - 2) * triangleUnit[orient][i].x;
-				dp->triangleList[orient][i].y =
+				dp->shape.triangle[orient][i].y =
 					(dp->ys - 2) * triangleUnit[orient][i].y;
 			}
 		}
@@ -734,17 +749,17 @@ init_dilemma(ModeInfo * mi)
 		dp->pm[1][0] = bonus;
 	dp->pm[1][1] = 0;
 
-	if (MI_NPIXELS(mi) > 2) {
+	if (MI_NPIXELS(mi) >= COLORS) {
 
-		dp->colors[0][0] = MI_PIXEL(mi, 43);	/* BLUE, COOPERATING, was cooperating */
-		dp->colors[0][1] = MI_PIXEL(mi, 23);	/* GREEN, COOPERATING, was defecting */
-		dp->colors[1][0] = MI_PIXEL(mi, 9);	/* YELLOW, DEFECTING, was cooperating */
-		dp->colors[1][1] = MI_PIXEL(mi, 0);	/* RED, DEFECTING, was defecting */
+		dp->colors[0][0] = MI_PIXEL(mi, BLUE);	/* COOPERATING, was cooperating */
+		dp->colors[0][1] = MI_PIXEL(mi, GREEN);	/* COOPERATING, was defecting */
+		dp->colors[1][0] = MI_PIXEL(mi, YELLOW);	/* DEFECTING, was cooperating */
+		dp->colors[1][1] = MI_PIXEL(mi, RED);	/* DEFECTING, was defecting */
 	} else {
-		dp->colors[0][0] = MI_WIN_WHITE_PIXEL(mi);
-		dp->colors[0][1] = MI_WIN_WHITE_PIXEL(mi);
-		dp->colors[1][0] = MI_WIN_WHITE_PIXEL(mi);
-		dp->colors[1][1] = MI_WIN_WHITE_PIXEL(mi);
+		dp->colors[0][0] = MI_WHITE_PIXEL(mi);
+		dp->colors[0][1] = MI_WHITE_PIXEL(mi);
+		dp->colors[1][0] = MI_WHITE_PIXEL(mi);
+		dp->colors[1][1] = MI_WHITE_PIXEL(mi);
 	}
 	if (dp->s != NULL)
 		(void) free((void *) dp->s);
@@ -758,7 +773,7 @@ init_dilemma(ModeInfo * mi)
 		(void) free((void *) dp->payoff);
 	dp->payoff = (float *) calloc(dp->npositions, sizeof (float));
 
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 
 	dp->defectors = MI_BATCHCOUNT(mi);
 	if (dp->defectors < -MINDEFECT) {
