@@ -2,7 +2,7 @@
 /* eyes --- follow the bouncing Grelb */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)eyes.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)eyes.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -55,6 +55,15 @@ static const char sccsid[] = "@(#)eyes.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt eyes_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   eyes_description =
+{"eyes", "init_eyes", "draw_eyes", "release_eyes",
+ "refresh_eyes", "init_eyes", NULL, &eyes_opts,
+ 20000, -8, 5, 1, 64, 1.0, "",
+ "Shows eyes following a bouncing grelb", 0, NULL};
+
+#endif
+
 extern Bool mouse;
 
 /* definitions for the xlock version of xeyes */
@@ -150,6 +159,8 @@ typedef struct {		/* info about a pair of eyes */
 
 typedef struct {		/* per-screen info */
 	Pixmap      flypix;
+  int         flywidth, flyheight;
+  int         graphics_format;
 	GC          eyeGC;
 	GC          flyGC;
 	int         num_eyes;
@@ -422,6 +433,7 @@ eyes_overlap(Eyes * e1, Eyes * e2)
 static void
 init_fly(ModeInfo * mi, Fly * f)
 {
+	EyeScrInfo *ep = &eye_info[MI_SCREEN(mi)];
 	int         win_width = MI_WIN_WIDTH(mi);
 	int         win_height = MI_WIN_HEIGHT(mi);
 
@@ -434,8 +446,8 @@ init_fly(ModeInfo * mi, Fly * f)
 		f->width = f->height = FLY_ICON_SIZE;
 		f->vx = NRAND(4) + 1;	/* slower when iconic */
 	} else {
-		f->width = FLY_WIDTH;
-		f->height = FLY_HEIGHT;
+		f->width = ep->flywidth;
+		f->height = ep->flyheight;
 		f->vx = NRAND(15) + 1;	/* random horiz velocity */
 	}
 
@@ -449,7 +461,7 @@ init_fly(ModeInfo * mi, Fly * f)
 	f->oldx = -(f->width);	/* prevent undraw 1st time */
 
 	if (MI_NPIXELS(mi) <= 2) {
-		f->pixel = MI_WIN_WHITE_PIXEL(mi);	/* always white when mono */
+		f->pixel = MI_WHITE_PIXEL(mi);	/* always white when mono */
 	} else {
 		f->pixel = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
 	}
@@ -467,11 +479,11 @@ unpaint_fly(ModeInfo * mi, Fly * f)
 	GC          gc = MI_GC(mi);
 
 	if (MI_WIN_IS_ICONIC(mi)) {
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		XFillArc(display, window, gc, f->oldx, f->oldy,
 			 f->width, f->height, 90 * 64, 360 * 64);
 	} else {
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 #ifdef FLASH
 		XFillRectangle(display, window, gc,
 			       f->oldx, f->oldy, f->width, f->height);
@@ -544,7 +556,7 @@ move_fly(ModeInfo * mi, Fly * f)
 		int         rx, ry;
 		unsigned int m;
 
-		XQueryPointer(MI_DISPLAY(mi), MI_WINDOW(mi),
+		(void) XQueryPointer(MI_DISPLAY(mi), MI_WINDOW(mi),
 			      &r, &c, &rx, &ry, &cx, &cy, &m);
 		if (cx <= 0 || cy <= 0 ||
 		    cx >= MI_WIN_WIDTH(mi) - f->width - 1 ||
@@ -617,9 +629,9 @@ create_eyes(ModeInfo * mi, Eyes * e, Eyes * eyes, int num_eyes)
 	Window      window = MI_WINDOW(mi);
 	int         win_width = MI_WIN_WIDTH(mi);
 	int         win_height = MI_WIN_HEIGHT(mi);
-	int         win_depth = MI_WIN_DEPTH(mi);
-	unsigned long black_pixel = MI_WIN_BLACK_PIXEL(mi);
-	unsigned long white_pixel = MI_WIN_WHITE_PIXEL(mi);
+	int         win_depth = MI_DEPTH(mi);
+	unsigned long black_pixel = MI_BLACK_PIXEL(mi);
+	unsigned long white_pixel = MI_WHITE_PIXEL(mi);
 	Bool        iconic = MI_WIN_IS_ICONIC(mi);
 	Pixmap      pix = e->pixmap;	/* preserve pixmap handle */
 	int         w = e->width;	/* remember last w/h */
@@ -747,9 +759,8 @@ paint_eyes(ModeInfo * mi, Eyes * e, Fly * f, Eyes * eyes, int num_eyes)
 		/* Sorry Bud, your time is up */
 		if (e->painted) {
 			/* only unpaint it if previously painted */
-			XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
-			XClearArea(display, window, e->x, e->y,
-				   e->width, e->height, False);
+			XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
+			XFillRectangle(display, window, gc, e->x, e->y, e->width, e->height);
 		}
 		/* randomly place the eyes elsewhere */
 		create_eyes(mi, e, eyes, num_eyes);
@@ -873,8 +884,9 @@ init_eyes(ModeInfo * mi)
 #endif /* !STANDALONE */
 
 	if (ep->flypix == None) {
-		ep->flypix = XCreateBitmapFromData(display, window,
-				   (char *) FLY_BITS, FLY_WIDTH, FLY_HEIGHT);
+    getPixmap(display, window, FLY_WIDTH, FLY_HEIGHT, FLY_BITS,
+    &(ep->flywidth), &(ep->flyheight), &(ep->flypix), 
+    &(ep->graphics_format));
 		if (ep->flypix == None) {
 			return;
 		}
@@ -882,8 +894,8 @@ init_eyes(ModeInfo * mi)
 	if (ep->flyGC == None) {
 		XGCValues   gcv;
 
-		gcv.foreground = MI_WIN_BLACK_PIXEL(mi);
-		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		gcv.foreground = MI_BLACK_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
 		if ((ep->flyGC = XCreateGC(display, window,
 				 GCForeground | GCBackground, &gcv)) == None)
 			return;
@@ -932,15 +944,15 @@ init_eyes(ModeInfo * mi)
 		black.red = black.green = black.blue = 0;
 		black.flags = DoRed | DoGreen | DoBlue;
 		bit = XCreatePixmapFromBitmapData(display, window, "\000", 1, 1,
-						  MI_WIN_BLACK_PIXEL(mi),
-						  MI_WIN_BLACK_PIXEL(mi), 1);
+						  MI_BLACK_PIXEL(mi),
+						  MI_BLACK_PIXEL(mi), 1);
 		ep->cursor = XCreatePixmapCursor(display, bit, bit, &black, &black,
 						 0, 0);
 		XFreePixmap(display, bit);
 	}
 	XDefineCursor(display, window, ep->cursor);
 
-	XClearWindow(display, window);	/* clear the window */
+	MI_CLEARWINDOW(mi);
 }
 
 /* ---------------------------------------------------------------------- */
@@ -1023,7 +1035,7 @@ refresh_eyes(ModeInfo * mi)
 	if (!stuff_alloced) {
 		return;		/* paranoia */
 	}
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 
 	/* simply flag all the eyes as not painted, will repaint next time */
 	for (i = 0; i < eye_info[screen].num_eyes; i++) {
