@@ -2,7 +2,7 @@
 /* fadeplot --- a fading plot of sine squared */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)fadeplot.c  4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)fadeplot.c  4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -43,8 +43,16 @@ static const char sccsid[] = "@(#)fadeplot.c  4.04 97/07/28 xlockmore";
 ModeSpecOpt fadeplot_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   fadeplot_description =
+{"fadeplot", "init_fadeplot", "draw_fadeplot", "release_fadeplot",
+ "refresh_fadeplot", "init_fadeplot", NULL, &fadeplot_opts,
+ 30000, 10, 1500, 1, 64, 0.6, "",
+ "Shows a fading plot of sine squared", 0, NULL};
+
+#endif
+
 #define MINSTEPS 1
-#define ANGLES 1000
 
 typedef struct {
 	XPoint      speed, step, factor, st;
@@ -52,7 +60,8 @@ typedef struct {
 	int         min;
 	int         width, height;
 	int         pix;
-	int         stab[ANGLES];
+	int         angles;
+	int        *stab;
 	XPoint     *pts;
 } fadeplotstruct;
 
@@ -65,8 +74,11 @@ initSintab(ModeInfo * mi)
 	int         i;
 	float       x;
 
-	for (i = 0; i < ANGLES; i++) {
-		x = SINF(i * 2 * M_PI / ANGLES);
+	fp->angles = NRAND(950) + 250;
+	fp->stab = (int *) malloc(fp->angles * sizeof (int));
+
+	for (i = 0; i < fp->angles; i++) {
+		x = SINF(2.0 * M_PI * i / fp->angles);
 		fp->stab[i] = (int) (x * ABS(x) * fp->min) + fp->min;
 	}
 }
@@ -110,9 +122,11 @@ init_fadeplot(ModeInfo * mi)
 	if (MI_NPIXELS(mi) > 2)
 		fp->pix = NRAND(MI_NPIXELS(mi));
 
+	if (fp->stab != NULL)
+		(void) free((void *) fp->stab);
 	initSintab(mi);
 
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 }
 
 void
@@ -122,10 +136,9 @@ draw_fadeplot(ModeInfo * mi)
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	GC          gc = MI_GC(mi);
-	int         i, j;
-	long        temp;
+	int         i, j, temp;
 
-	XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+	XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	XDrawPoints(display, window, gc, fp->pts, fp->maxpts, CoordModeOrigin);
 
 	if (MI_NPIXELS(mi) > 2) {
@@ -133,43 +146,56 @@ draw_fadeplot(ModeInfo * mi)
 		if (++fp->pix >= MI_NPIXELS(mi))
 			fp->pix = 0;
 	} else
-		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 
-	for (temp = fp->nbstep - 1; temp >= 0; temp--) {
-		j = temp;
+	temp = 0;
+	for (j = 0; j < fp->nbstep; j++) {
 		for (i = 0; i < fp->maxpts / fp->nbstep; i++) {
-			fp->pts[temp * i + i].x =
-				fp->stab[(fp->st.x + fp->speed.x * j + i * fp->step.x) % ANGLES] *
+			fp->pts[temp].x =
+				fp->stab[(fp->st.x + fp->speed.x * j + i * fp->step.x) % fp->angles] *
 				fp->factor.x + fp->width / 2 - fp->min;
-			fp->pts[temp * i + i].y =
-				fp->stab[(fp->st.y + fp->speed.y * j + i * fp->step.y) % ANGLES] *
+			fp->pts[temp].y =
+				fp->stab[(fp->st.y + fp->speed.y * j + i * fp->step.y) % fp->angles] *
 				fp->factor.y + fp->height / 2 - fp->min;
+			temp++;
 		}
 	}
-	XDrawPoints(display, window, gc, fp->pts, fp->maxpts, CoordModeOrigin);
+	XDrawPoints(display, window, gc, fp->pts, temp, CoordModeOrigin);
 	XFlush(display);
-	fp->st.x = (fp->st.x + fp->speed.x) % ANGLES;
-	fp->st.y = (fp->st.y + fp->speed.y) % ANGLES;
+	fp->st.x = (fp->st.x + fp->speed.x) % fp->angles;
+	fp->st.y = (fp->st.y + fp->speed.y) % fp->angles;
 	fp->temps++;
-	if ((fp->temps % (ANGLES / 2)) == 0) {
-		fp->temps = fp->temps % ANGLES * 5;
-		if ((fp->temps % (ANGLES)) == 0)
+	if ((fp->temps % (fp->angles / 2)) == 0) {
+		fp->temps = fp->temps % fp->angles * 5;
+		if ((fp->temps % (fp->angles)) == 0)
 			fp->speed.y = (fp->speed.y++) % 30 + 1;
-		if ((fp->temps % (ANGLES * 2)) == 0)
+		if ((fp->temps % (fp->angles * 2)) == 0)
 			fp->speed.x = (fp->speed.x) % 20;
-		if ((fp->temps % (ANGLES * 3)) == 0)
+		if ((fp->temps % (fp->angles * 3)) == 0)
 			fp->step.y = (fp->step.y++) % 2 + 1;
-		XClearWindow(display, window);
+
+		MI_CLEARWINDOW(mi);
 	}
 }
 void
 refresh_fadeplot(ModeInfo * mi)
 {
-
+	/* Do nothing, it will refresh by itself */
 }
 
 void
 release_fadeplot(ModeInfo * mi)
 {
-	/* Do nothing, it will refresh by itself */
+	if (fadeplots != NULL) {
+		int         screen;
+
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			fadeplotstruct *fp = &fadeplots[screen];
+
+			(void) free((void *) fp->pts);
+			(void) free((void *) fp->stab);
+		}
+		(void) free((void *) fadeplots);
+		fadeplots = NULL;
+	}
 }
