@@ -2,7 +2,7 @@
 /* flame --- recursive fractal cosmic flames */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)flame.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)flame.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -51,6 +51,15 @@ static const char sccsid[] = "@(#)flame.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt flame_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   flame_description =
+{"flame", "init_flame", "draw_flame", "release_flame",
+ "refresh_flame", "init_flame", NULL, &flame_opts,
+ 750000, 20, 10000, 1, 64, 1.0, "",
+ "Shows cosmic flame fractals", 0, NULL};
+
+#endif
+
 #define MAXLEV    4
 #define MAXKINDS  9
 #define MAXBATCH  12
@@ -67,6 +76,7 @@ typedef struct {
 	int         total_points;
 	int         pixcol;
 	int         cycles;
+  int         alt;
 	XPoint      pts[MAXBATCH];
 } flamestruct;
 
@@ -262,14 +272,14 @@ init_flame(ModeInfo * mi)
 	fp->max_levels = MI_BATCHCOUNT(mi);
 	fp->cycles = MI_CYCLES(mi);
 
-	XClearWindow(display, MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 
 	if (MI_NPIXELS(mi) > 2) {
 		fp->pixcol = halfrandom(MI_NPIXELS(mi));
 		XSetForeground(display, gc, MI_PIXEL(mi, fp->pixcol));
-	} else
-		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
-
+	} else {
+		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
+	}
 	fp->variation = NRAND(MAXKINDS);
 }
 
@@ -279,11 +289,12 @@ draw_flame(ModeInfo * mi)
 	Display    *display = MI_DISPLAY(mi);
 	flamestruct *fp = &flames[MI_SCREEN(mi)];
 	int         i, j, k;
-	static      alt = 0;
 
 	if (!(fp->cur_level++ % fp->max_levels)) {
-		XClearWindow(display, MI_WINDOW(mi));
-		alt = !alt;
+		MI_CLEARWINDOW(mi);
+		if (MI_NPIXELS(mi) <= 2)
+			XSetForeground(display, MI_GC(mi), MI_WHITE_PIXEL(mi));
+		fp->alt = !fp->alt;
 	} else {
 		if (MI_NPIXELS(mi) > 2) {
 			XSetForeground(display, MI_GC(mi), MI_PIXEL(mi, fp->pixcol));
@@ -296,7 +307,7 @@ draw_flame(ModeInfo * mi)
 	fp->snum = 2 + (fp->cur_level % (MAXLEV - 1));
 
 	/* how many of them are of alternate form */
-	if (alt)
+	if (fp->alt)
 		fp->anum = 0;
 	else
 		fp->anum = halfrandom(fp->snum) + 2;
@@ -326,5 +337,5 @@ release_flame(ModeInfo * mi)
 void
 refresh_flame(ModeInfo * mi)
 {
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 }
