@@ -2,7 +2,7 @@
 /* morph3d --- Shows 3D morphing objects */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)morph3d.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)morph3d.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -83,6 +83,15 @@ static const char sccsid[] = "@(#)morph3d.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt morph3d_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   morph3d_description =
+{"morph3d", "init_morph3d", "draw_morph3d", "release_morph3d",
+ "draw_morph3d", "change_morph3d", NULL, &morph3d_opts,
+ 1000, 0, 1, 1, 4, 1.0, "",
+ "Shows GL morphing polyhedra", 0, NULL};
+
+#endif
+
 #define Scale4Window               0.3
 #define Scale4Iconic               1.0
 
@@ -277,7 +286,7 @@ static morph3dstruct *morph3d = NULL;
   GLfloat   AmpVr2=(Amp)/sqr((Edge)*cossec36_2);                                                                 \
                                                                                                                  \
   static    GLfloat x[6],y[6];                                                                                   \
-  static    arrayninit=1;                                                                                        \
+  static    int arrayninit=1;                                                                                    \
                                                                                                                  \
   if (arrayninit) {                                                                                              \
     for(Fi=0;Fi<6;Fi++) {                                                                                        \
@@ -790,6 +799,8 @@ init_morph3d(ModeInfo * mi)
 		if (mp->object <= 0 || mp->object > 5)
 			mp->object = NRAND(5) + 1;
 		pinit(mi);
+	} else {
+		MI_CLEARWINDOW(mi);
 	}
 }
 
@@ -874,7 +885,7 @@ release_morph3d(ModeInfo * mi)
 		(void) free((void *) morph3d);
 		morph3d = NULL;
 	}
-	FreeAllGL(MI_DISPLAY(mi));
+	FreeAllGL(mi);
 }
 
 #endif
