@@ -2,7 +2,7 @@
 /* ico --- bouncing polyhedra */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)ico.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)ico.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -130,7 +130,16 @@ static OptionStruct desc[] =
 };
 
 ModeSpecOpt ico_opts =
-{4, opts, 2, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   ico_description =
+{"ico", "init_ico", "draw_ico", "release_ico",
+ "refresh_ico", "change_ico", NULL, &ico_opts,
+ 100000, 0, 400, 0, 64, 1.0, "",
+ "Shows a bouncing polyhedron", 0, NULL};
+
+#endif
 
 #define MINSIZE 5
 #define DEFAULT_DELTAX 13
@@ -503,9 +512,13 @@ icoClearArea(ModeInfo * mi, int x, int y, int w, int h)
 {
 	Display    *display = MI_DISPLAY(mi);
 
-
+#if 1
+  /* my monochrome likes this better */
+	XSetForeground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
+	XFillRectangle(display, MI_WINDOW(mi), MI_GC(mi), x, y, w, h);
+#else
 	XClearArea(display, MI_WINDOW(mi), x, y, w, h, 0);
-
+#endif
 }
 
 /******************************************************************************
@@ -749,7 +762,7 @@ drawPoly(ModeInfo * mi, Polyinfo * poly, GC gc,
 				ppts[j].y = pv2[p0].y;
 			}
 			if (MI_NPIXELS(mi) <= 2)
-				XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+				XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 			else
 				XSetForeground(display, gc, MI_PIXEL(mi, facecolor));
 			XFillPolygon(display, window, gc, ppts, pcount,
@@ -779,13 +792,13 @@ drawPoly(ModeInfo * mi, Polyinfo * poly, GC gc,
 	/* Erase previous, draw current icosahedrons; sync for smoothness. */
 
 	if (ip->edges) {
+		icoClearArea(mi, prevX, prevY, icoW + 1, icoH + 1);
 		if (MI_NPIXELS(mi) <= 2)
-			XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+			XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 		else {
 			ip->color = (ip->color + 1) % MI_NPIXELS(mi);
 			XSetForeground(display, gc, MI_PIXEL(mi, ip->color));
 		}
-		icoClearArea(mi, prevX, prevY, icoW + 1, icoH + 1);
 		XDrawSegments(display, window, gc, edge_segs, pe - edge_segs);
 	}
 }
@@ -846,7 +859,9 @@ init_ico(ModeInfo * mi)
 	ip->poly = polygons + ip->object;
 	if (MI_NPIXELS(mi) > 2)
 		ip->color = NRAND(MI_NPIXELS(mi));
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+
+	MI_CLEARWINDOW(mi);
+
 	initPoly(mi, ip->poly, ip->polyW, ip->polyH);
 }
 
@@ -900,6 +915,8 @@ change_ico(ModeInfo * mi)
 	ip->object = (ip->object + 1) % polysize;
 	ip->poly = polygons + ip->object;
 	ip->loopcount = 0;
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+
+	MI_CLEARWINDOW(mi);
+
 	initPoly(mi, ip->poly, ip->polyW, ip->polyH);
 }
