@@ -2,7 +2,7 @@
 /* julia --- continuously varying Julia set */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)julia.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)julia.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -57,6 +57,15 @@ static const char sccsid[] = "@(#)julia.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt julia_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   julia_description =
+{"julia", "init_julia", "draw_julia", "release_julia",
+ "refresh_julia", "init_julia", NULL, &julia_opts,
+ 10000, 1000, 20, 1, 64, 1.0, "",
+ "Shows the Julia set", 0, NULL};
+
+#endif
+
 extern Bool mouse;
 
 #define numpoints ((0x2<<jp->depth)-1)
@@ -127,7 +136,7 @@ incr(ModeInfo * mi, juliastruct * jp)
 		int         rx, ry;
 		unsigned int m;
 
-		XQueryPointer(MI_DISPLAY(mi), MI_WINDOW(mi),
+		(void) XQueryPointer(MI_DISPLAY(mi), MI_WINDOW(mi),
 			      &r, &c, &rx, &ry, &cx, &cy, &m);
 		if (cx <= 0 || cy <= 0 ||
 		    cx >= MI_WIN_WIDTH(mi) - jp->circsize - 1 ||
@@ -182,8 +191,8 @@ init_julia(ModeInfo * mi)
 		black.red = black.green = black.blue = 0;
 		black.flags = DoRed | DoGreen | DoBlue;
 		bit = XCreatePixmapFromBitmapData(display, window, "\000", 1, 1,
-						  MI_WIN_BLACK_PIXEL(mi),
-						  MI_WIN_BLACK_PIXEL(mi), 1);
+						  MI_BLACK_PIXEL(mi),
+						  MI_BLACK_PIXEL(mi), 1);
 		jp->cursor = XCreatePixmapCursor(display, bit, bit, &black, &black,
 						 0, 0);
 		XFreePixmap(display, bit);
@@ -216,8 +225,8 @@ init_julia(ModeInfo * mi)
 			XFreeGC(display, bg_gc);
 	}
 	if (!jp->stippledGC) {
-		gcv.foreground = MI_WIN_BLACK_PIXEL(mi);
-		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		gcv.foreground = MI_BLACK_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
 		if ((jp->stippledGC = XCreateGC(display, window,
 				 GCForeground | GCBackground, &gcv)) == None)
 			return;
@@ -237,7 +246,8 @@ init_julia(ModeInfo * mi)
 	jp->buffer = 0;
 	jp->redrawing = 0;
 	jp->erase = 0;
-	XClearWindow(display, window);
+
+	MI_CLEARWINDOW(mi);
 }
 
 void
@@ -257,13 +267,13 @@ draw_julia(ModeInfo * mi)
 	incr(mi, jp);
 	new_circle.x = (int) (jp->centerx * jp->cr / 2) + jp->centerx - 2;
 	new_circle.y = (int) (jp->centery * jp->ci / 2) + jp->centery - 2;
-	XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+	XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	ERASE_IMAGE(display, window, gc, new_circle.x, new_circle.y,
 		    old_circle.x, old_circle.y, jp->circsize, jp->circsize);
 	/* draw a circle at the c-parameter so you can see it's effect on the
 	   structure of the julia set */
 	XSetTSOrigin(display, jp->stippledGC, new_circle.x, new_circle.y);
-	XSetForeground(display, jp->stippledGC, MI_WIN_WHITE_PIXEL(mi));
+	XSetForeground(display, jp->stippledGC, MI_WHITE_PIXEL(mi));
 	XSetStipple(display, jp->stippledGC, jp->pixmap);
 	XSetFillStyle(display, jp->stippledGC, FillOpaqueStippled);
 	XFillRectangle(display, window, jp->stippledGC, new_circle.x, new_circle.y,
@@ -279,7 +289,7 @@ draw_julia(ModeInfo * mi)
 		if (++jp->pix >= MI_NPIXELS(mi))
 			jp->pix = 0;
 	} else
-		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 	while (k--) {
 
 		/* save calls to LRAND by using bit shifts over and over on the same
