@@ -2,7 +2,7 @@
 /* life1d --- Stephen Wolfram's 1d game of Life */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)life1d.c	4.06 97/10/27 xlockmore";
+static const char sccsid[] = "@(#)life1d.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -80,7 +80,16 @@ static OptionStruct desc[] =
 };
 
 ModeSpecOpt life1d_opts =
-{2, opts, 1, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   life1d_description =
+{"life1d", "init_life1d", "draw_life1d", "release_life1d",
+ "refresh_life1d", "init_life1d", NULL, &life1d_opts,
+ 10000, 1, 10, 0, 64, 1.0, "",
+ "Shows Wolfram's game of 1D Life", 0, NULL};
+
+#endif
 
 #define LIFE1DBITS(n,w,h)\
   lp->pixmaps[lp->init_bits++]=\
@@ -108,19 +117,8 @@ static int  maxsum_size;
 #define MINGRIDSIZE 10
 #define MINSIZE 2		/* 3 may be better here */
 #define REPEAT 24
-#define PATTERNSIZE 8
 #define MAXSTATES 5
 
-static unsigned char patterns[MAXSTATES][PATTERNSIZE] =
-{
-	{0xee, 0xdd, 0xbb, 0x77, 0xee, 0xdd, 0xbb, 0x76},	/* dark \ stripe */
-	{0xff, 0x99, 0x99, 0xff, 0xff, 0x99, 0x99, 0xff},	/* spots */
-	{0xaa, 0xff, 0xff, 0x55, 0xaa, 0xff, 0xff, 0x55},	/* black+grey - stripe */
-	{0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff},	/* black */
-
-	{0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa}	/* 50% grey */
-};
-
 typedef struct {
 	int         init_bits;
 	int         pixelmode;
@@ -138,11 +136,12 @@ typedef struct {
 	unsigned char *newcells;
 	unsigned char *oldcells;
 	unsigned char *buffer;
-  XImage     *logo;
-  Colormap    cmap;
-  unsigned long black, white;
-  int         graphics_format;
-  GC          backGC;
+	XImage     *logo;
+	Colormap    cmap;
+	unsigned long black;
+	int         graphics_format;
+	GC          backGC;
+	int         busyLoop;
 } life1dstruct;
 
 static life1dstruct *life1ds = NULL;
@@ -419,7 +418,7 @@ drawcell(ModeInfo * mi, int col, int row, unsigned int state)
 			XGCValues   gcv;
 
 			gcv.stipple = lp->pixmaps[state - 1];
-			gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
+			gcv.foreground = MI_WHITE_PIXEL(mi);
 			gcv.background = lp->black;
 			gcv.fill_style = FillOpaqueStippled;
 			XChangeGC(display, lp->stippledGC,
@@ -430,8 +429,8 @@ drawcell(ModeInfo * mi, int col, int row, unsigned int state)
 			XFillRectangle(display, window, gc,
 				       lp->xb + lp->xs * col, lp->yb + lp->ys * row, lp->xs, lp->ys);
 	} else {
-		XPutImage(display, window, gc,
-		    lp->logo, 0, 0, lp->xb + lp->xs * col, lp->yb + lp->ys * row,
+		(void) XPutImage(display, window, gc,
+		lp->logo, 0, 0, lp->xb + lp->xs * col, lp->yb + lp->ys * row,
 			  lp->logo->width, lp->logo->height);
 	}
 }
@@ -542,44 +541,43 @@ compare(ModeInfo * mi)
 static void
 init_stuff(ModeInfo * mi)
 {
-  Display    *display = MI_DISPLAY(mi);
-  Window      window = MI_WINDOW(mi);
-  life1dstruct *lp = &life1ds[MI_SCREEN(mi)];
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	life1dstruct *lp = &life1ds[MI_SCREEN(mi)];
 
-  if (!lp->logo)
-    getImage(mi, &lp->logo, CELL_WIDTH, CELL_HEIGHT, CELL_BITS,
+	if (!lp->logo)
+		getImage(mi, &lp->logo, CELL_WIDTH, CELL_HEIGHT, CELL_BITS,
 #if defined( USE_XPM ) || defined( USE_XPMINC )
-       DEFAULT_XPM, CELL_NAME,
+			 DEFAULT_XPM, CELL_NAME,
 #endif
-       &lp->graphics_format, &lp->cmap,
-       &lp->black, &lp->white);
-  if (lp->cmap != None) {
-    setColormap(display, window, lp->cmap, MI_WIN_IS_INWINDOW(mi));
-    if (lp->backGC == None) {
-      XGCValues   xgcv;
+			 &lp->graphics_format, &lp->cmap, &lp->black);
+	if (lp->cmap != None) {
+		setColormap(display, window, lp->cmap, MI_WIN_IS_INWINDOW(mi));
+		if (lp->backGC == None) {
+			XGCValues   xgcv;
 
-      xgcv.background = lp->black;
-      lp->backGC = XCreateGC(display, window, GCBackground, &xgcv);
-    }
-  } else {
-    lp->black = MI_WIN_BLACK_PIXEL(mi);
-    lp->backGC = MI_GC(mi);
-  }
+			xgcv.background = lp->black;
+			lp->backGC = XCreateGC(display, window, GCBackground, &xgcv);
+		}
+	} else {
+		lp->black = MI_BLACK_PIXEL(mi);
+		lp->backGC = MI_GC(mi);
+	}
 }
 
 static void
 free_stuff(Display * display, life1dstruct * lp)
 {
-  if (lp->cmap != None) {
-    XFreeColormap(display, lp->cmap);
-    if (lp->backGC != None) {
-      XFreeGC(display, lp->backGC);
-      lp->backGC = None;
-    }
-    lp->cmap = None;
-  } else
-    lp->backGC = None;
-  destroyImage(&lp->logo, &lp->graphics_format);
+	if (lp->cmap != None) {
+		XFreeColormap(display, lp->cmap);
+		if (lp->backGC != None) {
+			XFreeGC(display, lp->backGC);
+			lp->backGC = None;
+		}
+		lp->cmap = None;
+	} else
+		lp->backGC = None;
+	destroyImage(&lp->logo, &lp->graphics_format);
 }
 
 void
@@ -598,7 +596,7 @@ init_life1d(ModeInfo * mi)
 	}
 	lp = &life1ds[MI_SCREEN(mi)];
 
-  init_stuff(mi);
+	init_stuff(mi);
 
 	lp->screen_generation = 0;
 	lp->row = 0;
@@ -620,8 +618,11 @@ init_life1d(ModeInfo * mi)
 
 		gcv.fill_style = FillOpaqueStippled;
 		lp->stippledGC = XCreateGC(display, window, GCFillStyle, &gcv);
-		for (i = 0; i < MAXSTATES; i++)
-			LIFE1DBITS(patterns[i], PATTERNSIZE, PATTERNSIZE);
+		for (i = 0; i < MAXSTATES - 1; i++)
+			LIFE1DBITS(stipples[i + NUMSTIPPLES - MAXSTATES + 1],
+				   STIPPLESIZE, STIPPLESIZE);
+		LIFE1DBITS(stipples[NUMSTIPPLES / 2],
+			   STIPPLESIZE, STIPPLESIZE);	/* grey */
 	}
 	if (lp->newcells != NULL)
 		(void) free((void *) lp->newcells);
@@ -675,15 +676,18 @@ init_life1d(ModeInfo * mi)
 
 	GetRule(lp, (int) NRAND((totalistic) ? TOTALISTICRULES : LCAURULES));
 	if (MI_WIN_IS_VERBOSE(mi)) {
-		(void) printf("colors %d, radius %d, code %ld, rule ",
-			      lp->k, lp->r, lp->code);
-		if (totalistic)
+		(void) fprintf(stdout, "colors %d, radius %d, code %ld, ",
+			       lp->k, lp->r, lp->code);
+		if (totalistic) {
+			(void) fprintf(stdout, "totalistic rule ");
 			for (i = (lp->k - 1) * (lp->r * 2 + 1); i >= 0; i--)
-				(void) printf("%d", (int) lp->nextstate[i]);
-		else
+				(void) fprintf(stdout, "%d", (int) lp->nextstate[i]);
+		} else {
+			(void) fprintf(stdout, "LCAU rule ");
 			for (i = power(lp->k, (lp->r * 2 + 1)); i >= 0; i--)
-				(void) printf("%d", (int) lp->nextstate[i]);
-		(void) printf("\n");
+				(void) fprintf(stdout, "%d", (int) lp->nextstate[i]);
+		}
+		(void) fprintf(stdout, "\n");
 	}
 	if (MI_NPIXELS(mi) > 2)
 		for (i = 0; i < lp->k - 1; i++)
@@ -692,10 +696,9 @@ init_life1d(ModeInfo * mi)
 	RandomSoup(lp, 40, 25);
 	(void) memcpy((char *) (lp->oldcells + maxradius + lp->border),
 		      (char *) (lp->newcells + lp->border), lp->ncols);
+	lp->busyLoop = 0;
 
-  XSetForeground(MI_DISPLAY(mi), lp->backGC, lp->black);
-  XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), lp->backGC,
-           0, 0, lp->width, lp->height);
+	MI_CLEARWINDOWCOLORMAP(mi, lp->backGC, lp->black);
 }
 
 void
@@ -705,6 +708,13 @@ draw_life1d(ModeInfo * mi)
 	Display    *display = MI_DISPLAY(mi);
 	int         col;
 
+	if (lp->busyLoop) {
+		if (lp->busyLoop >= 250)
+			lp->busyLoop = 0;
+		else
+			lp->busyLoop++;
+		return;
+	}
 	if (lp->row == 0) {
 		lp->repeating = 0;
 		if (lp->screen_generation > MI_CYCLES(mi))
@@ -777,7 +787,7 @@ draw_life1d(ModeInfo * mi)
 	}
 	if (lp->row >= lp->nrows) {
 		lp->screen_generation++;
-		MI_PAUSE(mi) = 2000000;
+		lp->busyLoop = 1;
 		lp->row = 0;
 	}
 }
@@ -807,7 +817,7 @@ release_life1d(ModeInfo * mi)
 				(void) free((void *) lp->buffer);
 			if (lp->nextstate != NULL)
 				(void) free((void *) lp->nextstate);
-      free_stuff(MI_DISPLAY(mi), lp);
+			free_stuff(MI_DISPLAY(mi), lp);
 		}
 		(void) free((void *) life1ds);
 		life1ds = NULL;
