@@ -1,8 +1,8 @@
 /* -*- Mode: C; tab-width: 4 -*- */
-/* life3d --- Extension to Conway's game of Life, Carter Bays' 4555 3d life */
+/* life3d --- Extension to Conway's game of Life, Carter Bays' S45/B5 3d life */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)life3d.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)life3d.c	4.07 98/01/18 xlockmore";
 
 #endif
 
@@ -24,7 +24,7 @@ static const char sccsid[] = "@(#)life3d.c	4.04 97/07/28 xlockmore";
  * Revision History:
  * 10-May-97: Compatible with xscreensaver
  * 18-Apr-97: Memory leak fixed by Tom Schmidt <tschmidt@micron.com>
- * 12-Mar-95: added LIFE_5766 compile-time option
+ * 12-Mar-95: added LIFE_S567B6 compile-time option
  * 12-Feb-95: shooting gliders added
  * 07-Dec-94: used life.c and a DOS version of 3dlife
  * Copyright 1993 Anthony Wesley awesley@canb.auug.org.au found at
@@ -56,28 +56,44 @@ static const char sccsid[] = "@(#)life3d.c	4.04 97/07/28 xlockmore";
 
 #endif /* STANDALONE */
 
-/* #define DEF_RULE3D  "0" */
-#define DEF_RULE3D  "1"
-/* #define DEF_RULE3D  "4555" */
-/* #define DEF_RULE3D  "5766" */
+#if 1
+#define DEF_RULE3D  "G"		/* All rules with gliders */
+#else
+#define DEF_RULE3D  "P"		/* All rules with known patterns */
+#define DEF_RULE3D  "S45/B5"	/* "B5/S45" */
+#define DEF_RULE3D  "S567/B6"	/* "B6/S567" */
+#endif
 
-static int  rule3d;
+static char *rule3d;
+static char *life3dfile;
 
 static XrmOptionDescRec opts[] =
 {
 	{"-rule3d", ".life3d.rule3d", XrmoptionSepArg, (caddr_t) NULL},
+	{"-life3dfile", ".life3d.life3dfile", XrmoptionSepArg, (caddr_t) NULL}
 };
 static argtype vars[] =
 {
-	{(caddr_t *) & rule3d, "rule3d", "Rule3D", DEF_RULE3D, t_Int}
+	{(caddr_t *) & rule3d, "rule3d", "Rule3D", DEF_RULE3D, t_String},
+	{(caddr_t *) & life3dfile, "life3dfile", "Life3DFile", "", t_String}
 };
 static OptionStruct desc[] =
 {
- {"-rule3d num", "<living_min><living_max><birth_min><birth_max> parameters"}
+	{"-rule3d string", "S<survial_neighborhood>/B<birth_neighborhood> parameters"},
+	{"-life3dfile file", "life file"},
 };
 
 ModeSpecOpt life3d_opts =
-{1, opts, 1, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   life3d_description =
+{"life3d", "init_life3d", "draw_life3d", "release_life3d",
+ "refresh_life3d", "change_life3d", NULL, &life3d_opts,
+ 1000000, 35, 85, 1, 64, 1.0, "",
+ "Shows Bays' game of 3D Life", 0, NULL};
+
+#endif
 
 #define ON 0x40
 #define OFF 0
@@ -90,6 +106,8 @@ ModeSpecOpt life3d_opts =
 
 #define RT_ANGLE 90
 #define HALFRT_ANGLE 45
+#define MAXNEIGHBORS 27
+
 /* Store state of cell in top bit. Reserve low bits for count of living nbrs */
 #define Set3D(x,y,z) SetMem(lp,(unsigned int)x,(unsigned int)y,(unsigned int)z,ON)
 #define Reset3D(x,y,z) SetMem(lp,(unsigned int)x,(unsigned int)y,(unsigned int)z,OFF)
@@ -109,7 +127,7 @@ ModeSpecOpt life3d_opts =
 
 #define COLORBASE 3
 #define COLORS (COLORBASE + 2)
-#define COLORSTEP (NUMCOLORS/COLORBASE)		/* 21 different colors per side */
+#define COLORSTEP (MI_NPIXELS(mi)/COLORBASE)		/* 21 different colors per side */
 
 #define BLACK 0
 #define RED 1
@@ -117,6 +135,7 @@ ModeSpecOpt life3d_opts =
 #define BLUE 3
 #define WHITE 4
 #define NUMPTS 42
+#define NUMFILEPTS (2*NUMPTS)
 
 typedef struct _CellList {
 	unsigned char x, y, z;	/* Location in world coordinates */
@@ -128,9 +147,12 @@ typedef struct _CellList {
 } CellList;
 
 typedef struct {
-	int         living_min, living_max, birth_min, birth_max;
+	int         survival, birth;
+} paramstruct;
+
+typedef struct {
+	paramstruct param;
 	int         pattern, patterned_rule;
-	int         rule3d;
 	int         initialized;
 	int         wireframe;	/* FALSE */
 	int         ox, oy, oz;	/* origin */
@@ -149,22 +171,20 @@ typedef struct {
 	CellList   *buckethead[NBUCKETS], *bucketend[NBUCKETS];		/* comfortable upper b */
 } life3dstruct;
 
-
 static life3dstruct *life3ds = NULL;
 
-static int  DrawCube(ModeInfo * mi, CellList * cell);
+static paramstruct input_param;
+static Bool allPatterns = False, allGliders = False;
+static char *filePattern = NULL;
 
-/* 4555 life is probably the best */
-/* 5766 life has gliders like 2d 2333 life */
-/* There are no known gliders for 5655 or 6767 so the others may be better */
-#define LIFE_4555 0
-#define LIFE_5766 1
-#define LIFE_GLIDERS 2
-#define LIFE_5655 2
-#define LIFE_6767 3
-#define LIFE_RULES 4
+/*-
+ * S45/B5 life is probably the best
+ * S567/B6 life has gliders like Conway's 2d S23/B3 life
+ * There are no known gliders for S56/B5 or S67/B67,
+ * so the others may be better
+ */
 
-static char patterns_4555[][3 * NUMPTS + 1] =
+static char patterns_S45B5[][3 * NUMPTS + 1] =
 {
 #if 0
 /* Still life */
@@ -440,7 +460,7 @@ static char patterns_4555[][3 * NUMPTS + 1] =
 	}
 };
 
-static char patterns_5766[][3 * NUMPTS + 1] =
+static char patterns_S567B6[][3 * NUMPTS + 1] =
 {
 	{			/* KNIFE-SWITCH BLINKER */
 		0, -1, -1,
@@ -694,9 +714,11 @@ static char patterns_5766[][3 * NUMPTS + 1] =
 	}
 };
 
-/* Many names of 5655 & 6767 are made up by bagleyd They are listed in order
-   given by Carter Bays all p2 except "H"  */
-static char patterns_5655[][3 * NUMPTS + 1] =
+/*-
+ * Many names of S56/B5 & S67/B67 are made up by David Bagley.
+ * They are listed in * order given by Carter Bays all p2 except "H"
+ */
+static char patterns_S56B5[][3 * NUMPTS + 1] =
 {
 	{			/* SEESAW */
 		0, -1, -1,
@@ -1080,7 +1102,7 @@ static char patterns_5655[][3 * NUMPTS + 1] =
 		127
 	}
 };
-static char patterns_6767[][3 * NUMPTS + 1] =
+static char patterns_S67B67[][3 * NUMPTS + 1] =
 {
 	{			/* WALKING BOX */
 		-1, -1, -2,
@@ -1158,46 +1180,190 @@ static char patterns_6767[][3 * NUMPTS + 1] =
 	}
 };
 
-static int  npatterns[] =
+static int  patterns_rules[] =
 {
-	(sizeof patterns_4555 / sizeof patterns_4555[0]),
-	(sizeof patterns_5766 / sizeof patterns_5766[0]),
-	(sizeof patterns_5655 / sizeof patterns_5655[0]),
-	(sizeof patterns_6767 / sizeof patterns_6767[0]),
+	(sizeof patterns_S45B5 / sizeof patterns_S45B5[0]),
+	(sizeof patterns_S567B6 / sizeof patterns_S567B6[0]),
+	(sizeof patterns_S56B5 / sizeof patterns_S56B5[0]),
+	(sizeof patterns_S67B67 / sizeof patterns_S67B67[0])
 };
 
-static int
-Rule2Array(int rule)
+static paramstruct param_rules[] =
 {
-	switch (rule) {
-		case 4555:
-			return LIFE_4555;
-		case 5766:
-			return LIFE_5766;
-		case 5655:
-			return LIFE_5655;
-		case 6767:
-			return LIFE_6767;
-		default:
-			return LIFE_RULES;
-	}
-}
+	{0x30, 0x20},
+	{0xE0, 0x40},
+	{0x60, 0x20},
+	{0xC0, 0xC0}
+};
+
+#define LIFE_S45B5 0
+#define LIFE_S567B6 1
+#define LIFE_GLIDERS 2		/* GLIDER rules are first in param_rules */
+#define LIFE_S56B5 2
+#define LIFE_S67B67 3
+#define LIFE_RULES (sizeof patterns_rules / sizeof patterns_rules[0])
 
 static int
-Array2Rule(int array)
+codeToPatternedRule(paramstruct param)
 {
-	switch (array) {
-		case LIFE_4555:
-			return 4555;
-		case LIFE_5766:
-			return 5766;
-		case LIFE_5655:
-			return 5655;
-		case LIFE_6767:
-			return 6767;
-		default:
-			return 0;
+	int         i;
+
+	for (i = 0; i < LIFE_RULES; i++)
+		if (param_rules[i].survival == param.survival &&
+		    param_rules[i].birth == param.birth)
+			return i;
+	return LIFE_RULES;
+}
+
+static void
+copyFromPatternedRule(paramstruct * param, int patterned_rule)
+{
+	param->survival = param_rules[patterned_rule].survival;
+	param->birth = param_rules[patterned_rule].birth;
+}
+
+static void
+printRule(paramstruct param)
+{
+	int         l;
+
+	(void) fprintf(stdout, "rule3d (Survival/Birth neighborhood): S");
+	for (l = 0; l <= MAXNEIGHBORS; l++) {
+		if (param.survival & (1 << l))
+			(void) fprintf(stdout, "%d", l);
 	}
+	(void) fprintf(stdout, "/B");
+	for (l = 0; l <= MAXNEIGHBORS; l++) {
+		if (param.birth & (1 << l))
+			(void) fprintf(stdout, "%d", l);
+	}
+	(void) fprintf(stdout, "\nbinary rule3d: Survival 0x%X, Birth 0x%X\n",
+		       param.survival, param.birth);
+}
+
+/*-
+ * This stuff is not good for rules above 9 cubes but it is unlikely that
+ * these modes would be much good anyway....  death assumed.
+ */
+static void
+parseRule(ModeInfo * mi)
+{
+	int         n, l;
+	char        serving = 0;
+	static Bool found = False;
+
+	if (found)
+		return;
+
+	input_param.survival = input_param.birth = 0;
+	if (rule3d) {
+		n = 0;
+		while (rule3d[n]) {
+			if (rule3d[n] == 'P') {
+				allPatterns = True;
+				found = True;
+				if (MI_WIN_IS_VERBOSE(mi))
+					(void) fprintf(stdout, "rule3d: All rules with known patterns\n");
+				return;
+			} else if (rule3d[n] == 'G') {
+				allGliders = True;
+				found = True;
+				if (MI_WIN_IS_VERBOSE(mi))
+					(void) fprintf(stdout, "rule3d: All rules with known gliders\n");
+				return;
+			} else if (rule3d[n] == 'S' || rule3d[n] == 'E' || rule3d[n] == 'L') {
+				serving = 'S';
+			} else if (rule3d[n] == 'B') {
+				serving = 'B';
+			} else {
+				l = rule3d[n] - '0';
+				if (l >= 0 && l <= 9 /*&& l <= MAXNEIGHBORS */ ) {	/* no 10..27 */
+					if (serving == 'S') {
+						found = True;
+						input_param.survival |= (1 << l);
+					} else if (serving == 'B') {
+						found = True;
+						input_param.birth |= (1 << l);
+					}
+				}
+			}
+			n++;
+		}
+	}
+	if (!found) {		/* Default to Bays' rules if very stupid */
+		allGliders = True;
+		found = True;
+		if (MI_WIN_IS_VERBOSE(mi))
+			(void) fprintf(stdout,
+				       "rule3d: Defaulting to all rules with known gliders\n");
+		return;
+	}
+	if (MI_WIN_IS_VERBOSE(mi))
+		printRule(input_param);
+}
+
+static void
+parseFile()
+{
+	FILE       *file;
+	static Bool done = False;
+	int         firstx = 0, firsty = 0, x = 0, y = 0, z = 0, i = 0;
+	char        line[256], c, cprev = ' ';
+
+	if (done)
+		return;
+	done = True;
+	if (!strlen(life3dfile) || ((file = my_fopen(life3dfile, "r")) == NULL)) {
+		/*(void) fprintf(stderr, "could not read file \"%s\"\n", life3dfile); */
+		return;
+	}
+	for (;;) {
+		if (!fgets(line, 256, file)) {
+			(void) fprintf(stderr, "could not read header of file \"%s\"\n",
+				       life3dfile);
+			(void) fclose(file);
+			return;
+		}
+		if (strncmp(line, "#P", (size_t) 2) == 0 &&
+		    sscanf(line, "#P %d %d %d", &x, &y, &z) == 3)
+			break;
+	}
+	c = getc(file);
+	while (c != EOF && !(c == '0' || c == 'O' || c == '*' || c == '.')) {
+		c = getc(file);
+	}
+	if (c == EOF || x <= -127 || y <= -127 || z <= -127 ||
+	    x >= 127 || y >= 127 || z >= 127) {
+		(void) fprintf(stderr, "corrupt file \"%s\" or file to large\n",
+			       life3dfile);
+		(void) fclose(file);
+		return;
+	}
+	firstx = x;
+	firsty = y;
+	filePattern = (char *) malloc((3 * NUMFILEPTS + 1) * sizeof (char));
+
+	while (c != EOF && x < 127 && y < 127 && i < 3 * NUMFILEPTS) {
+		if (c == '0' || c == 'O' || c == '*') {
+			filePattern[i++] = x++;
+			filePattern[i++] = y;
+			filePattern[i++] = z;
+		} else if (c == '.') {
+			x++;
+		} else if (c == '\n') {
+			if (cprev == '\n') {
+				z++;
+				y = firsty;
+			} else {
+				x = firstx;
+				y++;
+			}
+		}
+		cprev = c;
+		c = getc(file);
+	}
+	(void) fclose(file);
+	filePattern[i] = 127;
 }
 
 /*--- list ---*/
@@ -1435,19 +1601,19 @@ RunLife3D(life3dstruct * lp)
 					if (x != xc || y != yc || z != zc)
 						if ((c = GetMem(lp, x, y, z))) {
 							if (CellState3D(c) == OFF) {
-								if (CellNbrs3D(c) >= lp->birth_min &&
-								    CellNbrs3D(c) <= lp->birth_max)
+								if (lp->param.birth & (1 << CellNbrs3D(c)))
 									SetList3D(x, y, z);
 								else
 									Reset3D(x, y, z);
 							}
 						}
 		c = GetMem(lp, xc, yc, zc);
-		if (CellNbrs3D(c) < lp->living_min || CellNbrs3D(c) > lp->living_max) {
+		if (lp->param.survival & (1 << CellNbrs3D(c)))
+			Set3D(xc, yc, zc);
+		else {
 			SetMem(lp, ptr->x, ptr->y, ptr->z, OFF);
 			DelFromList(lp, ptr);
-		} else
-			Set3D(xc, yc, zc);
+		}
 		ptr = ptrnextcell;
 	}
 	ClearMem(lp);
@@ -1499,8 +1665,9 @@ RandomSoup(ModeInfo * mi, int n, int v)
 			for (x = lp->ncolumns / 2 - v; x < lp->ncolumns / 2 + v; ++x)
 				if (NRAND(100) < n)
 					SetList3D(x, y, z);
-	if (MI_WIN_IS_VERBOSE(mi))
+	if (MI_WIN_IS_VERBOSE(mi)) {
 		(void) fprintf(stdout, "random pattern\n");
+	}
 }
 
 static void
@@ -1510,19 +1677,23 @@ GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 	int         x, y, z;
 	char       *patptr = NULL;
 
-	switch (pattern_rule) {
-		case LIFE_4555:
-			patptr = &patterns_4555[pattern][0];
-			break;
-		case LIFE_5766:
-			patptr = &patterns_5766[pattern][0];
-			break;
-		case LIFE_5655:
-			patptr = &patterns_5655[pattern][0];
-			break;
-		case LIFE_6767:
-			patptr = &patterns_6767[pattern][0];
-			break;
+	if (filePattern) {
+		patptr = &filePattern[0];
+	} else {
+		switch (pattern_rule) {
+			case LIFE_S45B5:
+				patptr = &patterns_S45B5[pattern][0];
+				break;
+			case LIFE_S567B6:
+				patptr = &patterns_S567B6[pattern][0];
+				break;
+			case LIFE_S56B5:
+				patptr = &patterns_S56B5[pattern][0];
+				break;
+			case LIFE_S67B67:
+				patptr = &patterns_S67B67[pattern][0];
+				break;
+		}
 	}
 	while ((x = *patptr++) != 127) {
 		y = *patptr++;
@@ -1534,8 +1705,9 @@ GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 		    x < lp->ncolumns && y < lp->nrows && z < lp->nstacks)
 			SetList3D(x, y, z);
 	}
-	if (MI_WIN_IS_VERBOSE(mi))
+	if (MI_WIN_IS_VERBOSE(mi) && !filePattern) {
 		(void) fprintf(stdout, "table number %d\n", pattern);
+	}
 }
 
 static void
@@ -1707,9 +1879,9 @@ DrawFace(ModeInfo * mi, int color, XPoint * cubepts,
 			     Convex, CoordModeOrigin);
 	}
 	if (color == BLACK || (MI_NPIXELS(mi) <= 2 && !lp->wireframe))
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	else
-		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 	XDrawLines(display, MI_WINDOW(mi), gc, facepts, 5, CoordModeOrigin);
 }
 
@@ -1832,7 +2004,7 @@ shooter(life3dstruct * lp)
 			voff = -1;
 		if (hsp > c2)
 			hoff = -1;
-		if (lp->rule3d == 4555) {
+		if (lp->patterned_rule == LIFE_S45B5) {
 			SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 0 * aoff);
 			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 0 * aoff);
 			SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 0 * aoff);
@@ -1843,7 +2015,7 @@ shooter(life3dstruct * lp)
 			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 1 * aoff);
 			SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 1 * aoff);
 			SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 1 * aoff);
-		} else if (lp->rule3d == 5766) {
+		} else if (lp->patterned_rule == LIFE_S567B6) {
 			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 0 * aoff);
 			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 0 * aoff);
 			SetList3D(hsp + 0 * hoff, vsp + 2 * voff, asp + 1 * aoff);
@@ -1865,7 +2037,7 @@ shooter(life3dstruct * lp)
 			voff = -1;
 		if (hsp > c2)
 			hoff = -1;
-		if (lp->rule3d == 4555) {
+		if (lp->patterned_rule == LIFE_S45B5) {
 			SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 0 * aoff);
 			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 0 * aoff);
 			SetList3D(hsp + 2 * hoff, vsp + 0 * voff, asp + 1 * aoff);
@@ -1876,7 +2048,7 @@ shooter(life3dstruct * lp)
 			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 1 * aoff);
 			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 2 * aoff);
 			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 2 * aoff);
-		} else if (lp->rule3d == 5766) {
+		} else if (lp->patterned_rule == LIFE_S567B6) {
 			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 0 * aoff);
 			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 1 * aoff);
 			SetList3D(hsp + 2 * hoff, vsp + 1 * voff, asp + 0 * aoff);
@@ -1898,7 +2070,7 @@ shooter(life3dstruct * lp)
 			voff = -1;
 		if (hsp > c2)
 			hoff = -1;
-		if (lp->rule3d == 4555) {
+		if (lp->patterned_rule == LIFE_S45B5) {
 			SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 0 * aoff);
 			SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 1 * aoff);
 			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 2 * aoff);
@@ -1909,7 +2081,7 @@ shooter(life3dstruct * lp)
 			SetList3D(hsp + 1 * hoff, vsp + 1 * voff, asp + 1 * aoff);
 			SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 0 * aoff);
 			SetList3D(hsp + 1 * hoff, vsp + 2 * voff, asp + 1 * aoff);
-		} else if (lp->rule3d == 5766) {
+		} else if (lp->patterned_rule == LIFE_S567B6) {
 			SetList3D(hsp + 0 * hoff, vsp + 0 * voff, asp + 1 * aoff);
 			SetList3D(hsp + 0 * hoff, vsp + 1 * voff, asp + 1 * aoff);
 			SetList3D(hsp + 1 * hoff, vsp + 0 * voff, asp + 2 * aoff);
@@ -1924,24 +2096,11 @@ shooter(life3dstruct * lp)
 	}
 }
 
-static void
-initrule(ModeInfo * mi)
-{
-	life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
-
-	if (MI_WIN_IS_VERBOSE(mi))
-		(void) fprintf(stdout, "rule3d %d\n", lp->rule3d);
-	lp->living_min = lp->rule3d / 1000;
-	lp->living_max = (lp->rule3d / 100) % 10;
-	lp->birth_min = (lp->rule3d / 10) % 10;
-	lp->birth_max = lp->rule3d % 10;
-}
-
 void
 init_life3d(ModeInfo * mi)
 {
 	life3dstruct *lp;
-	int         i, pattern_rule, npats;
+	int         i, npats;
 
 	if (life3ds == NULL) {
 		if ((life3ds = (life3dstruct *) calloc(MI_NUM_SCREENS(mi),
@@ -1952,21 +2111,21 @@ init_life3d(ModeInfo * mi)
 
 	lp->generation = 0;
 
-	lp->rule3d = rule3d;
-	if (rule3d == 0)	/* All rules with patterns */
-		lp->rule3d = Array2Rule(NRAND(LIFE_RULES));
-	else if (rule3d == 1)	/* All rules with known gliders */
-		lp->rule3d = Array2Rule(NRAND(LIFE_GLIDERS));
-	if (lp->rule3d != rule3d || !lp->initialized) {
-		/*
-		 * This stuff is not good for rules above 9 cubes but it is unlikely that
-		 * these modes would be much good anyway....
-		 * Rules should not have to be contiguous ...  that is, it should be
-		 * possible to have rules where cells will live if there are 5 or 7
-		 * (not 6) cells around them.   These rules are not general enough
-		 * to allow for that.
-		 */
-		initrule(mi);
+	parseRule(mi);
+	parseFile();
+	if (allPatterns) {
+		lp->patterned_rule = NRAND(LIFE_RULES);
+		copyFromPatternedRule(&lp->param, lp->patterned_rule);
+		if (MI_WIN_IS_VERBOSE(mi))
+			printRule(lp->param);
+	} else if (allGliders) {
+		lp->patterned_rule = NRAND(LIFE_GLIDERS);
+		copyFromPatternedRule(&lp->param, lp->patterned_rule);
+		if (MI_WIN_IS_VERBOSE(mi))
+			printRule(lp->param);
+	} else {
+		lp->param.survival = input_param.survival;
+		lp->param.birth = input_param.birth;
 	}
 	if (!lp->initialized) {
 		lp->dist = 50.0 /*30.0 */ ;
@@ -1983,7 +2142,7 @@ init_life3d(ModeInfo * mi)
 		Init3D(lp);
 	} else
 		End3D(lp);
-	lp->color[0] = MI_WIN_BLACK_PIXEL(mi);
+	lp->color[0] = MI_BLACK_PIXEL(mi);
 	if (MI_NPIXELS(mi) > 2) {
 		i = NRAND(3);
 
@@ -1995,14 +2154,14 @@ init_life3d(ModeInfo * mi)
 					  NRAND(MI_NPIXELS(mi) / COLORBASE) +
 					     2 * MI_NPIXELS(mi) / COLORBASE);
 	} else
-		lp->color[1] = lp->color[2] = lp->color[3] = MI_WIN_WHITE_PIXEL(mi);
-	lp->color[4] = MI_WIN_WHITE_PIXEL(mi);
+		lp->color[1] = lp->color[2] = lp->color[3] = MI_WHITE_PIXEL(mi);
+	lp->color[4] = MI_WHITE_PIXEL(mi);
 	lp->width = MI_WIN_WIDTH(mi);
 	lp->height = MI_WIN_HEIGHT(mi);
 	lp->memstart = 1;
 	/*lp->tablesMade = 0; */
 
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 
 	if (lp->alt > 89)
 		lp->alt = 89;
@@ -2014,16 +2173,16 @@ init_life3d(ModeInfo * mi)
 	lp->vz = (sin(lp->alt * IP) * lp->dist);
 	NewViewpoint(lp, lp->vx, lp->vy, lp->vz);
 
-	pattern_rule = Rule2Array(lp->rule3d);
-	if (pattern_rule < LIFE_RULES)
-		npats = npatterns[pattern_rule];
+	lp->patterned_rule = codeToPatternedRule(lp->param);
+	if (lp->patterned_rule < LIFE_RULES)
+		npats = patterns_rules[lp->patterned_rule];
 	else
 		npats = 0;
 	lp->pattern = NRAND(npats + 2);
-	if (lp->pattern >= npats)
+	if (lp->pattern >= npats && !filePattern)
 		RandomSoup(mi, 30, 10);
 	else
-		GetPattern(mi, pattern_rule, lp->pattern);
+		GetPattern(mi, lp->patterned_rule, lp->pattern);
 
 	DrawScreen(mi);
 }
@@ -2076,41 +2235,43 @@ void
 change_life3d(ModeInfo * mi)
 {
 	life3dstruct *lp = &life3ds[MI_SCREEN(mi)];
-	int         npats, pattern_rule;
+	int         npats;
 
 	lp->generation = 0;
 
 	End3D(lp);
 	/*lp->tablesMade = 0; */
 
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 
-	lp->pattern = lp->pattern + 1;
-	pattern_rule = Rule2Array(lp->rule3d);
-	if (pattern_rule < LIFE_RULES)
-		npats = npatterns[pattern_rule];
+	lp->pattern++;
+	lp->patterned_rule = codeToPatternedRule(lp->param);
+	if (lp->patterned_rule < LIFE_RULES)
+		npats = patterns_rules[lp->patterned_rule];
 	else
 		npats = 0;
 	if (lp->pattern >= npats + 2) {
 		lp->pattern = 0;
-		if (rule3d == 0) {
-			pattern_rule++;
-			if (pattern_rule >= LIFE_RULES)
-				pattern_rule = 0;
-			lp->rule3d = Array2Rule(pattern_rule);
-			initrule(mi);
-		} else if (rule3d == 1) {
-			pattern_rule++;
-			if (pattern_rule >= LIFE_GLIDERS)
-				pattern_rule = 0;
-			lp->rule3d = Array2Rule(pattern_rule);
-			initrule(mi);
+		if (allPatterns) {
+			lp->patterned_rule++;
+			if (lp->patterned_rule >= LIFE_RULES)
+				lp->patterned_rule = 0;
+			copyFromPatternedRule(&lp->param, lp->patterned_rule);
+			if (MI_WIN_IS_VERBOSE(mi))
+				printRule(lp->param);
+		} else if (allGliders) {
+			lp->patterned_rule++;
+			if (lp->patterned_rule >= LIFE_GLIDERS)
+				lp->patterned_rule = 0;
+			copyFromPatternedRule(&lp->param, lp->patterned_rule);
+			if (MI_WIN_IS_VERBOSE(mi))
+				printRule(lp->param);
 		}
 	}
 	if (lp->pattern >= npats)
 		RandomSoup(mi, 30, 10);
 	else
-		GetPattern(mi, pattern_rule, lp->pattern);
+		GetPattern(mi, lp->patterned_rule, lp->pattern);
 
 	DrawScreen(mi);
 }
