@@ -2,7 +2,7 @@
 /* loop --- Chris Langton's self-producing loops */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)loop.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)loop.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -55,7 +55,7 @@ static const char sccsid[] = "@(#)loop.c	4.04 97/07/28 xlockmore";
 #define DEFAULTS "*delay: 100000 \n" \
  "*cycles: 1600 \n" \
  "*size: -12 \n" \
- "*ncolors: 64 \n"
+ "*ncolors: 15 \n"
 #define SPREAD_COLORS
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
@@ -65,31 +65,28 @@ static const char sccsid[] = "@(#)loop.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt loop_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   loop_description =
+{"loop", "init_loop", "draw_loop", "release_loop",
+ "refresh_loop", "init_loop", NULL, &loop_opts,
+ 100000, 1, 1600, -12, 64, 1.0, 15, "",
+ "Shows Langton's self-producing loops", 0, NULL};
+
+#endif
+
 #define LOOPBITS(n,w,h)\
   lp->pixmaps[lp->init_bits++]=\
   XCreatePixmapFromBitmapData(display,window,(char *)n,w,h,1,0,1)
 
 #define COLORS 8
+#define REALCOLORS (COLORS-2)
 #define MINLOOPS 1
-#define PATTERNSIZE 8
 #define REDRAWSTEP 2000		/* How many cells to draw per cycle */
 #define ADAM_LOOPX 16
 #define ADAM_LOOPY 10
 #define MINGRIDSIZE (3*ADAM_LOOPX)
 #define MINSIZE 1
 
-static unsigned char patterns[COLORS][PATTERNSIZE] =
-{
-	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},	/* white */
-	{0x00, 0x66, 0x66, 0x00, 0x00, 0x66, 0x66, 0x00},	/* spots */
-	{0x89, 0x44, 0x22, 0x11, 0x88, 0x44, 0x22, 0x11},	/* lt. / stripe */
-	{0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66},	/* | bars */
-	{0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00},	/* - bars */
-	{0xee, 0xdd, 0xbb, 0x77, 0xee, 0xdd, 0xbb, 0x76},	/* dark \ stripe */
-	{0xff, 0x99, 0x99, 0xff, 0xff, 0x99, 0x99, 0xff},	/* spots */
-	{0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff},	/* black */
-};
-
 /* Singly linked list */
 typedef struct _RectList {
 	XPoint      pt;
@@ -253,13 +250,13 @@ drawcell(ModeInfo * mi, int col, int row, int state)
 	XGCValues   gcv;
 	GC          gc;
 
-	if (MI_NPIXELS(mi) > 2) {
+	if (MI_NPIXELS(mi) >= COLORS) {
 		gc = MI_GC(mi);
 		XSetForeground(MI_DISPLAY(mi), gc, lp->colors[state]);
 	} else {
 		gcv.stipple = lp->pixmaps[state];
-		gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
-		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		gcv.foreground = MI_WHITE_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
 		XChangeGC(MI_DISPLAY(mi), lp->stippledGC,
 			  GCStipple | GCForeground | GCBackground, &gcv);
 		gc = lp->stippledGC;
@@ -277,13 +274,13 @@ draw_state(ModeInfo * mi, int state)
 	XGCValues   gcv;
 	RectList   *current;
 
-	if (MI_NPIXELS(mi) > 2) {
+	if (MI_NPIXELS(mi) >= COLORS) {
 		gc = MI_GC(mi);
 		XSetForeground(MI_DISPLAY(mi), gc, lp->colors[state]);
 	} else {
 		gcv.stipple = lp->pixmaps[state];
-		gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
-		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		gcv.foreground = MI_WHITE_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
 		XChangeGC(MI_DISPLAY(mi), lp->stippledGC,
 			  GCStipple | GCForeground | GCBackground, &gcv);
 		gc = lp->stippledGC;
@@ -437,7 +434,6 @@ init_loop(ModeInfo * mi)
 	int         size = MI_SIZE(mi);
 	loopstruct *lp;
 	XGCValues   gcv;
-	int         i;
 
 	if (loops == NULL) {
 		if ((loops = (loopstruct *) calloc(MI_NUM_SCREENS(mi),
@@ -446,26 +442,32 @@ init_loop(ModeInfo * mi)
 	}
 	lp = &loops[MI_SCREEN(mi)];
 	lp->redrawing = 0;
-	if (MI_NPIXELS(mi) <= 2) {
+	if (MI_NPIXELS(mi) < COLORS) {
 		if (lp->stippledGC == None) {
 			gcv.fill_style = FillOpaqueStippled;
 			lp->stippledGC = XCreateGC(display, window, GCFillStyle, &gcv);
 		}
 		if (lp->init_bits == 0) {
-			for (i = 0; i < COLORS; i++)
-				LOOPBITS(patterns[i], PATTERNSIZE, PATTERNSIZE);
+			LOOPBITS(stipples[0], STIPPLESIZE, STIPPLESIZE);
+			LOOPBITS(stipples[2], STIPPLESIZE, STIPPLESIZE);
+			LOOPBITS(stipples[3], STIPPLESIZE, STIPPLESIZE);
+			LOOPBITS(stipples[4], STIPPLESIZE, STIPPLESIZE);
+			LOOPBITS(stipples[6], STIPPLESIZE, STIPPLESIZE);
+			LOOPBITS(stipples[7], STIPPLESIZE, STIPPLESIZE);
+			LOOPBITS(stipples[8], STIPPLESIZE, STIPPLESIZE);
+			LOOPBITS(stipples[10], STIPPLESIZE, STIPPLESIZE);
 		}
 	}
-	if (MI_NPIXELS(mi) > 2) {
+	if (MI_NPIXELS(mi) >= COLORS) {
 		/* Maybe these colors should be randomized */
-		lp->colors[0] = MI_WIN_BLACK_PIXEL(mi);
+		lp->colors[0] = MI_BLACK_PIXEL(mi);
 		lp->colors[1] = MI_PIXEL(mi, 0);	/* RED */
-		lp->colors[2] = MI_PIXEL(mi, 45);	/* BLUE */
-		lp->colors[3] = MI_PIXEL(mi, 53);	/* MAGENTA */
-		lp->colors[4] = MI_PIXEL(mi, 21);	/* GREEN */
-		lp->colors[5] = MI_PIXEL(mi, 8);	/* YELLOW */
-		lp->colors[6] = MI_PIXEL(mi, 32);	/* CYAN */
-		lp->colors[7] = MI_WIN_WHITE_PIXEL(mi);
+		lp->colors[5] = MI_PIXEL(mi, MI_NPIXELS(mi) / REALCOLORS);	/* YELLOW */
+		lp->colors[4] = MI_PIXEL(mi, 2 * MI_NPIXELS(mi) / REALCOLORS);	/* GREEN */
+		lp->colors[6] = MI_PIXEL(mi, 3 * MI_NPIXELS(mi) / REALCOLORS);	/* CYAN */
+		lp->colors[2] = MI_PIXEL(mi, 4 * MI_NPIXELS(mi) / REALCOLORS);	/* BLUE */
+		lp->colors[3] = MI_PIXEL(mi, 5 * MI_NPIXELS(mi) / REALCOLORS);	/* MAGENTA */
+		lp->colors[7] = MI_WHITE_PIXEL(mi);
 	}
 	free_list(lp);
 	lp->generation = 0;
@@ -492,7 +494,8 @@ init_loop(ModeInfo * mi)
 
 	lp->bncols = lp->ncols + 2;
 	lp->bnrows = lp->nrows + 2;
-	XClearWindow(display, MI_WINDOW(mi));
+
+	MI_CLEARWINDOW(mi);
 
 	if (lp->oldcells != NULL)
 		(void) free((void *) lp->oldcells);
