@@ -2,7 +2,7 @@
 /* marquee --- types a text-file or a text ribbon */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)marquee.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)marquee.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -35,8 +35,6 @@ static const char sccsid[] = "@(#)marquee.c	4.04 97/07/28 xlockmore";
 #define DEFAULTS "*delay: 100000 \n" \
  "*ncolors: 64 \n"
 #define SMOOTH_COLORS
-#define DEF_DELAY            100000
-#define DEF_NCOLORS          64
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
@@ -45,10 +43,19 @@ static const char sccsid[] = "@(#)marquee.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt marquee_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   marquee_description =
+{"marquee", "init_marquee", "draw_marquee", "release_marquee",
+ "refresh_marquee", "init_marquee", NULL, &marquee_opts,
+ 100000, 1, 1, 1, 64, 1.0, "",
+ "Shows messages", 0, NULL};
+
+#endif
+
 #define font_height(f) ((f==None)?8:f->ascent + f->descent)
 
 extern XFontStruct *getFont(Display * display);
-extern char *getWords(int screen);
+extern char *getWords(int screen, int screens);
 extern int  isRibbon(void);
 
 typedef struct {
@@ -79,7 +86,7 @@ font_width(XFontStruct * font, char ch)
 	int         dummy;
 	XCharStruct xcs;
 
-	XTextExtents(font, &ch, 1, &dummy, &dummy, &dummy, &xcs);
+	(void) XTextExtents(font, &ch, 1, &dummy, &dummy, &dummy, &xcs);
 	return xcs.width;
 }
 
@@ -300,15 +307,17 @@ init_marquee(ModeInfo * mi)
 	mp->t = 0;
 	mp->nonblanks = 0;
 	mp->x = 0;
-	XClearWindow(display, MI_WINDOW(mi));
+
+	MI_CLEARWINDOW(mi);
+
 	if (mode_font == None)
 		mode_font = getFont(display);
 	if (mp->gc == NULL && mode_font != None) {
 		gcv.font = mode_font->fid;
 		XSetFont(display, MI_GC(mi), mode_font->fid);
 		gcv.graphics_exposures = False;
-		gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
-		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		gcv.foreground = MI_WHITE_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
 		mp->gc = XCreateGC(display, MI_WINDOW(mi),
 				   GCForeground | GCBackground | GCGraphicsExposures | GCFont, &gcv);
 		mp->ascent = mode_font->ascent;
@@ -323,7 +332,7 @@ init_marquee(ModeInfo * mi)
 		for (i = 0; i < 256; i++)
 			char_width[i] = 8;
 	}
-	mp->words = fixup_back(getWords(MI_SCREEN(mi)));
+	mp->words = fixup_back(getWords(MI_SCREEN(mi), MI_NUM_SCREENS(mi)));
 	mp->y = 0;
 
 	if (isRibbon()) {
@@ -386,8 +395,8 @@ draw_marquee(ModeInfo * mi)
 			if (++mp->color == MI_NPIXELS(mi))
 				mp->color = 0;
 		} else
-			XSetForeground(display, mp->gc, MI_WIN_WHITE_PIXEL(mi));
-		XDrawImageString(display, MI_WINDOW(mi), mp->gc,
+			XSetForeground(display, mp->gc, MI_WHITE_PIXEL(mi));
+		(void) XDrawImageString(display, MI_WINDOW(mi), mp->gc,
 			 mp->x, mp->y + mp->ascent, mp->modwords, mp->t + 2);
 	} else {
 		switch (*ch) {
@@ -411,14 +420,14 @@ draw_marquee(ModeInfo * mi)
 				if (mp->y + mp->height > mp->win_height) {
 					XCopyArea(display, window, window, mp->gc,
 						  0, mp->height, mp->win_width, mp->y - mp->height, 0, 0);
-					XSetForeground(display, mp->gc, MI_WIN_BLACK_PIXEL(mi));
+					XSetForeground(display, mp->gc, MI_BLACK_PIXEL(mi));
 					mp->y -= mp->height;
 					XFillRectangle(display, window, mp->gc,
 					0, mp->y, mp->win_width, mp->height);
 				}
 				break;
 			case '\t':
-				XDrawString(display, window, mp->gc, mp->x, mp->y + mp->ascent,
+				(void) XDrawString(display, window, mp->gc, mp->x, mp->y + mp->ascent,
 					    space, 8 - (mp->t % 8));
 				mp->x += char_width[' '] * (8 - (mp->t % 8));
 				mp->t = ((mp->t + 8) / 8) * 8;
@@ -431,26 +440,43 @@ draw_marquee(ModeInfo * mi)
 					if (++mp->color == MI_NPIXELS(mi))
 						mp->color = 0;
 				} else
-					XSetForeground(display, mp->gc, MI_WIN_WHITE_PIXEL(mi));
+					XSetForeground(display, mp->gc, MI_WHITE_PIXEL(mi));
 				if (is_char_back_char(ch)) {
 					int         xmid = mp->x + (char_back_char_width(ch) + 1) / 2;
 
-					XDrawString(display, window, mp->gc,
+					(void) XDrawString(display, window, mp->gc,
 						    xmid - char_width[(int) (unsigned char) *ch] / 2,
 						  mp->y + mp->ascent, ch, 1);
-					XDrawString(display, window, mp->gc,
+					(void) XDrawString(display, window, mp->gc,
 						    xmid - char_width[(int) (unsigned char) *(ch + 2)] / 2,
 					      mp->y + mp->ascent, ch + 2, 1);
 					mp->x += char_back_char_width(ch);
 					mp->words += 2;
 				} else {
-					XDrawString(display, window, mp->gc,
+#ifdef USE_MB
+					int mb = (*ch & 0x80) ? 2 : 1;
+					(void) XDrawString(display, window, mp->gc,
+					   mp->x, mp->y + mp->ascent, ch, mb);
+					if (mb == 1)
+						mp->x += char_width[(int) (unsigned char) *ch];
+					else {
+						XRectangle rect;
+						XmbTextExtents(fontset, ch, 2, NULL, &rect);
+						mp->x += rect.width;
+					}
+#else
+					(void) XDrawString(display, window, mp->gc,
 					   mp->x, mp->y + mp->ascent, ch, 1);
 					mp->x += char_width[(int) (unsigned char) *ch];
+#endif
 				}
 				mp->t++;
 		}
+#ifdef USE_MB
+		mp->words += (*mp->words & 0x80) ? 2 : 1;
+#else
 		mp->words++;
+#endif
 	}
 }
 
