@@ -2,7 +2,7 @@
 /* maze --- a varying maze */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)maze.c	4.06 97/10/27 xlockmore";
+static const char sccsid[] = "@(#)maze.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -73,7 +73,7 @@ static const char sccsid[] = "@(#)maze.c	4.06 97/10/27 xlockmore";
 #define HACK_DRAW draw_maze
 #define maze_opts xlockmore_opts
 #define DEFAULTS "*delay: 1000 \n" \
- "*cycles: 300 \n" \
+ "*cycles: 3000 \n" \
  "*size: -40 \n" \
  "*ncolors: 64 \n"
 #define DEF_BITMAP ""
@@ -85,6 +85,15 @@ static const char sccsid[] = "@(#)maze.c	4.06 97/10/27 xlockmore";
 ModeSpecOpt maze_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   maze_description =
+{"maze", "init_maze", "draw_maze", "release_maze",
+ "refresh_maze", "init_maze", NULL, &maze_opts,
+ 1000, 1, 3000, -40, 64, 1.0, "",
+ "Shows a random maze and a depth first search solution", 0, NULL};
+
+#endif
+
 #include "bitmaps/gray1.xbm"
 
 /* aliases for vars defined in the bitmap file */
@@ -136,17 +145,18 @@ typedef struct {
 	int         width, height;
 	int         xs, ys, logo_size_x, logo_size_y;
 	int         solving, current_path, stage;
-	/*int         cycles; */
+	int         cycles;
 	unsigned short *maze;
 	paths      *move_list;
 	paths      *save_path, *path;
 	GC          grayGC;
 	Pixmap      graypix;
-  XImage     *logo;
-  Colormap    cmap;
-  unsigned long black, white;
-  int         graphics_format;
-  GC          backGC;
+	XImage     *logo;
+	Colormap    cmap;
+	unsigned long black;
+	int         graphics_format;
+	GC          backGC;
+	int         time;
 } mazestruct;
 
 static mazestruct *mazes = NULL;
@@ -225,8 +235,8 @@ initialize_maze(ModeInfo * mi)
 	register int i, j, wall;
 
 	if (MI_NPIXELS(mi) <= 2) {
-		XSetForeground(display, mp->backGC, MI_WIN_WHITE_PIXEL(mi));
-		XSetForeground(display, mp->grayGC, MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(display, mp->backGC, MI_WHITE_PIXEL(mi));
+		XSetForeground(display, mp->grayGC, MI_WHITE_PIXEL(mi));
 	} else {
 		i = NRAND(MI_NPIXELS(mi));
 		XSetForeground(display, mp->backGC, MI_PIXEL(mi, i));
@@ -532,12 +542,12 @@ draw_maze_border(ModeInfo * mi)
 	}
 
 	if (mp->logo_x != -1) {
-		XPutImage(display, window, gc, mp->logo,
+		(void) XPutImage(display, window, gc, mp->logo,
 			  0, 0,
 			  mp->xb + mp->xs * mp->logo_x +
-			  (mp->xs * mp->logo_size_x - mp->logo->width + 1) / 2,
+			(mp->xs * mp->logo_size_x - mp->logo->width + 1) / 2,
 			  mp->yb + mp->ys * mp->logo_y +
-			  (mp->ys * mp->logo_size_y - mp->logo->height + 1) / 2,
+		       (mp->ys * mp->logo_size_y - mp->logo->height + 1) / 2,
 			  mp->logo->width, mp->logo->height);
 	}
 	draw_solid_square(mi, gc, mp->start_x, mp->start_y, mp->start_dir);
@@ -709,44 +719,43 @@ enter_square(ModeInfo * mi, int n)
 static void
 init_stuff(ModeInfo * mi)
 {
-  Display    *display = MI_DISPLAY(mi);
-  Window      window = MI_WINDOW(mi);
-  mazestruct *mp = &mazes[MI_SCREEN(mi)];
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	mazestruct *mp = &mazes[MI_SCREEN(mi)];
 
-  if (!mp->logo)
-    getImage(mi, &mp->logo, ICON_WIDTH, ICON_HEIGHT, ICON_BITS,
+	if (!mp->logo)
+		getImage(mi, &mp->logo, ICON_WIDTH, ICON_HEIGHT, ICON_BITS,
 #if defined( USE_XPM ) || defined( USE_XPMINC )
-       DEFAULT_XPM, ICON_NAME,
+			 DEFAULT_XPM, ICON_NAME,
 #endif
-       &mp->graphics_format, &mp->cmap,
-       &mp->black, &mp->white);
-  if (mp->cmap != None) {
-    setColormap(display, window, mp->cmap, MI_WIN_IS_INWINDOW(mi));
-    if (mp->backGC == None) {
-      XGCValues   xgcv;
+			 &mp->graphics_format, &mp->cmap, &mp->black);
+	if (mp->cmap != None) {
+		setColormap(display, window, mp->cmap, MI_WIN_IS_INWINDOW(mi));
+		if (mp->backGC == None) {
+			XGCValues   xgcv;
 
-      xgcv.background = mp->black;
-      mp->backGC = XCreateGC(display, window, GCBackground, &xgcv);
-    }
-  } else {
-    mp->black = MI_WIN_BLACK_PIXEL(mi);
-    mp->backGC = MI_GC(mi);
-  }
+			xgcv.background = mp->black;
+			mp->backGC = XCreateGC(display, window, GCBackground, &xgcv);
+		}
+	} else {
+		mp->black = MI_BLACK_PIXEL(mi);
+		mp->backGC = MI_GC(mi);
+	}
 }
 
 static void
 free_stuff(Display * display, mazestruct * mp)
 {
-  if (mp->cmap != None) {
-    XFreeColormap(display, mp->cmap);
-    if (mp->backGC != None) {
-      XFreeGC(display, mp->backGC);
-      mp->backGC = None;
-    }
-    mp->cmap = None;
-  } else
-    mp->backGC = None;
-  destroyImage(&mp->logo, &mp->graphics_format);
+	if (mp->cmap != None) {
+		XFreeColormap(display, mp->cmap);
+		if (mp->backGC != None) {
+			XFreeGC(display, mp->backGC);
+			mp->backGC = None;
+		}
+		mp->cmap = None;
+	} else
+		mp->backGC = None;
+	destroyImage(&mp->logo, &mp->graphics_format);
 }
 
 void
@@ -762,7 +771,7 @@ init_maze(ModeInfo * mi)
 	}
 	mp = &mazes[MI_SCREEN(mi)];
 
-  init_stuff(mi);
+	init_stuff(mi);
 
 	mp->width = MI_WIN_WIDTH(mi);
 	mp->height = MI_WIN_HEIGHT(mi);
@@ -781,10 +790,10 @@ init_maze(ModeInfo * mi)
 	}
 	mp->solving = 0;
 	mp->stage = 0;
-	/*mp->time = 0;
-	   mp->cycles = MI_CYCLES(mi);
-	   if (mp->cycles < 4)
-	   mp->cycles = 4; */
+	mp->time = 0;
+	mp->cycles = MI_CYCLES(mi);
+	if (mp->cycles < 4)
+		mp->cycles = 4;
 }
 
 void
@@ -798,9 +807,7 @@ draw_maze(ModeInfo * mi)
 	}
 	switch (mp->stage) {
 		case 0:
-			XSetForeground(MI_DISPLAY(mi), mp->backGC, mp->black);
-			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), mp->backGC,
-           0, 0, mp->width, mp->height);
+			MI_CLEARWINDOWCOLORMAP(mi, mp->backGC, mp->black);
 
 			set_maze_sizes(mi);
 			initialize_maze(mi);
@@ -816,18 +823,16 @@ draw_maze(ModeInfo * mi)
 			mp->stage++;
 			break;
 		case 3:
-			MI_PAUSE(mi) = 1000000;
-			/*if (++mp->time > mp->cycles / 4) */
-			mp->stage++;
+			if (++mp->time > mp->cycles / 4)
+				mp->stage++;
 			break;
 		case 4:
 			solve_maze(mi);
 			mp->stage++;
 			break;
 		case 5:
-			MI_PAUSE(mi) = 2000000;
-			/*if (++mp->time > mp->cycles) */
-			init_maze(mi);
+			if (++mp->time > 3 * mp->cycles / 4)
+				init_maze(mi);
 			break;
 	}
 }				/*  end of draw_maze() */
@@ -840,14 +845,14 @@ release_maze(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
 			mazestruct *mp = &mazes[screen];
-      Display    *display = MI_DISPLAY(mi);
+			Display    *display = MI_DISPLAY(mi);
 
 			free_maze(mp);
 			if (mp->grayGC != NULL)
 				XFreeGC(display, mp->grayGC);
 			if (mp->graypix != None)
 				XFreePixmap(display, mp->graypix);
-      free_stuff(display, mp);
+			free_stuff(display, mp);
 		}
 		(void) free((void *) mazes);
 		mazes = NULL;
