@@ -5,7 +5,7 @@
  */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)nose.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)nose.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -63,6 +63,15 @@ static const char sccsid[] = "@(#)nose.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt nose_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   nose_description =
+{"nose", "init_nose", "draw_nose", "release_nose",
+ "refresh_nose", "init_nose", NULL, &nose_opts,
+ 100000, 1, 1, 1, 64, 1.0, "",
+ "Shows a man with a big nose runs around spewing out messages", 0, NULL};
+
+#endif
+
 #include "bitmaps/nose-hat.xbm"
 #include "bitmaps/nose-hatd.xbm"
 #include "bitmaps/nose-facef.xbm"
@@ -92,7 +101,7 @@ ModeSpecOpt nose_opts =
 #define FREEZE 2
 
 extern XFontStruct *getFont(Display * display);
-extern char *getWords(int screen);
+extern char *getWords(int screen, int screens);
 extern int  isRibbon(void);
 
 typedef struct {
@@ -108,7 +117,7 @@ typedef struct {
 	int         tinymode;	/* walking or getting passwd */
 	int         length, dir, lastdir;
 	int         up;
-	long        nose_pause;
+	int         busyLoop;
 	int         frames;
 	int         state;
 	Bool        talking;
@@ -127,6 +136,7 @@ static XFontStruct *mode_font = None;
 #define FRONT 020
 #define X_INCR 3
 #define Y_INCR 2
+#define YELLOW (MI_NPIXELS(mi) / 6)
 
 #define COPY(d,g,c,p,np,x,y,w,h) XSetForeground(d,g,c);\
 XSetStipple(d,g,p); XSetTSOrigin(d,g,x,y);\
@@ -142,25 +152,25 @@ pickClothes(ModeInfo * mi)
 	XGCValues   gcv;
 	Pixmap      face_pix, hat_pix, shoe_pix, shoel_pix, shoer_pix;
 	unsigned long hat_color = (MI_NPIXELS(mi) <= 2) ?
-	MI_WIN_WHITE_PIXEL(mi) : MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
+	MI_WHITE_PIXEL(mi) : MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
 	unsigned long face_color = (MI_NPIXELS(mi) <= 2) ?
-	MI_WIN_WHITE_PIXEL(mi) : MI_PIXEL(mi, (NUMCOLORS / 5 + 1));
+	MI_WHITE_PIXEL(mi) : MI_PIXEL(mi, (YELLOW));  /* Racism? */
 	unsigned long shoe_color = (MI_NPIXELS(mi) <= 2) ?
-	MI_WIN_WHITE_PIXEL(mi) : MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
+	MI_WHITE_PIXEL(mi) : MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
 	int         i;
 
 	gcv.graphics_exposures = False;
-	gcv.foreground = MI_WIN_BLACK_PIXEL(mi);
-	gcv.background = MI_WIN_BLACK_PIXEL(mi);
+	gcv.foreground = MI_BLACK_PIXEL(mi);
+	gcv.background = MI_BLACK_PIXEL(mi);
 	for (i = 0; i < PIXMAPS; i++) {
 		np->position[i] = XCreatePixmap(display, window, PIXMAP_SIZE, PIXMAP_SIZE,
-						MI_WIN_DEPTH(mi));
+						MI_DEPTH(mi));
 		np->noseGC[i] = XCreateGC(display, np->position[i],
-					  GCForeground | GCBackground | GCGraphicsExposures, &gcv);
+		    GCForeground | GCBackground | GCGraphicsExposures, &gcv);
 		XFillRectangle(display, np->position[i], np->noseGC[i],
 			       0, 0, PIXMAP_SIZE, PIXMAP_SIZE);
 	}
-	XSetBackground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+	XSetBackground(display, gc, MI_BLACK_PIXEL(mi));
 	XSetFillStyle(display, gc, FillStippled);
 	/* DOWN NOSE GUY */
 	hat_pix = XCreateBitmapFromData(display, window,
@@ -184,7 +194,7 @@ pickClothes(ModeInfo * mi)
 	     nose_hat_down_width, nose_hat_down_height);
 	if (MI_NPIXELS(mi) <= 2) {
 		XSetFillStyle(display, gc, FillSolid);
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		XFillRectangle(display, np->position[D], gc,
 			       0, nose_hat_down_height + 6, PIXMAP_SIZE, 1);
 		XSetFillStyle(display, gc, FillStippled);
@@ -207,7 +217,7 @@ pickClothes(ModeInfo * mi)
 	     nose_hat_width, nose_hat_height);
 	if (MI_NPIXELS(mi) <= 2) {
 		XSetFillStyle(display, gc, FillSolid);
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		XFillRectangle(display, np->position[F], gc,
 			       0, nose_hat_height + 3, PIXMAP_SIZE, 1);
 		XSetFillStyle(display, gc, FillStippled);
@@ -229,7 +239,7 @@ pickClothes(ModeInfo * mi)
 	     nose_hat_width, nose_hat_height);
 	if (MI_NPIXELS(mi) <= 2) {
 		XSetFillStyle(display, gc, FillSolid);
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		XFillRectangle(display, np->position[LF], gc,
 			       0, nose_hat_height + 3, PIXMAP_SIZE, 1);
 		XSetFillStyle(display, gc, FillStippled);
@@ -250,7 +260,7 @@ pickClothes(ModeInfo * mi)
 	     nose_hat_width, nose_hat_height);
 	if (MI_NPIXELS(mi) <= 2) {
 		XSetFillStyle(display, gc, FillSolid);
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		XFillRectangle(display, np->position[RF], gc,
 			       0, nose_hat_height + 3, PIXMAP_SIZE, 1);
 		XSetFillStyle(display, gc, FillStippled);
@@ -272,7 +282,7 @@ pickClothes(ModeInfo * mi)
 	     nose_hat_width, nose_hat_height);
 	if (MI_NPIXELS(mi) <= 2) {
 		XSetFillStyle(display, gc, FillSolid);
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		XFillRectangle(display, np->position[L], gc,
 			       0, nose_hat_height + 3, PIXMAP_SIZE, 1);
 		XSetFillStyle(display, gc, FillStippled);
@@ -294,7 +304,7 @@ pickClothes(ModeInfo * mi)
 	     nose_hat_width, nose_hat_height);
 	if (MI_NPIXELS(mi) <= 2) {
 		XSetFillStyle(display, gc, FillSolid);
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		XFillRectangle(display, np->position[LSTEP], gc,
 			       0, nose_hat_height + 3, PIXMAP_SIZE, 1);
 		XSetFillStyle(display, gc, FillStippled);
@@ -317,7 +327,7 @@ pickClothes(ModeInfo * mi)
 	     nose_hat_width, nose_hat_height);
 	if (MI_NPIXELS(mi) <= 2) {
 		XSetFillStyle(display, gc, FillSolid);
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		XFillRectangle(display, np->position[R], gc,
 			       0, nose_hat_height + 3, PIXMAP_SIZE, 1);
 		XSetFillStyle(display, gc, FillStippled);
@@ -339,7 +349,7 @@ pickClothes(ModeInfo * mi)
 	     nose_hat_width, nose_hat_height);
 	if (MI_NPIXELS(mi) <= 2) {
 		XSetFillStyle(display, gc, FillSolid);
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 		XFillRectangle(display, np->position[RSTEP], gc,
 			       0, nose_hat_height + 3, PIXMAP_SIZE, 1);
 		XSetFillStyle(display, gc, FillStippled);
@@ -419,7 +429,7 @@ think(ModeInfo * mi)
 	if (LRAND() & 1)
 		walk(mi, FRONT);
 	if (LRAND() & 1) {
-		np->words = getWords(MI_SCREEN(mi));
+		np->words = getWords(MI_SCREEN(mi), MI_NUM_SCREENS(mi));
 		return 1;
 	}
 	return 0;
@@ -523,17 +533,17 @@ talk(ModeInfo * mi, Bool force_erase)
 
 	/* now print each string in reverse order (start at bottom of box) */
 	for (Z = 0; Z < height; Z++) {
-		XDrawString(display, window, np->text_fg_gc, np->s.x + 15, np->s.y + Y,
-			    args[Z], strlen(args[Z]));
+		(void) XDrawString(display, window, np->text_fg_gc,
+			np->s.x + 15, np->s.y + Y, args[Z], strlen(args[Z]));
 		Y += font_height(mode_font);
 	}
-	np->nose_pause = (total / 15) * 1000000;
-	if (np->nose_pause < 3000000)
-		np->nose_pause = 3000000;
+	np->busyLoop = (total / 15) * 10;
+	if (np->busyLoop < 30)
+		np->busyLoop = 30;
 	np->state = TALK;
 }
 
-static unsigned long
+static int
 look(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
@@ -547,7 +557,7 @@ look(ModeInfo * mi)
 		i = (LRAND() & 1) ? D : F;
 		XCopyArea(display, np->position[i], window, np->noseGC[i],
 			  0, 0, PIXMAP_SIZE, PIXMAP_SIZE, np->x, np->y);
-		return 100000L;
+		return 3;
 	}
 	if (!NRAND(5))
 		return 0;
@@ -555,14 +565,14 @@ look(ModeInfo * mi)
 		i = (LRAND() & 1) ? LF : RF;
 		XCopyArea(display, np->position[i], window, np->noseGC[i],
 			  0, 0, PIXMAP_SIZE, PIXMAP_SIZE, np->x, np->y);
-		return 100000L;
+		return 3;
 	}
 	if (!NRAND(5))
 		return 0;
 	i = (LRAND() & 1) ? L : R;
 	XCopyArea(display, np->position[i], window, np->noseGC[i],
 		  0, 0, PIXMAP_SIZE, PIXMAP_SIZE, np->x, np->y);
-	return 100000L;
+	return 3;
 }
 
 static void
@@ -578,11 +588,11 @@ move(ModeInfo * mi)
 			talk(mi, False);	/* sets timeout to itself */
 			return;
 		}
-		if (!NRAND(3) && (np->nose_pause = look(mi))) {
+		if (!NRAND(3) && (np->busyLoop = look(mi))) {
 			np->state = MOVE;
 			return;
 		}
-		np->nose_pause = 2000 + 10 * NRAND(100);
+		np->busyLoop = 3 + NRAND(3);
 		do {
 			if (!tries)
 				np->length = np->width / 100 + NRAND(90), tries = 8;
@@ -659,8 +669,9 @@ init_nose(ModeInfo * mi)
 	np->xs = PIXMAP_SIZE;
 	np->ys = PIXMAP_SIZE;
 
-	XClearWindow(display, window);
-	XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+	MI_CLEARWINDOW(mi);
+
+	XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 
 	/* don't want any exposure events from XCopyPlane */
 	XSetGraphicsExposures(display, gc, False);
@@ -669,17 +680,17 @@ init_nose(ModeInfo * mi)
 		mode_font = getFont(display);
 	if (np->noseGC[0] == NULL)
 		pickClothes(mi);
-	np->words = getWords(MI_SCREEN(mi));
+	np->words = getWords(MI_SCREEN(mi), MI_NUM_SCREENS(mi));
 	if (np->text_fg_gc == NULL && mode_font != None) {
 		gcv.font = mode_font->fid;
 		XSetFont(display, gc, mode_font->fid);
 		gcv.graphics_exposures = False;
-		gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
-		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		gcv.foreground = MI_WHITE_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
 		np->text_fg_gc = XCreateGC(display, window,
 					   GCForeground | GCBackground | GCGraphicsExposures | GCFont, &gcv);
-		gcv.foreground = MI_WIN_BLACK_PIXEL(mi);
-		gcv.background = MI_WIN_WHITE_PIXEL(mi);
+		gcv.foreground = MI_BLACK_PIXEL(mi);
+		gcv.background = MI_WHITE_PIXEL(mi);
 		np->text_bg_gc = XCreateGC(display, window,
 					   GCForeground | GCBackground | GCGraphicsExposures | GCFont, &gcv);
 	}
@@ -706,7 +717,10 @@ draw_nose(ModeInfo * mi)
 {
 	nosestruct *np = &noses[MI_SCREEN(mi)];
 
-	np->nose_pause = 0;	/* use default if not changed here */
+	if (np->busyLoop > 0) {
+		np->busyLoop--;
+		return;
+	}
 	switch (np->state) {
 		case MOVE:
 			move(mi);
@@ -715,8 +729,6 @@ draw_nose(ModeInfo * mi)
 			talk(mi, 0);
 			break;
 	}
-	/* TODO: This is wrong for multi-screens *** */
-	MI_PAUSE(mi) = np->nose_pause;	/* pass back desired pause time */
 }
 
 void
