@@ -2,7 +2,7 @@
 /* pacman --- Mr. Pacman and his ghost friends */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)pacman.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)pacman.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -43,6 +43,7 @@ static const char sccsid[] = "@(#)pacman.c	4.04 97/07/28 xlockmore";
 #define DEFAULTS "*delay: 100000 \n" \
  "*count: 10 \n" \
  "*size: 0 \n"
+ "*ncolors: 5 \n"
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
@@ -51,6 +52,15 @@ static const char sccsid[] = "@(#)pacman.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt pacman_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   pacman_description =
+{"pacman", "init_pacman", "draw_pacman", "release_pacman",
+ "refresh_pacman", "init_pacman", NULL, &pacman_opts,
+ 100000, 10, 1, 0, 5, 64, 1.0, "",
+ "Shows Pacman(tm)", 0, NULL};
+
+#endif
+
 /* aliases for vars defined in the bitmap file */
 #define CELL_WIDTH   image_width
 #define CELL_HEIGHT    image_height
@@ -71,9 +81,9 @@ ModeSpecOpt pacman_opts =
 #define LB   0x0100
 #define ALL  0x1111
 
-#define YELLOW 9
-#define GREEN 23
-#define BLUE 45
+#define YELLOW (MI_NPIXELS(mi) / 6)  
+#define GREEN (23 * MI_NPIXELS(mi) / 64)  
+#define BLUE (45 * MI_NPIXELS(mi) / 64)  
 
 typedef struct {
 	int         col, row;
@@ -88,9 +98,11 @@ typedef struct {
 	int         width, height;
 	int         nrows, ncols;
 	int         xs, ys, xb, yb;
-	int         inc;
+	int         incx, incy;
 	GC          stippledGC;
 	Pixmap      ghostPixmap;
+	int ghostWidth, ghostHeight;
+  int graphics_format;
 	beingstruct pacman;
 	beingstruct *ghosts;
 	int         nghosts;
@@ -101,7 +113,6 @@ typedef struct {
 } pacmangamestruct;
 
 static pacmangamestruct *pacmangames = NULL;
-static int  icon_width, icon_height;
 
 #if DEFUNCT
 static void
@@ -112,7 +123,10 @@ clearcorners(ModeInfo * mi)
 	Window      window = MI_WINDOW(mi);
 	pacmangamestruct *pp = &pacmangames[MI_SCREEN(mi)];
 
-	XSetForeground(display, gc, MI_PIXEL(mi, GREEN));
+	if (MI_NPIXELS(mi) > 2)
+	  XSetForeground(display, gc, MI_PIXEL(mi, GREEN));
+		else
+			XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 	if ((pp->pacman.nextrow == 0) && (pp->pacman.nextcol == 0)) {
 		XFillRectangle(display, window, gc, 0, 0,
 		((pp->width / pp->ncols) / 2), ((pp->width / pp->ncols) / 2));
@@ -146,8 +160,6 @@ clearcorners(ModeInfo * mi)
 static void
 repopulate(ModeInfo * mi)
 {
-	Display    *display = MI_DISPLAY(mi);
-	Window      window = MI_WINDOW(mi);
 	pacmangamestruct *pp = &pacmangames[MI_SCREEN(mi)];
 	int         ghost;
 
@@ -159,7 +171,7 @@ repopulate(ModeInfo * mi)
 
 #endif
 
-	XClearWindow(display, window);
+	MI_CLEARWINDOW(mi);
 
 	for (ghost = 0; ghost < pp->nghosts; ghost++) {
 		do {
@@ -277,22 +289,19 @@ movepac(ModeInfo * mi)
 				pp->yb;
 		}
 	}
-	for (delta.x = pp->inc, delta.y = pp->inc;
-	     delta.x <= pp->xs + pp->inc - 1 &&
-	     delta.y <= pp->ys + pp->inc - 1;
-	     delta.x += pp->inc, delta.y += pp->inc) {
+	for (delta.x = pp->incx, delta.y = pp->incy;
+	     (delta.x < pp->xs + pp->incx) || (delta.y < pp->ys + pp->incy);
+	     delta.x += pp->incx, delta.y += pp->incy) {
 		if (delta.x > pp->xs)
 			delta.x = pp->xs;
 		if (delta.y > pp->ys)
 			delta.y = pp->ys;
-		p.cf = pp->pacman.col * pp->xs + delta.x * p.cfactor +
-			pp->xb;
-		p.rf = pp->pacman.row * pp->ys + delta.y * p.rfactor +
-			pp->yb;
+		p.cf = pp->pacman.col * pp->xs + delta.x * p.cfactor + pp->xb;
+		p.rf = pp->pacman.row * pp->ys + delta.y * p.rfactor + pp->yb;
 
 		dir = (ABS(p.cfactor) * (2 - p.cfactor) +
 		       ABS(p.rfactor) * (1 + p.rfactor)) % 4;
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 #ifdef FLASH
 		XFillRectangle(display, window, gc,
 			       p.oldcf, p.oldrf,
@@ -302,7 +311,11 @@ movepac(ModeInfo * mi)
 			    p.oldcf, p.oldrf, pp->xs, pp->ys);
 #endif
 		XSetTSOrigin(display, pp->stippledGC, p.cf, p.rf);
-		XSetForeground(display, pp->stippledGC, MI_PIXEL(mi, YELLOW));
+		if (MI_NPIXELS(mi) > 2)
+			XSetForeground(display, pp->stippledGC, MI_PIXEL(mi, YELLOW));
+		else
+			XSetForeground(display, pp->stippledGC, MI_WHITE_PIXEL(mi));
+
 		XSetStipple(display, pp->stippledGC,
 			    pp->pacmanPixmap[dir][pp->pacman.mouthstage]);
 #ifdef FLASH
@@ -327,12 +340,15 @@ movepac(ModeInfo * mi)
 					delta.x * g[ghost].cfactor + pp->xb;
 				g[ghost].rf = pp->ghosts[ghost].row * pp->ys +
 					delta.y * g[ghost].rfactor + pp->yb;
-				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 				if (pp->pixelmode) {
 					XFillRectangle(display, window, gc,
 					      g[ghost].oldcf, g[ghost].oldrf,
 						       pp->xs, pp->ys);
-					XSetForeground(display, gc, MI_PIXEL(mi, BLUE));
+					if (MI_NPIXELS(mi) > 2)
+						XSetForeground(display, gc, MI_PIXEL(mi, BLUE));
+					else
+						XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 					XFillRectangle(display, window, gc,
 						       g[ghost].cf, g[ghost].rf, pp->xs, pp->ys);
 					XFlush(display);
@@ -346,7 +362,10 @@ movepac(ModeInfo * mi)
 						    g[ghost].oldcf, g[ghost].oldrf, pp->xs, pp->ys);
 #endif
 					XSetTSOrigin(display, pp->stippledGC, g[ghost].cf, g[ghost].rf);
-					XSetForeground(display, pp->stippledGC, MI_PIXEL(mi, BLUE));
+					if (MI_NPIXELS(mi) > 2)
+						XSetForeground(display, pp->stippledGC, MI_PIXEL(mi, BLUE));
+					else
+						XSetForeground(display, pp->stippledGC, MI_WHITE_PIXEL(mi));
 					XSetStipple(display, pp->stippledGC, pp->ghostPixmap);
 #ifdef FLASH
 					XSetFillStyle(display, pp->stippledGC, FillStippled);
@@ -363,6 +382,7 @@ movepac(ModeInfo * mi)
 		}
 		XFlush(display);
 	}
+
 #if 0
 	clearcorners(mi);
 #endif
@@ -387,7 +407,7 @@ movepac(ModeInfo * mi)
 			     (pp->ghosts[ghost].row == pp->pacman.nextrow) &&
 			     (pp->ghosts[ghost].col == pp->pacman.nextcol))) {
 				pp->ghosts[ghost].dead = 1;
-				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 				/*XFillRectangle(display, window, gc,
 				   pp->ghosts[ghost].col * pp->xs + pp->xb,
 				   pp->ghosts[ghost].row * pp->ys + pp->yb,
@@ -436,35 +456,37 @@ init_pacman(ModeInfo * mi)
 		if ((pacmangames = (pacmangamestruct *) calloc(MI_NUM_SCREENS(mi),
 					 sizeof (pacmangamestruct))) == NULL)
 			return;
-		icon_width = CELL_WIDTH;
-		icon_height = CELL_HEIGHT;
 	}
 	pp = &pacmangames[MI_SCREEN(mi)];
+	pp->width = MI_WIN_WIDTH(mi);
+	pp->height = MI_WIN_HEIGHT(mi);
 	if (pp->stippledGC == None) {
-		gcv.foreground = MI_WIN_BLACK_PIXEL(mi);
-		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		gcv.foreground = MI_BLACK_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
 		if ((pp->stippledGC = XCreateGC(display, window,
 				 GCForeground | GCBackground, &gcv)) == None)
 			return;
-	}
-	pp->width = MI_WIN_WIDTH(mi);
-	pp->height = MI_WIN_HEIGHT(mi);
-	if (size == 0 ||
-	 MINGRIDSIZE * size > pp->width || MINGRIDSIZE * size > pp->height) {
-		if (pp->width > MINGRIDSIZE * icon_width &&
-		    pp->height > MINGRIDSIZE * icon_height) {
-			pp->xs = icon_width;
-			pp->ys = icon_height;
-			pp->pixelmode = 0;
+		pp->ghostWidth = CELL_WIDTH;
+		pp->ghostHeight = CELL_HEIGHT;
+		if (size == 0) {
 			if (pp->ghostPixmap == None) {
-				pp->ghostPixmap = XCreateBitmapFromData(display, window,
-				(char *) CELL_BITS, CELL_WIDTH, CELL_HEIGHT);
+    		getPixmap(display, window, CELL_WIDTH, CELL_HEIGHT, CELL_BITS,
+    			&(pp->ghostWidth), &(pp->ghostHeight), &(pp->ghostPixmap),
+    			&(pp->graphics_format));
+				pp->xs = pp->ghostWidth;
+				pp->ys = pp->ghostHeight;
 			}
-			pp->inc = pp->xs / 10 + 1;
-		} else {
+		}
+	}
+	if (size == 0) {
+   if (MINGRIDSIZE * pp->ghostWidth > pp->width ||
+       MINGRIDSIZE * pp->ghostHeight > pp->height) {
 			pp->pixelmode = 1;
 			pp->xs = pp->ys = MAX(MINSIZE, MIN(pp->width, pp->height) / MINGRIDSIZE);
-			pp->inc = pp->xs / 10 + 1;
+		} else {
+			pp->pixelmode = 0;
+			pp->xs = pp->ghostWidth;
+			pp->ys = pp->ghostHeight;
 		}
 	} else {
 		pp->pixelmode = 1;
@@ -477,8 +499,9 @@ init_pacman(ModeInfo * mi)
 			pp->ys = MIN(size, MAX(MINSIZE, MIN(pp->width, pp->height) /
 					       MINGRIDSIZE));
 		pp->xs = pp->ys;
-		pp->inc = pp->xs / 10 + 1;
 	}
+	pp->incx = pp->xs / 10 + 1;
+	pp->incy = pp->ys / 10 + 1;
 	pp->ncols = MAX(pp->width / pp->xs, 2);
 	pp->nrows = MAX(pp->height / pp->ys, 2);
 	pp->xb = pp->width - pp->ncols * pp->xs;
@@ -535,7 +558,7 @@ init_pacman(ModeInfo * mi)
 	pp->pacman.row = NRAND(pp->nrows);
 	pp->pacman.col = NRAND(pp->ncols);
 
-	XClearWindow(display, window);
+	MI_CLEARWINDOW(mi);
 
 	pp->pacman.mouthstage = MAXMOUTH - 1;
 	for (ghost = 0; ghost < pp->nghosts; ghost++) {
