@@ -2,7 +2,7 @@
 /* puzzle --- the familiar Sam Loyd puzzle */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)puzzle.c        4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)puzzle.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -59,6 +59,15 @@ cartoon.c has the same disease.
 ModeSpecOpt puzzle_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   puzzle_description =
+{"puzzle", "init_puzzle", "draw_puzzle", "release_puzzle",
+ "init_puzzle", "init_puzzle", NULL, &puzzle_opts,
+ 10000, 250, 1, 1, 64, 1.0, "",
+ "Shows a puzzle being scrambled and then solved", 0, NULL};
+
+#endif
+
 #include <time.h>
 
 #define PUZZLE_WIDTH   image_width
@@ -73,13 +82,14 @@ ModeSpecOpt puzzle_opts =
 #define DEFAULT_XPM 1
 #endif
 
+  /*int storedmoves, *moves, *position, space;  To keep track of puzzle */
 typedef struct {
 	int         excount;
 	int        *fixbuff;
 	XPoint      count, boxsize, windowsize;
 	XPoint      usablewindow, offsetwindow;
 	XPoint      randompos, randpos;
-	unsigned long black, white;
+	unsigned long black;
 	int         row, col, nextrow, nextcol, nextbox;
 	int         incrementOfMove;
 	int         lastbox;
@@ -122,7 +132,7 @@ font_width(XFontStruct * font, char ch)
 	int         dummy;
 	XCharStruct xcs;
 
-	XTextExtents(font, &ch, 1, &dummy, &dummy, &dummy, &xcs);
+	(void) XTextExtents(font, &ch, 1, &dummy, &dummy, &dummy, &xcs);
 	return xcs.width;
 }
 
@@ -142,15 +152,15 @@ NumberScreen(ModeInfo * mi)
 		gcv.font = mode_font->fid;
 		XSetFont(display, MI_GC(mi), mode_font->fid);
 		gcv.graphics_exposures = False;
-		gcv.foreground = MI_WIN_WHITE_PIXEL(mi);
-		gcv.background = MI_WIN_BLACK_PIXEL(mi);
+		gcv.foreground = MI_WHITE_PIXEL(mi);
+		gcv.background = MI_BLACK_PIXEL(mi);
 		pp->gc = XCreateGC(display, window,
 				   GCForeground | GCBackground | GCGraphicsExposures | GCFont, &gcv);
 		pp->ascent = mode_font->ascent;
 		pp->fontHeight = font_height(mode_font);
 		pp->fontWidth = font_width(mode_font, '5');
 	}
-	XSetForeground(display, pp->gc, MI_WIN_WHITE_PIXEL(mi));
+	XSetForeground(display, pp->gc, MI_WHITE_PIXEL(mi));
 
 	{
 		XPoint      pos, letter;
@@ -169,7 +179,7 @@ NumberScreen(ModeInfo * mi)
 					if (pp->boxsize.x > 2 * pp->fontWidth &&
 					    pp->boxsize.y > pp->fontHeight) {
 						(void) sprintf(buf, "%d", count);
-						XDrawString(display, window, pp->gc,
+						(void) XDrawString(display, window, pp->gc,
 							    pos.x + letter.x - letterOffset * digitOffset +
 							    pp->randompos.x,
 							    pos.y + letter.y + pp->randompos.y, buf, digitOffset);
@@ -273,7 +283,7 @@ setupmovedelta(ModeInfo * mi)
 	 */
 	/* UnmapNotify VisibilityPartiallyObscured)) */
 	if (pp->box) {
-		XDestroyImage(pp->box);
+		(void) XDestroyImage(pp->box);
 		pp->box = NULL;
 	} {
 		XWindowAttributes wa, pwa =
@@ -377,7 +387,7 @@ wrapupmovedelta(ModeInfo * mi)
 	puzzlestruct *pp = &puzzs[MI_SCREEN(mi)];
 
 	if (pp->box) {
-		XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC, pp->box, 0, 0,
+		(void) XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC, pp->box, 0, 0,
 			  pp->col * pp->boxsize.x + pp->randompos.x + 1,
 			  pp->row * pp->boxsize.y + pp->randompos.y + 1,
 			  pp->boxsize.x - 2, pp->boxsize.y - 2);
@@ -386,7 +396,7 @@ wrapupmovedelta(ModeInfo * mi)
 		pp->row = pp->nextrow;
 		pp->col = pp->nextcol;
 
-		XDestroyImage(pp->box);
+		(void) XDestroyImage(pp->box);
 		pp->box = NULL;
 	}
 }
@@ -402,7 +412,7 @@ moveboxdelta(ModeInfo * mi)
 
 	if (pp->Lp <= pp->lengthOfMove) {
 		if (pp->box) {
-			XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC, pp->box,
+			(void) XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC, pp->box,
 				  0, 0, cf + 1, rf + 1, pp->boxsize.x - 2, pp->boxsize.y - 2);
 			XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), pp->backGC,
 				       cf + pp->cbs - 1, rf + pp->rbs - 1, pp->cbw + 2, pp->rbw + 2);
@@ -428,8 +438,7 @@ init_stuff(ModeInfo * mi)
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 			 DEFAULT_XPM, PUZZLE_NAME,
 #endif
-			 &pp->graphics_format, &pp->cmap,
-			 &pp->black, &pp->white);
+			 &pp->graphics_format, &pp->cmap, &pp->black);
 	if (pp->cmap != None) {
 		setColormap(display, window, pp->cmap, MI_WIN_IS_INWINDOW(mi));
 		if (pp->backGC == None) {
@@ -439,7 +448,7 @@ init_stuff(ModeInfo * mi)
 			pp->backGC = XCreateGC(display, window, GCBackground, &xgcv);
 		}
 	} else {
-		pp->black = MI_WIN_BLACK_PIXEL(mi);
+		pp->black = MI_BLACK_PIXEL(mi);
 		pp->backGC = MI_GC(mi);
 	}
 }
@@ -498,31 +507,12 @@ init_puzzle(ModeInfo * mi)
 	pp->prev = 0;
 
 	if (pp->box) {
-		XDestroyImage(pp->box);
+		(void) XDestroyImage(pp->box);
 		pp->box = NULL;
 	}
-	if (!pp->logo)
-		getImage(mi, &pp->logo, PUZZLE_WIDTH, PUZZLE_HEIGHT, PUZZLE_BITS,
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-			 DEFAULT_XPM, PUZZLE_NAME,
-#endif
-			 &pp->graphics_format, &pp->cmap,
-			 &pp->black, &pp->white);
-	if (pp->cmap != None) {
-		setColormap(display, window, pp->cmap, MI_WIN_IS_INWINDOW(mi));
-		if (pp->backGC == None) {
-			XGCValues   xgcv;
 
-			xgcv.background = pp->black;
-			pp->backGC = XCreateGC(display, window, GCBackground, &xgcv);
-		}
-	} else {
-		pp->black = MI_WIN_BLACK_PIXEL(mi);
-		pp->backGC = MI_GC(mi);
-	}
-	XSetForeground(display, pp->backGC, pp->black);
-	XFillRectangle(display, window, pp->backGC, 0, 0,
-		       pp->windowsize.x, pp->windowsize.y);
+	MI_CLEARWINDOWCOLORMAP(mi, pp->backGC, pp->black);
+
 	if (pp->logo) {
 		size.x = (pp->logo->width < pp->windowsize.x) ?
 			pp->logo->width : pp->windowsize.x;
@@ -554,10 +544,10 @@ init_puzzle(ModeInfo * mi)
 		pp->randompos.y = NRAND(MAX((pp->windowsize.y - pp->logo->height),
 					    2 * pp->offsetwindow.y + 1));
 		if (MI_NPIXELS(mi) <= 2)
-			XSetForeground(display, pp->backGC, MI_WIN_WHITE_PIXEL(mi));
+			XSetForeground(display, pp->backGC, MI_WHITE_PIXEL(mi));
 		else
 			XSetForeground(display, pp->backGC, MI_PIXEL(mi, NRAND(MI_NPIXELS(mi))));
-		XPutImage(display, window, pp->backGC, pp->logo,
+		(void) XPutImage(display, window, pp->backGC, pp->logo,
 		(int) (NRAND(MAX(1, (pp->logo->width - pp->usablewindow.x)))),
 			  (int) (NRAND(MAX(1, (pp->logo->height - pp->usablewindow.y)))),
 			  pp->randompos.x, pp->randompos.y,
@@ -587,7 +577,7 @@ init_puzzle(ModeInfo * mi)
 #ifdef NUMBERED
 	else {
 		if (pp->image)
-			XDestroyImage(pp->image);
+			(void) XDestroyImage(pp->image);
 		pp->randompos.x = pp->offsetwindow.x;
 		pp->randompos.y = pp->offsetwindow.y;
 		NumberScreen(mi);
@@ -643,7 +633,7 @@ release_puzzle(ModeInfo * mi)
 			if (pp->fixbuff != NULL)
 				(void) free((void *) pp->fixbuff);
 			if (pp->box)
-				XDestroyImage(pp->box);
+				(void) XDestroyImage(pp->box);
 			free_stuff(MI_DISPLAY(mi), pp);
 		}
 		(void) free((void *) puzzs);
