@@ -2,7 +2,7 @@
 /* random --- run random modes for a certain duration */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)random.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)random.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -75,7 +75,22 @@ static OptionStruct desc[] =
 };
 
 ModeSpecOpt random_opts =
-{4, opts, 3, vars, desc};
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+
+#ifdef USE_MODULES
+ModStruct   random_description =
+{"random", "init_random", "draw_random", "release_random",
+ "refresh_random", "change_random", NULL, &random_opts,
+ 1, 1, 1, 1, 64, 1.0, "",
+#ifdef USE_BOMB
+ "Shows a random mode from above except blank and bomb", 0, NULL};
+
+#else
+ "Shows a random mode from above except blank", 0, NULL};
+
+#endif
+
+#endif
 
 #ifdef USE_BOMB
 #define NUMSPECIAL	3	/* bomb, blank, random */
@@ -85,19 +100,21 @@ ModeSpecOpt random_opts =
 
 #define GC_SAVE_VALUES (GCFunction|GCLineWidth|GCLineStyle|GCCapStyle|GCJoinStyle|GCGraphicsExposures|GCFont|GCSubwindowMode)
 
-extern float saturation;
 extern int  delay;
 extern int  batchcount;
 extern int  cycles;
 extern int  size;
+extern int  ncolors;
+extern float saturation;
+extern char *imagefile;
 
 #define MAXMODECHARS 14
 
 #ifdef USE_GL
 static char gl_modes[][MAXMODECHARS] =
 {
-	"escher", "gears", "morph3d", "pipes", "rubik",
-	"sproingies", "superquadrics"
+	"cage", "gears", "moebius", "morph3d", "pipes", "rubik",
+	"sproingies", "stairs", "superquadrics"
 };
 
 #define NUMGL (sizeof (gl_modes) / sizeof (gl_modes[0]))
@@ -111,20 +128,26 @@ static char gl_modes[][MAXMODECHARS] =
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 static char xpm_modes[][MAXMODECHARS] =
 {
-	"bat", "cartoon", "image", "puzzle",
+	"bat", "cartoon", "image", "flag", "life", "life1d", "maze", "puzzle"
 };
+
 #define NUMXPM (sizeof (xpm_modes) / sizeof (xpm_modes[0]))
 #else
+/* They will not really be xpm_modes anymore... */
 static char xpm_modes[][MAXMODECHARS] =
-{""};
+{
+	"bat", "image", "flag", "life", "life1d", "maze", "puzzle"
+};
+
+#define NUMXPM (sizeof (xpm_modes) / sizeof (xpm_modes[0]))
 
-#define NUMXPM 0
 #endif
 
 static char write_modes[][MAXMODECHARS] =
 {
-	"mandelbrot", "swirl", "tube" /* XPM modes are writable too ... */
+	"mandelbrot", "swirl", "tube"	/* XPM modes are usually writable too ... */
 };
+
 #define NUMWRITE (sizeof (write_modes) / sizeof (write_modes[0]))
 
 static char nice_modes[][MAXMODECHARS] =
@@ -178,7 +201,7 @@ static char automata_modes[][MAXMODECHARS] =
 static char fractal_modes[][MAXMODECHARS] =
 {
 	"coral", "drift", "flame", "forest", "julia", "hop",
-  "ifs", "lightning", "mandelbrot", "mountain",
+	"ifs", "lightning", "mandelbrot", "mountain",
 	"sierpinski", "strange", "triangle", "turtle", "vines"
 };
 
@@ -324,7 +347,8 @@ strpmtok(int *sign, char *str)
 static void
 parsemodelist(ModeInfo * mi)
 {
-	int         i, j, sign = 1;
+	int         i, sign = 1;
+	unsigned int j;
 	char       *p;
 
 	modes = (int *) calloc(numprocs - 1, sizeof (int));
@@ -429,7 +453,9 @@ setMode(ModeInfo * mi, int newmode)
 	batchcount = MI_BATCHCOUNT(mi) = LockProcs[currentmode].def_batchcount;
 	cycles = MI_CYCLES(mi) = LockProcs[currentmode].def_cycles;
 	size = MI_SIZE(mi) = LockProcs[currentmode].def_size;
+	ncolors = MI_NCOLORS(mi) = LockProcs[currentmode].def_ncolors;
 	saturation = MI_SATURATION(mi) = LockProcs[currentmode].def_saturation;
+	imagefile = MI_IMAGEFILE(mi) = LockProcs[currentmode].def_imagefile;
 
 	for (i = 0; i < MI_NUM_SCREENS(mi); i++) {
 
@@ -467,8 +493,8 @@ init_random(ModeInfo * mi)
 			duration = 0;
 	}
 	if (rp->fix) {
-		fixColormap(MI_DISPLAY(mi), MI_WINDOW(mi), MI_SCREEN(mi),
-		MI_SATURATION(mi), MI_WIN_IS_MONO(mi), MI_WIN_IS_INSTALL(mi),
+		fixColormap(MI_DISPLAY(mi), MI_WINDOW(mi), MI_SCREEN(mi), MI_NCOLORS(mi),
+			    MI_SATURATION(mi), MI_WIN_IS_MONO(mi), MI_WIN_IS_INSTALL(mi),
 			    MI_WIN_IS_INROOT(mi), MI_WIN_IS_INWINDOW(mi), MI_WIN_IS_VERBOSE(mi));
 		rp->fix = False;
 	}
@@ -491,7 +517,7 @@ draw_random(ModeInfo * mi)
 	if ((scrn == 0) && (change_now || (has_run > duration))) {
 		newmode = pickMode();
 
-		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+		MI_CLEARWINDOW(mi);
 
 		setMode(mi, newmode);
 		starttime = now;
@@ -499,8 +525,8 @@ draw_random(ModeInfo * mi)
 		change_now = False;
 	}
 	if (rp->fix) {
-		fixColormap(MI_DISPLAY(mi), MI_WINDOW(mi), MI_SCREEN(mi),
-		MI_SATURATION(mi), MI_WIN_IS_MONO(mi), MI_WIN_IS_INSTALL(mi),
+		fixColormap(MI_DISPLAY(mi), MI_WINDOW(mi), MI_SCREEN(mi), MI_NCOLORS(mi),
+			    MI_SATURATION(mi), MI_WIN_IS_MONO(mi), MI_WIN_IS_INSTALL(mi),
 			    MI_WIN_IS_INROOT(mi), MI_WIN_IS_INWINDOW(mi), MI_WIN_IS_VERBOSE(mi));
 		rp->fix = False;
 	}
