@@ -2,7 +2,7 @@
 /* rotor --- a swirly rotor */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)rotor.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)rotor.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -48,6 +48,15 @@ static const char sccsid[] = "@(#)rotor.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt rotor_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   rotor_description =
+{"rotor", "init_rotor", "draw_rotor", "release_rotor",
+ "refresh_rotor", "init_rotor", NULL, &rotor_opts,
+ 10000, 4, 20, 1, 64, 0.4, "",
+ "Shows Tom's Roto-Rooter", 0, NULL};
+
+#endif
+
 /*-
  * A 'batchcount' of 3 or 4 works best!
  */
@@ -150,7 +159,11 @@ init_rotor(ModeInfo * mi)
 		if (rp->nsave <= 1)
 			rp->nsave = 2;
 		if (rp->save == NULL)
-			rp->save = (XPoint *) calloc(rp->nsave, sizeof (XPoint));
+			rp->save = (XPoint *) malloc(rp->nsave * sizeof (XPoint));
+			for (x = 0; x < rp->nsave; x++) {
+				rp->save[x].x = rp->centerx;
+				rp->save[x].y = rp->centery;
+      }
 
 		pelem = rp->elements;
 
@@ -174,7 +187,8 @@ init_rotor(ModeInfo * mi)
 		rp->angle = (float) NRAND((long) MAXANGLE) / 3.0;
 		rp->forward = rp->firsttime = True;
 	}
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+
+	MI_CLEARWINDOW(mi);
 }
 
 void
@@ -225,7 +239,7 @@ draw_rotor(ModeInfo * mi)
 	if (rp->firsttime)
 		rp->firsttime = False;
 	else {
-		XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 
 		x_1 = (int) rp->save[rp->rotor].x;
 		y_1 = (int) rp->save[rp->rotor].y;
@@ -245,7 +259,7 @@ draw_rotor(ModeInfo * mi)
 			if (++rp->pix >= MI_NPIXELS(mi))
 				rp->pix = 0;
 		} else
-			XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+			XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 
 		x_1 = rp->lastx;
 		y_1 = rp->lasty;
