@@ -1,8 +1,9 @@
+
 /* -*- Mode: C; tab-width: 4 -*- */
 /* strange --- strange attractors */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)strange.c	   4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)strange.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -45,6 +46,15 @@ static const char sccsid[] = "@(#)strange.c	   4.04 97/07/28 xlockmore";
 ModeSpecOpt strange_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   strange_description =
+{"strange", "init_strange", "draw_strange", "release_strange",
+ "init_strange", "init_strange", NULL, &strange_opts,
+ 1000, 1, 1, 1, 64, 1.0, "",
+ "Shows strange attractors", 0, NULL};
+
+#endif
+
 typedef float DBL;
 typedef int PRM;
 
@@ -73,7 +83,7 @@ DBL_To_PRM( 1.0-exp( -16.0*(a)/UNIT2 ) )
 
 typedef struct _ATTRACTOR {
 	DBL         Prm1[MAX_PRM], Prm2[MAX_PRM];
-  PRM Prm[MAX_PRM], *Fold;
+	PRM         Prm[MAX_PRM], *Fold;
 	void        (*Iterate) (struct _ATTRACTOR *, PRM, PRM, PRM *, PRM *);
 	XPoint     *Buffer1, *Buffer2;
 	int         Cur_Pt, Max_Pt;
@@ -125,7 +135,7 @@ Random_Prm(DBL * Prm)
    /* 2 examples of non-linear map */
 
 static void
-Iterate_X2(ATTRACTOR *A, PRM x, PRM y, PRM * xo, PRM * yo)
+Iterate_X2(ATTRACTOR * A, PRM x, PRM y, PRM * xo, PRM * yo)
 {
 	PRM         xx, yy, xy, x2y, y2x, Tmp;
 
@@ -144,7 +154,7 @@ Iterate_X2(ATTRACTOR *A, PRM x, PRM y, PRM * xo, PRM * yo)
 }
 
 static void
-Iterate_X3(ATTRACTOR *A, PRM x, PRM y, PRM * xo, PRM * yo)
+Iterate_X3(ATTRACTOR * A, PRM x, PRM y, PRM * xo, PRM * yo)
 {
 	PRM         xx, yy, xy, x2y, y2x, Tmp_x, Tmp_y, Tmp_z;
 
@@ -190,7 +200,7 @@ draw_strange(ModeInfo * mi)
 	ATTRACTOR  *A = &Root[MI_SCREEN(mi)];
 	DBL         Lx, Ly;
 	void        (*Iterate) (ATTRACTOR *, PRM, PRM, PRM *, PRM *);
-  PRM  xmin, xmax, ymin, ymax;
+	PRM         xmin, xmax, ymin, ymax;
 
 
 
@@ -235,7 +245,7 @@ draw_strange(ModeInfo * mi)
 		y = yo + NRAND(8) - 4;
 	}
 
-	XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+	XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 
 	if (A->dbuf) {		/* jwz */
 		XSetForeground(display, A->dbuf_gc, 0);
@@ -246,7 +256,7 @@ draw_strange(ModeInfo * mi)
 		XDrawPoints(display, window, gc, A->Buffer1, Cur_Pt, CoordModeOrigin);
 
 	if (MI_NPIXELS(mi) < 2)
-		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 	else
 		XSetForeground(display, gc, MI_PIXEL(mi, A->Col % MI_NPIXELS(mi)));
 
@@ -283,9 +293,9 @@ draw_strange(ModeInfo * mi)
 void
 init_strange(ModeInfo * mi)
 {
-  Display    *display = MI_DISPLAY(mi);
-  Window      window = MI_WINDOW(mi);
-  GC          gc = MI_GC(mi);
+	Display    *display = MI_DISPLAY(mi);
+	Window      window = MI_WINDOW(mi);
+	GC          gc = MI_GC(mi);
 	ATTRACTOR  *Attractor;
 
 	if (Root == NULL) {
@@ -294,7 +304,7 @@ init_strange(ModeInfo * mi)
 		if (Root == NULL)
 			return;
 	}
-  Attractor = &Root[MI_SCREEN(mi)];
+	Attractor = &Root[MI_SCREEN(mi)];
 
 	if (Attractor->Fold == NULL) {
 		int         i;
@@ -315,16 +325,17 @@ init_strange(ModeInfo * mi)
 			Attractor->Fold[i] = DBL_To_PRM(x);
 		}
 	}
-
-	Attractor->Buffer1 = (XPoint *) calloc(MAX_POINTS, sizeof (XPoint));
-	Attractor->Buffer2 = (XPoint *) calloc(MAX_POINTS, sizeof (XPoint));
+	if (Attractor->Buffer1 == NULL)
+		Attractor->Buffer1 = (XPoint *) calloc(MAX_POINTS, sizeof (XPoint));
+	if (Attractor->Buffer2 == NULL)
+		Attractor->Buffer2 = (XPoint *) calloc(MAX_POINTS, sizeof (XPoint));
 	if (Attractor->Buffer1 == NULL || Attractor->Buffer2 == NULL) {
 		if (Attractor->Buffer1 != NULL) {
-			free(Attractor->Buffer1);
+			(void) free((void *) Attractor->Buffer1);
 			Attractor->Buffer1 = NULL;
 		}
 		if (Attractor->Buffer2 != NULL) {
-			free(Attractor->Buffer2);
+			(void) free((void *) Attractor->Buffer2);
 			Attractor->Buffer2 = NULL;
 		}
 		Attractor->Cur_Pt = 0;
@@ -343,13 +354,13 @@ init_strange(ModeInfo * mi)
 	Random_Prm(Attractor->Prm1);
 	Random_Prm(Attractor->Prm2);
 
-  if (Attractor->dbuf)
-    XFreePixmap(display, Attractor->dbuf);
+	if (Attractor->dbuf)
+		XFreePixmap(display, Attractor->dbuf);
 
 	Attractor->dbuf = XCreatePixmap(display, window,
 				     Attractor->Width, Attractor->Height, 1);
 
-	/* don't want any exposure events from XCopyPlane */
+	/* Do not want any exposure events from XCopyPlane */
 	XSetGraphicsExposures(display, MI_GC(mi), False);
 
 	if (Attractor->dbuf) {
@@ -359,19 +370,19 @@ init_strange(ModeInfo * mi)
 		gcv.background = 0;
 		gcv.graphics_exposures = False;
 		gcv.function = GXcopy;
-    if (Attractor->dbuf_gc)
-      XFreeGC(display, Attractor->dbuf_gc);
+		if (Attractor->dbuf_gc)
+			XFreeGC(display, Attractor->dbuf_gc);
 
 		Attractor->dbuf_gc = XCreateGC(display, Attractor->dbuf,
-				    GCForeground | GCBackground | GCGraphicsExposures | GCFunction,
+					       GCForeground | GCBackground | GCGraphicsExposures | GCFunction,
 					       &gcv);
 		XFillRectangle(display, Attractor->dbuf,
 			       Attractor->dbuf_gc, 0, 0, Attractor->Width,
 			       Attractor->Height);
-		XSetBackground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+		XSetBackground(display, gc, MI_BLACK_PIXEL(mi));
 		XSetFunction(display, gc, GXcopy);
 	}
-	XClearWindow(display, window);
+	MI_CLEARWINDOW(mi);
 }
 
 /***************************************************************/
@@ -383,21 +394,21 @@ release_strange(ModeInfo * mi)
 
 	if (Root != NULL) {
 
-	for (screen = 0; screen < MI_NUM_SCREENS(mi); ++screen) {
-		ATTRACTOR *Attractor = &Root[screen];
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); ++screen) {
+			ATTRACTOR  *Attractor = &Root[screen];
 
-		if (Attractor->Buffer1 != NULL)
-			(void) free((void *) Attractor->Buffer1);
-		if (Attractor->Buffer2 != NULL)
-			(void) free((void *) Attractor->Buffer2);
-		if (Attractor->dbuf)
-			XFreePixmap(MI_DISPLAY(mi), Attractor->dbuf);
-		if (Attractor->dbuf_gc)
-			XFreeGC(MI_DISPLAY(mi), Attractor->dbuf_gc);
-		if (Attractor->Fold != NULL)
-			free(Attractor->Fold);
+			if (Attractor->Buffer1 != NULL)
+				(void) free((void *) Attractor->Buffer1);
+			if (Attractor->Buffer2 != NULL)
+				(void) free((void *) Attractor->Buffer2);
+			if (Attractor->dbuf)
+				XFreePixmap(MI_DISPLAY(mi), Attractor->dbuf);
+			if (Attractor->dbuf_gc)
+				XFreeGC(MI_DISPLAY(mi), Attractor->dbuf_gc);
+			if (Attractor->Fold != NULL)
+				(void) free((void *) Attractor->Fold);
+		}
+		(void) free((void *) Root);
+		Root = NULL;
 	}
-	(void) free((void *) Root);
-	Root = NULL;
-  }
 }
