@@ -2,7 +2,7 @@
 /* swarm --- swarm of bees */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)swarm.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)swarm.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -31,7 +31,7 @@ static const char sccsid[] = "@(#)swarm.c	4.04 97/07/28 xlockmore";
 #define HACK_INIT init_swarm
 #define HACK_DRAW draw_swarm
 #define swarm_opts xlockmore_opts
-#define DEFAULTS "*delay: 10000 \n" \
+#define DEFAULTS "*delay: 15000 \n" \
  "*count: 100 \n"
 #define BRIGHT_COLORS
 #include "xlockmore.h"		/* from the xscreensaver distribution */
@@ -43,6 +43,15 @@ static const char sccsid[] = "@(#)swarm.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt swarm_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   swarm_description =
+{"swarm", "init_swarm", "draw_swarm", "release_swarm",
+ "refresh_swarm", "init_swarm", NULL, &swarm_opts,
+ 15000, 100, 1, 1, 64, 1.0, "",
+ "Shows a swarm of bees following a wasp", 0, NULL};
+
+#endif
+
 #define TIMES	4		/* number of time positions recorded */
 #define BEEACC	2		/* acceleration of bees */
 #define WASPACC 5		/* maximum acceleration of wasp */
@@ -136,15 +145,15 @@ init_swarm(ModeInfo * mi)
 		black.red = black.green = black.blue = 0;
 		black.flags = DoRed | DoGreen | DoBlue;
 		bit = XCreatePixmapFromBitmapData(display, window, "\000", 1, 1,
-						  MI_WIN_BLACK_PIXEL(mi),
-						  MI_WIN_BLACK_PIXEL(mi), 1);
+						  MI_BLACK_PIXEL(mi),
+						  MI_BLACK_PIXEL(mi), 1);
 		sp->cursor = XCreatePixmapCursor(display, bit, bit, &black, &black,
 						 0, 0);
 		XFreePixmap(display, bit);
 	}
 	XDefineCursor(display, window, sp->cursor);
-	/* Clear the background. */
-	XClearWindow(display, window);
+
+	MI_CLEARWINDOW(mi);
 
 	/* Allocate memory. */
 
@@ -195,7 +204,7 @@ draw_swarm(ModeInfo * mi)
 		int         rx, ry;
 		unsigned int m;
 
-		XQueryPointer(display, window,
+		(void) XQueryPointer(display, window,
 			      &r, &c, &rx, &ry, &cx, &cy, &m);
 		if (cx <= sp->border || cy <= sp->border ||
 		    cx >= MI_WIN_WIDTH(mi) - 1 - sp->border ||
@@ -288,12 +297,12 @@ draw_swarm(ModeInfo * mi)
 		sp->old_segs[b].y2 = Y(2, b);
 	}
 
-	XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+	XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	XDrawLine(display, window, gc,
 		  sp->wx[1], sp->wy[1], sp->wx[2], sp->wy[2]);
 	XDrawSegments(display, window, gc, sp->old_segs, sp->beecount);
 
-	XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+	XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 	XDrawLine(display, window, gc,
 		  sp->wx[0], sp->wy[0], sp->wx[1], sp->wy[1]);
 	if (MI_NPIXELS(mi) > 2) {
