@@ -2,7 +2,7 @@
 /* swirl --- swirly patterns */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)swirl.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)swirl.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -51,6 +51,15 @@ static const char sccsid[] = "@(#)swirl.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt swirl_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   swirl_description =
+{"swirl", "init_swirl", "draw_swirl", "release_swirl",
+ "refresh_swirl", "init_swirl", NULL, &swirl_opts,
+ 5000, 5, 1, 1, 64, 1.0, "",
+ "Shows animated swirling patterns", 0, NULL};
+
+#endif
+
 #include <time.h>
 
 #define MASS            4	/* maximum mass of a knot */
@@ -132,10 +141,10 @@ typedef struct swirl_data {
 #ifndef STANDALONE
 	Bool        fixed_colourmap;	/* fixed colourmap? */
 	int         current_map;	/* current colour map, 0..dcolours-1 */
-	unsigned long fg, bg, white, black;	/* black and white pixel values */
+	unsigned long black, white, bg, fg;	/* black and white pixel values */
 	int         shift;	/* colourmap shift */
 	int         dshift;	/* colourmap shift while drawing */
-	XColor      fgcol, bgcol;	/* foreground and background colour specs */
+	XColor      bgcol, fgcol;	/* foreground and background colour specs */
 #endif				/* !STANDALONE */
 } SWIRL    , *SWIRL_P;
 
@@ -174,13 +183,8 @@ random_no(unsigned int n)
    -      swirl is the swirl data */
 
 static void
-initialise_swirl(ModeInfo * mi, SWIRL_P swirl)
+initialise_swirl(SWIRL_P swirl)
 {
-#ifndef STANDALONE
-	Display    *display = MI_DISPLAY(mi);
-
-#endif /* !STANDALONE */
-
 	swirl->width = 0;	/* width and height of window */
 	swirl->height = 0;
 	swirl->depth = 1;
@@ -209,20 +213,6 @@ initialise_swirl(ModeInfo * mi, SWIRL_P swirl)
 	swirl->dcolours = 0;	/* how many colours for shading */
 	swirl->cmap = (Colormap) NULL;
 	swirl->rgb_values = NULL;	/* colour definitions array */
-
-#ifndef STANDALONE
-	swirl->current_map = 0;	/* current colour map, 0..dcolours-1 */
-
-	/* set up fg fb colour specs */
-	swirl->white = MI_WIN_WHITE_PIXEL(mi);
-	swirl->black = MI_WIN_BLACK_PIXEL(mi);
-	swirl->fg = MI_FG_COLOR(mi);
-	swirl->bg = MI_BG_COLOR(mi);
-	swirl->fgcol.pixel = swirl->fg;
-	swirl->bgcol.pixel = swirl->bg;
-	XQueryColor(display, MI_WIN_COLORMAP(mi), &(swirl->fgcol));
-	XQueryColor(display, MI_WIN_COLORMAP(mi), &(swirl->bgcol));
-#endif /* !STANDALONE */
 }
 
 /****************************************************************/
@@ -255,7 +245,7 @@ initialise_image(Display * dpy, SWIRL_P swirl)
 	pad = BitmapPad(dpy);
 	/* destroy the old image (destroy XImage and data) */
 	if (swirl->ximage != NULL)
-		XDestroyImage(swirl->ximage);
+		(void) XDestroyImage(swirl->ximage);
 
 	/* how many bytes per line? (bits rounded up to pad) */
 	bytes_per_line = ((swirl->width * data_depth + pad - 1) / pad) * (pad / 8);
@@ -378,8 +368,8 @@ set_black_and_white(SWIRL_P swirl, XColor * values)
 	values[black].blue = 0;
 
 	/* copy the colour specs from the original entries */
-	values[swirl->fg] = swirl->fgcol;
 	values[swirl->bg] = swirl->bgcol;
+	values[swirl->fg] = swirl->fgcol;
 }
 
 /****************************************************************/
@@ -406,13 +396,13 @@ static void
 set_colour(SWIRL_P swirl, XColor ** value, unsigned long *pixel, COLOUR_P c)
 {
 	Bool        done;
-	unsigned long fg, bg, white, black;
+	unsigned long black, white, bg, fg;
 
-	/* where are foreground, background, white, and black? */
-	fg = swirl->fg;
-	bg = swirl->bg;
-	white = swirl->white;
+	/* where are black, white, background, foreground? */
 	black = swirl->black;
+	white = swirl->white;
+	bg = swirl->bg;
+	fg = swirl->fg;
 
 	/* haven't set it yet */
 	done = False;
@@ -422,9 +412,9 @@ set_colour(SWIRL_P swirl, XColor ** value, unsigned long *pixel, COLOUR_P c)
 		(**value).flags = DoRed | DoGreen | DoBlue;
 		(**value).pixel = *pixel;
 
-		/* white, black, fg, bg, or a colour? */
-		if ((*pixel != fg) && (*pixel != bg) &&
-		    (*pixel != white) && (*pixel != black)) {
+		/* white, black, bg, fg, or a colour? */
+		if ((*pixel != black) && (*pixel != white) &&
+		    (*pixel != bg) && (*pixel != fg)) {
 			(**value).red = c->r;
 			(**value).green = c->g;
 			(**value).blue = c->b;
@@ -456,13 +446,13 @@ static void
 get_colour(SWIRL_P swirl, XColor ** value, COLOUR_P c)
 {
 	Bool        done;
-	unsigned long fg, bg, white, black;
+	unsigned long black, white, bg, fg;
 
 	/* where is white and black? */
-	fg = swirl->fg;
-	bg = swirl->bg;
-	white = swirl->white;
 	black = swirl->black;
+	white = swirl->white;
+	bg = swirl->bg;
+	fg = swirl->fg;
 
 	/* haven't set it yet */
 	done = False;
@@ -470,8 +460,8 @@ get_colour(SWIRL_P swirl, XColor ** value, COLOUR_P c)
 	/* try and set the colour */
 	while (!done) {
 		/* black, white or a colour? */
-		if (((*value)->pixel != fg) && ((*value)->pixel != bg) &&
-		  ((*value)->pixel != white) && ((*value)->pixel != black)) {
+		if (((*value)->pixel != black) && ((*value)->pixel != white) &&
+		  ((*value)->pixel != bg) && ((*value)->pixel != fg)) {
 			c->r = (*value)->red;
 			c->g = (*value)->green;
 			c->b = (*value)->blue;
@@ -700,7 +690,7 @@ static void
 create_colourmap(ModeInfo * mi, SWIRL_P swirl)
 {
 	Display    *display = MI_DISPLAY(mi);
-	int         preserve;
+	int         preserve = 0;
 	int         n_rotations;
 	int         i;
 	Bool        truecolor;
@@ -708,11 +698,8 @@ create_colourmap(ModeInfo * mi, SWIRL_P swirl)
 
 	swirl->fixed_colourmap = !setupColormap(mi, &(swirl->colours),
 				&truecolor, &redmask, &greenmask, &bluemask);
-	preserve = preserveColors(swirl->fg, swirl->bg, swirl->white, swirl->black);
 
 	/* how many colours should we animate? */
-	swirl->dcolours = (swirl->colours > preserve + 1) ?
-		swirl->colours - preserve : swirl->colours;
 
 	if (MI_NPIXELS(mi) < 2)
 		return;
@@ -721,22 +708,37 @@ create_colourmap(ModeInfo * mi, SWIRL_P swirl)
 	swirl->shift = (swirl->colours > 64) ? swirl->colours / 64 : 1;
 	swirl->dshift = (swirl->shift > 1) ? swirl->shift * 2 : 1;
 
-	/* how may colour map rotations are there? */
-	n_rotations = (swirl->fixed_colourmap) ? 1 : swirl->dcolours;
-
-	/* allocate space for colour definitions (if not already there) */
-	if (!swirl->rgb_values)
-		XFree((caddr_t) swirl->rgb_values);
-	swirl->rgb_values = (XColor *) calloc((swirl->colours + 3) * n_rotations,
-					      sizeof (XColor));
-
 	/* create a colour map */
 	if (!swirl->fixed_colourmap) {
+		swirl->current_map = 0;		/* current colour map, 0..dcolours-1 */
+
+		/* set up fg fb colour specs */
+		swirl->black = MI_BLACK_PIXEL(mi);
+		swirl->white = MI_WHITE_PIXEL(mi);
+		swirl->bg = MI_BG_PIXEL(mi);
+		swirl->fg = MI_FG_PIXEL(mi);
+		preserve = preserveColors(swirl->black, swirl->white,
+					  swirl->bg, swirl->fg);
+		swirl->bgcol.pixel = swirl->bg;
+		swirl->fgcol.pixel = swirl->fg;
+		XQueryColor(display, MI_COLORMAP(mi), &(swirl->bgcol));
+		XQueryColor(display, MI_COLORMAP(mi), &(swirl->fgcol));
 		if (swirl->cmap)
 			XFreeColormap(display, swirl->cmap);
 		swirl->cmap =
 			XCreateColormap(display, swirl->win, swirl->visual, AllocAll);
 	}
+	swirl->dcolours = (swirl->colours > preserve + 1) ?
+		swirl->colours - preserve : swirl->colours;
+	/* how may colour map rotations are there? */
+	n_rotations = (swirl->fixed_colourmap) ? 1 : swirl->dcolours;
+
+	/* allocate space for colour definitions (if not already there) */
+	if (swirl->rgb_values != NULL)
+		XFree((caddr_t) swirl->rgb_values);
+	swirl->rgb_values = (XColor *) calloc((swirl->colours + 3) * n_rotations,
+					      sizeof (XColor));
+
 	/* select a set of colours for the colour map */
 	basic_map(swirl, swirl->rgb_values);
 
@@ -772,7 +774,7 @@ create_colourmap(ModeInfo * mi, SWIRL_P swirl)
 		} else {
 			/* lookup the colours in the fixed colour map */
 			for (i = 0; i < swirl->colours; i++)
-				XAllocColor(display, MI_WIN_COLORMAP(mi),
+				(void) XAllocColor(display, MI_COLORMAP(mi),
 					    &(swirl->rgb_values[i]));
 		}
 	}
@@ -1034,12 +1036,12 @@ do_point(SWIRL_P swirl, int i, int j)
 		value = dcolours - (abs(value) % (dcolours - 1));
 
 #ifndef STANDALONE
-	/* if fg and bg are 1 and 0 we should be OK, but just in case */
+	/* if bg and fg are 0 and 1 we should be OK, but just in case */
 	while ((dcolours > 2) &&
-	       (((value % swirl->colours) == (int) swirl->fg) ||
-		((value % swirl->colours) == (int) swirl->bg) ||
+	       (((value % swirl->colours) == (int) swirl->black) ||
 		((value % swirl->colours) == (int) swirl->white) ||
-		((value % swirl->colours) == (int) swirl->black))) {
+		((value % swirl->colours) == (int) swirl->bg) ||
+		((value % swirl->colours) == (int) swirl->fg))) {
 		value++;
 	}
 #endif /* !STANDALONE */
@@ -1120,15 +1122,10 @@ draw_point(ModeInfo * mi, SWIRL_P swirl)
 		draw_block(swirl->ximage, x, y, r, do_point(swirl, x, y));
 
 	/* update the screen */
-/* PURIFY 4.0.1 on SunOS4 and on Solaris 2 reports a 256 byte memory leak on * 
-
-
-
-
-
-
-   the next line. */
-	XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi), swirl->ximage,
+/*-
+ * PURIFY 4.0.1 on SunOS4 and on Solaris 2 reports a 256 byte memory leak on
+ * the next line. */
+	(void) XPutImage(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi), swirl->ximage,
 		  x, y, x, y, r, r);
 }
 
@@ -1255,7 +1252,7 @@ init_swirl(ModeInfo * mi)
 
 		/* initialise them all */
 		for (i = 0; i < ScreenCount(display); i++)
-			initialise_swirl(mi, &swirls[i]);
+			initialise_swirl(&swirls[i]);
 	}
 	/* get a pointer to this swirl */
 	swirl = &(swirls[MI_SCREEN(mi)]);
@@ -1264,7 +1261,7 @@ init_swirl(ModeInfo * mi)
 	swirl->win = window;
 	swirl->width = MI_WIN_WIDTH(mi);
 	swirl->height = MI_WIN_HEIGHT(mi);
-	swirl->depth = MI_WIN_DEPTH(mi);
+	swirl->depth = MI_DEPTH(mi);
 	swirl->rdepth = swirl->depth;
 	swirl->visual = MI_VISUAL(mi);
 
@@ -1274,8 +1271,7 @@ init_swirl(ModeInfo * mi)
 	/* initialise image for speeding up drawing */
 	initialise_image(display, swirl);
 
-	/* clear the window (before setting the colourmap) */
-	XClearWindow(display, MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
 
 #ifdef STANDALONE
 
@@ -1351,7 +1347,7 @@ draw_swirl(ModeInfo * mi)
 		if (swirl->drawing) {
 #ifdef STANDALONE
 			if (mi->writable_p)
-				rotate_colors(MI_DISPLAY(mi), MI_WIN_COLORMAP(mi),
+				rotate_colors(MI_DISPLAY(mi), MI_COLORMAP(mi),
 				       swirl->rgb_values, swirl->colours, 1);
 #else /* !STANDALONE */
 			/* rotate the colours */
@@ -1373,7 +1369,7 @@ draw_swirl(ModeInfo * mi)
 		} else {
 #ifdef STANDALONE
 			if (mi->writable_p)
-				rotate_colors(MI_DISPLAY(mi), MI_WIN_COLORMAP(mi),
+				rotate_colors(MI_DISPLAY(mi), MI_COLORMAP(mi),
 				       swirl->rgb_values, swirl->colours, 1);
 #else /* !STANDALONE */
 			/* rotate the colours */
@@ -1410,12 +1406,12 @@ draw_swirl(ModeInfo * mi)
 
 #ifdef STANDALONE
 					/* Pick a new colormap! */
-					XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
-					free_colors(MI_DISPLAY(mi), MI_WIN_COLORMAP(mi),
+					MI_CLEARWINDOW(mi);
+					free_colors(MI_DISPLAY(mi), MI_COLORMAP(mi),
 						    mi->colors, mi->npixels);
 					make_smooth_colormap(MI_DISPLAY(mi),
 							     MI_VISUAL(mi),
-							 MI_WIN_COLORMAP(mi),
+							 MI_COLORMAP(mi),
 					      mi->colors, &mi->npixels, True,
 						      &mi->writable_p, True);
 					swirl->colours = mi->npixels;
@@ -1449,7 +1445,7 @@ release_swirl(ModeInfo * mi)
 			if (swirl->rgb_values != NULL)
 				XFree((caddr_t) swirl->rgb_values);
 			if (swirl->ximage != NULL)
-				XDestroyImage(swirl->ximage);
+				(void) XDestroyImage(swirl->ximage);
 			if (swirl->knots)
 				(void) free((void *) swirl->knots);
 		}
