@@ -2,7 +2,7 @@
 /* tube --- animated tube */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)tube.c 4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)tube.c 4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -48,6 +48,15 @@ static const char sccsid[] = "@(#)tube.c 4.04 97/07/28 xlockmore";
 ModeSpecOpt tube_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   tube_description =
+{"tube", "init_tube", "draw_tube", "release_tube",
+ NULL, "init_tube", NULL, &tube_opts,
+ 25000, -9, 20000, -200, 64, 1.0, "",
+ "Shows an animated tube", 0, NULL};
+
+#endif
+
 #define MINSIZE 3
 #define MINSHAPE 1
 
@@ -62,8 +71,7 @@ typedef struct {
 	XColor     *colors;
 	XColor      top;
 	XColor      bottom;
-	unsigned int fg, bg;
-	XColor      fgcol, bgcol;	/* foreground and background colour specs */
+	XColor      bgcol, fgcol;	/* foreground and background colour specs */
 	int         counter;
 	int         ncolors;
 	int         usable_colors;
@@ -146,9 +154,9 @@ init_tube(ModeInfo * mi)
 		} while ((star != 1) && (!(tp->shape % star)));
 		for (i = 0; i < tp->shape; i++) {
 			tp->proto_pts[i].x = tp->average / 2 +
-				cos((double) start * 2.0 * M_PI / 360.0) * tp->average / 2.0;
+				(int) (cos((double) start * 2.0 * M_PI / 360.0) * tp->average / 2.0);
 			tp->proto_pts[i].y = tp->average / 2 +
-				sin((double) start * 2.0 * M_PI / 360.0) * tp->average / 2.0;
+				(int) (sin((double) start * 2.0 * M_PI / 360.0) * tp->average / 2.0);
 			start += star * 360 / tp->shape;
 		}
 		tp->proto_pts[tp->shape] = tp->proto_pts[0];
@@ -159,29 +167,31 @@ init_tube(ModeInfo * mi)
 	lw = tp->linewidth / 2;
 	tp->x1 = NRAND(screen_width - tp->width - 2 * lw) + lw;
 	tp->y1 = NRAND(screen_height - tp->height - 2 * lw) + lw;
-	XClearWindow(display, window);
-	tp->counter = 0;
 
-	tp->fg = MI_FG_COLOR(mi);
-	tp->bg = MI_BG_COLOR(mi);
-	tp->fgcol.pixel = tp->fg;
-	tp->bgcol.pixel = tp->bg;
-	preserve = preserveColors(tp->fg, tp->bg,
-			     MI_WIN_WHITE_PIXEL(mi), MI_WIN_BLACK_PIXEL(mi));
-	XQueryColor(display, MI_WIN_COLORMAP(mi), &(tp->fgcol));
-	XQueryColor(display, MI_WIN_COLORMAP(mi), &(tp->bgcol));
+	MI_CLEARWINDOW(mi);
+
+	tp->counter = 0;
 
 	tp->fixed_colormap = !setupColormap(mi,
 		&(tp->ncolors), &truecolor, &redmask, &greenmask, &bluemask);
 	if (!tp->fixed_colormap) {
-		/* color in "the bottom third */
-		tp->bottom.red = NRAND(65536 / 3);
-		tp->bottom.blue = NRAND(65536 / 3);
-		tp->bottom.green = NRAND(65536 / 3);
-		/* color in "the top third */
-		tp->top.red = NRAND(65536 / 3) + 65536 * 2 / 3;
-		tp->top.blue = NRAND(65536 / 3) + 65536 * 2 / 3;
-		tp->top.green = NRAND(65536 / 3) + 65536 * 2 / 3;
+		preserve = preserveColors(MI_BLACK_PIXEL(mi), MI_WHITE_PIXEL(mi),
+			     MI_BG_PIXEL(mi), MI_FG_PIXEL(mi));
+		tp->bgcol.pixel = MI_BG_PIXEL(mi);
+		tp->fgcol.pixel = MI_FG_PIXEL(mi);
+		XQueryColor(display, MI_COLORMAP(mi), &(tp->bgcol));
+		XQueryColor(display, MI_COLORMAP(mi), &(tp->fgcol));
+
+#define RANGE 65536
+#define DENOM 2
+		/* color in the bottom part */
+		tp->bottom.red = NRAND(RANGE / DENOM);
+		tp->bottom.blue = NRAND(RANGE / DENOM);
+		tp->bottom.green = NRAND(RANGE / DENOM);
+		/* color in the top part */
+		tp->top.red = RANGE - NRAND(RANGE / DENOM);
+		tp->top.blue = RANGE - NRAND(RANGE / DENOM);
+		tp->top.green = RANGE - NRAND(RANGE / DENOM);
 
 		/* allocate colormap, if needed */
 		if (tp->colors == NULL)
@@ -189,8 +199,9 @@ init_tube(ModeInfo * mi)
 		if (tp->cmap == None)
 			tp->cmap = XCreateColormap(display, window,
 						   MI_VISUAL(mi), AllocAll);
+		tp->usable_colors = tp->ncolors - preserve;
 	}
-	tp->usable_colors = tp->ncolors - preserve;
+	tp->cur_color = NRAND(tp->ncolors);
 }
 
 void
@@ -204,16 +215,17 @@ draw_tube(ModeInfo * mi)
 	int         k, lw = tp->linewidth / 2;
 
 	/* advance drawing color */
-	tp->cur_color = (tp->cur_color + 1) % MI_NPIXELS(mi);
+	tp->cur_color = (tp->cur_color + 1) % tp->ncolors;
 	if (!tp->fixed_colormap && tp->usable_colors > 2) {
-		while (tp->cur_color == MI_WIN_WHITE_PIXEL(mi) ||
-		       tp->cur_color == MI_WIN_BLACK_PIXEL(mi) ||
-		       tp->cur_color == MI_FG_COLOR(mi) ||
-		       tp->cur_color == MI_BG_COLOR(mi)) {
-			tp->cur_color = (tp->cur_color + 1) % MI_NPIXELS(mi);
+		while (tp->cur_color == MI_WHITE_PIXEL(mi) ||
+		       tp->cur_color == MI_BLACK_PIXEL(mi) ||
+		       tp->cur_color == MI_BG_PIXEL(mi) ||
+		       tp->cur_color == MI_FG_PIXEL(mi)) {
+			tp->cur_color = (tp->cur_color + 1) % tp->ncolors;
 		}
-	}
-	XSetForeground(display, gc, tp->cur_color);
+		XSetForeground(display, gc, tp->cur_color);
+	} else
+		XSetForeground(display, gc, MI_PIXEL(mi, tp->cur_color));
 
 	/* move rectangle forward, horiz */
 	tp->x1 += tp->dx1;
@@ -244,7 +256,7 @@ draw_tube(ModeInfo * mi)
 		XDrawArc(display, window, gc,
 			 tp->x1, tp->y1, tp->width, tp->height, 0, 23040);
 	else {
-		for (i = 0; i <= tp->shape; i++) {
+		for (i = 0; i <= (unsigned int) tp->shape; i++) {
 			tp->pts[i].x = tp->x1 + tp->proto_pts[i].x;
 			tp->pts[i].y = tp->y1 + tp->proto_pts[i].y;
 		}
@@ -256,46 +268,46 @@ draw_tube(ModeInfo * mi)
 	/* advance colormap */
 	if (!tp->fixed_colormap && tp->usable_colors > 2) {
 		for (i = 0, j = tp->cur_color, k = 0;
-		     i < tp->ncolors; i++) {
-			if (i == MI_WIN_WHITE_PIXEL(mi)) {
+		     i < (unsigned int) tp->ncolors; i++) {
+			if (i == MI_WHITE_PIXEL(mi)) {
 				tp->colors[i].pixel = i;
 				tp->colors[i].red = 65535;
 				tp->colors[i].blue = 65535;
 				tp->colors[i].green = 65535;
 				tp->colors[i].flags = DoRed | DoGreen | DoBlue;
-			} else if (i == MI_WIN_BLACK_PIXEL(mi)) {
+			} else if (i == MI_BLACK_PIXEL(mi)) {
 				tp->colors[i].pixel = i;
 				tp->colors[i].red = 0;
 				tp->colors[i].blue = 0;
 				tp->colors[i].green = 0;
 				tp->colors[i].flags = DoRed | DoGreen | DoBlue;
-			} else if (i == tp->fg) {
-				tp->colors[i] = tp->fgcol;
-			} else if (i == tp->bg) {
+			} else if (i == MI_BG_PIXEL(mi)) {
 				tp->colors[i] = tp->bgcol;
+			} else if (i == MI_FG_PIXEL(mi)) {
+				tp->colors[i] = tp->fgcol;
 			} else {
 				double      range;
 
 				tp->colors[i].pixel = i;
 				range = ((double) (tp->top.red - tp->bottom.red)) /
-					MI_NPIXELS(mi);
+				 tp->ncolors;	
 				tp->colors[i].red = (short unsigned int) (range * j +
 							     tp->bottom.red);
 				range = ((double) (tp->top.green - tp->bottom.green)) /
-					MI_NPIXELS(mi);
+				 tp->ncolors;	
 				tp->colors[i].green = (short unsigned int) (range * j +
 							   tp->bottom.green);
 				range = ((double) (tp->top.blue - tp->bottom.blue)) /
-					MI_NPIXELS(mi);
+				 tp->ncolors;	
 				tp->colors[i].blue = (short unsigned int) (range * j +
 							    tp->bottom.blue);
 				tp->colors[i].flags = DoRed | DoGreen | DoBlue;
-				j = (j + 1) % MI_NPIXELS(mi);
+				j = (j + 1) % tp->ncolors;
 				k++;
 			}
 		}
 		/* make the entire tube move forward */
-		XStoreColors(display, tp->cmap, tp->colors, MI_NPIXELS(mi));
+		XStoreColors(display, tp->cmap, tp->colors, tp->ncolors);
 		setColormap(display, window, tp->cmap, MI_WIN_IS_INWINDOW(mi));
 	}
 	tp->counter++;
