@@ -2,7 +2,7 @@
 /* voters --- Dewdney's Voting Simulation */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)voters.c	4.06 97/10/28 xlockmore";
+static const char sccsid[] = "@(#)voters.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -45,6 +45,15 @@ static const char sccsid[] = "@(#)voters.c	4.06 97/10/28 xlockmore";
 ModeSpecOpt voters_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   voters_description =
+{"voters", "init_voters", "draw_voters", "release_voters",
+ "refresh_voters", "init_voters", NULL, &voters_opts,
+ 1000, 0, 327670, 0, 64, 1.0, "",
+ "Shows Dewdney's Voters", 0, NULL};
+
+#endif
+
 extern int  neighbors;
 
 /*-
@@ -99,11 +108,12 @@ typedef struct {
 	CellList   *last, *first;
 	char       *arr;
 	int         neighbors;
+	int         busyLoop;
 	XPoint      hexagonList[6];
 	XPoint      triangleList[2][3];
 } voterstruct;
 
-static int  initVal[NEIGHBORKINDS] =
+static char plots[NEIGHBORKINDS] =
 {
 	3, 4, 6, 8, 9, 12	/* Neighborhoods */
 };
@@ -120,7 +130,7 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 	GC          gc = MI_GC(mi);
 	voterstruct *vp = &voters[MI_SCREEN(mi)];
 	unsigned long colour = (MI_NPIXELS(mi) > 2) ?
-	MI_PIXEL(mi, color) : MI_WIN_WHITE_PIXEL(mi);
+	MI_PIXEL(mi, color) : MI_WHITE_PIXEL(mi);
 
 	XSetForeground(display, gc, colour);
 	if (vp->neighbors == 6) {
@@ -136,7 +146,7 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 			      vp->hexagonList, 6, Convex, CoordModePrevious);
 		else {
 			if (firstChange) {
-				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 				XFillPolygon(display, window, gc,
 					     vp->hexagonList, 6, Convex, CoordModePrevious);
 				XSetForeground(display, gc, colour);
@@ -160,7 +170,7 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 					       vp->xs, vp->ys);
 			else {
 				if (firstChange) {
-					XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+					XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 					XFillRectangle(display, window, gc,
 						       vp->xb + vp->xs * col, vp->yb + vp->ys * row,
 						       vp->xs, vp->ys);
@@ -171,7 +181,7 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 					 0, 23040);
 			}
 		} else
-			XPutImage(display, window, gc,
+			(void) XPutImage(display, window, gc,
 				  &logo[bitmap], 0, 0,
 				vp->xb + vp->xs * col, vp->yb + vp->ys * row,
 				  icon_width, icon_height);
@@ -194,7 +204,7 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 					     vp->triangleList[orient], 3, Convex, CoordModePrevious);
 			else {
 				if (firstChange) {
-					XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+					XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 					XFillPolygon(display, window, gc,
 						     vp->triangleList[orient], 3, Convex, CoordModePrevious);
 					XSetForeground(display, gc, colour);
@@ -506,6 +516,22 @@ advanceColors(ModeInfo * mi, int col, int row)
 	}
 }
 
+void
+refresh_voters(ModeInfo * mi)
+{
+	voterstruct *vp = &voters[MI_SCREEN(mi)];
+	int         col, row, colrow;
+
+	for (row = 0; row < vp->nrows; row++)
+		for (col = 0; col < vp->ncols; col++) {
+			colrow = col + row * vp->ncols;
+			/* Draw all old, will get corrected soon if wrong... */
+			drawcell(mi, col, row,
+				 (unsigned long) (MI_NPIXELS(mi) * vp->arr[colrow] / BITMAPS),
+				 vp->arr[colrow], False);
+		}
+}
+
 void
 init_voters(ModeInfo * mi)
 {
@@ -538,13 +564,13 @@ init_voters(ModeInfo * mi)
 	vp->height = MI_WIN_HEIGHT(mi);
 
 	for (i = 0; i < NEIGHBORKINDS; i++) {
-		if (neighbors == initVal[i]) {
+		if (neighbors == plots[i]) {
 			vp->neighbors = neighbors;
 			break;
 		}
 		if (i == NEIGHBORKINDS - 1) {
 #if 0
-			vp->neighbors = initVal[NRAND(NEIGHBORKINDS)];
+			vp->neighbors = plots[NRAND(NEIGHBORKINDS)];
 			vp->neighbors = (LRAND() & 1) ? 4 : 8;
 #else
 			vp->neighbors = 8;
@@ -662,7 +688,8 @@ init_voters(ModeInfo * mi)
 	if (vp->pixelmode)
 		vp->nparties = 2;
 
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	vp->busyLoop = 0;
+	MI_CLEARWINDOW(mi);
 
 	for (i = 0; i < BITMAPS; i++)
 		vp->number_in_party[i] = 0;
@@ -687,6 +714,13 @@ draw_voters(ModeInfo * mi)
 	int         new_opinion, old_opinion;
 	cellstruct  info;
 
+	if (vp->busyLoop) {
+		if (vp->busyLoop >= 5000)
+			vp->busyLoop = 0;
+		else
+			vp->busyLoop++;
+		return;
+	}
 	for (i = 0; i < BITMAPS; i++)
 		if (vp->number_in_party[i] == vp->npositions) {		/* The End of the WORLD */
 			init_voters(mi);	/* Create a more interesting planet */
@@ -724,8 +758,8 @@ draw_voters(ModeInfo * mi)
 	vp->generation++;
 	for (i = 0; i < BITMAPS; i++)
 		if (vp->number_in_party[i] == vp->npositions) {		/* The End of the WORLD */
+			vp->busyLoop = 1;
 			refresh_voters(mi);
-			MI_PAUSE(mi) = 1000000;
 		}
 }
 
@@ -748,19 +782,3 @@ release_voters(ModeInfo * mi)
 		voters = NULL;
 	}
 }
-
-void
-refresh_voters(ModeInfo * mi)
-{
-	voterstruct *vp = &voters[MI_SCREEN(mi)];
-	int         col, row, colrow;
-
-	for (row = 0; row < vp->nrows; row++)
-		for (col = 0; col < vp->ncols; col++) {
-			colrow = col + row * vp->ncols;
-			/* Draw all old, will get corrected soon if wrong... */
-			drawcell(mi, col, row,
-				 (unsigned long) (MI_NPIXELS(mi) * vp->arr[colrow] / BITMAPS),
-				 vp->arr[colrow], False);
-		}
-}
