@@ -2,7 +2,7 @@
 /* wator --- Dewdney's Wa-Tor, water torus simulation */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)wator.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)wator.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -47,6 +47,15 @@ static const char sccsid[] = "@(#)wator.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt wator_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   wator_description =
+{"wator", "init_wator", "draw_wator", "release_wator",
+ "refresh_wator", "init_wator", NULL, &wator_opts,
+ 750000, 1, 32767, 0, 64, 1.0, "",
+ "Shows Dewdney's Water-Torus planet of fish and sharks", 0, NULL};
+
+#endif
+
 extern int  neighbors;
 
 #include "bitmaps/fish-0.xbm"
@@ -125,11 +134,13 @@ typedef struct {
 	CellList   *currkind, *babykind, *lastkind[KINDS + 1], *firstkind[KINDS + 1];
 	CellList  **arr;	/* 0=empty or pts to a fish or shark */
 	int         neighbors;
-	XPoint      hexagonList[6];
-	XPoint      triangleList[2][3];
+	union {
+		XPoint      hexagon[6];
+		XPoint      triangle[2][3];
+	} shape;
 } watorstruct;
 
-static int  initVal[NEIGHBORKINDS] =
+static char plots[NEIGHBORKINDS] =
 {
 	3, 4, 6, 8, 9, 12	/* Neighborhoods */
 };
@@ -150,7 +161,7 @@ drawshape(ModeInfo * mi, int x, int y, int sizex, int sizey,
 	GC          gc = MI_GC(mi);
 
 	if (sides == 4 && sizex == 0 && sizey == 0) {
-		XPutImage(display, window, gc, &logo[shape], 0, 0,
+		(void) XPutImage(display, window, gc, &logo[shape], 0, 0,
 		x - icon_width, y - icon_height / 2, icon_width, icon_height);
 	} else if (sizex < 3 || sizey < 3 || (sides == 4 && shape == 1)) {
 		XFillRectangle(display, window, gc,
@@ -177,32 +188,32 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 	unsigned long colour;
 
 	if (!alive)
-		colour = MI_WIN_BLACK_PIXEL(mi);
+		colour = MI_BLACK_PIXEL(mi);
 	else if (MI_NPIXELS(mi) > 2)
 		colour = MI_PIXEL(mi, color);
 	else
-		colour = MI_WIN_WHITE_PIXEL(mi);
+		colour = MI_WHITE_PIXEL(mi);
 	XSetForeground(display, gc, colour);
 	if (wp->neighbors == 6) {
 		int         ccol = 2 * col + !(row & 1), crow = 2 * row;
 
-		wp->hexagonList[0].x = wp->xb + ccol * wp->xs;
-		wp->hexagonList[0].y = wp->yb + crow * wp->ys;
+		wp->shape.hexagon[0].x = wp->xb + ccol * wp->xs;
+		wp->shape.hexagon[0].y = wp->yb + crow * wp->ys;
 		if (wp->xs == 1 && wp->ys == 1)
 			XFillRectangle(display, window, gc,
-			   wp->hexagonList[0].x, wp->hexagonList[0].y, 1, 1);
+			wp->shape.hexagon[0].x, wp->shape.hexagon[0].y, 1, 1);
 		else if (bitmap >= KINDBITMAPS || !alive)
 			XFillPolygon(display, window, gc,
-			      wp->hexagonList, 6, Convex, CoordModePrevious);
+			    wp->shape.hexagon, 6, Convex, CoordModePrevious);
 		else {
-			XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+			XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 			XFillPolygon(display, window, gc,
-			      wp->hexagonList, 6, Convex, CoordModePrevious);
+			    wp->shape.hexagon, 6, Convex, CoordModePrevious);
 			XSetForeground(display, gc, colour);
 			if (wp->xs <= 6 || wp->ys <= 2)
 				XFillRectangle(display, window, gc,
-				       wp->hexagonList[0].x - 3 * wp->xs / 4,
-					       wp->hexagonList[0].y + wp->ys / 4, wp->xs, wp->ys);
+				     wp->shape.hexagon[0].x - 3 * wp->xs / 4,
+					       wp->shape.hexagon[0].y + wp->ys / 4, wp->xs, wp->ys);
 			else
 				XFillArc(display, window, gc,
 				     wp->xb + wp->xs * ccol - 3 * wp->xs / 4,
@@ -217,7 +228,7 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 				wp->xb + wp->xs * col, wp->yb + wp->ys * row,
 					       wp->xs, wp->ys);
 			else {
-				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 				XFillRectangle(display, window, gc,
 				wp->xb + wp->xs * col, wp->yb + wp->ys * row,
 					       wp->xs, wp->ys);
@@ -227,31 +238,31 @@ drawcell(ModeInfo * mi, int col, int row, unsigned long color, int bitmap,
 					 0, 23040);
 			}
 		} else
-			XPutImage(display, window, gc,
+			(void) XPutImage(display, window, gc,
 				  &logo[bitmap], 0, 0,
 				wp->xb + wp->xs * col, wp->yb + wp->ys * row,
 				  icon_width, icon_height);
 	} else {		/* TRI */
 		int         orient = (col + row) % 2;	/* O left 1 right */
 
-		wp->triangleList[orient][0].x = wp->xb + col * wp->xs;
-		wp->triangleList[orient][0].y = wp->yb + row * wp->ys;
+		wp->shape.triangle[orient][0].x = wp->xb + col * wp->xs;
+		wp->shape.triangle[orient][0].y = wp->yb + row * wp->ys;
 		if (wp->xs <= 3 || wp->ys <= 3)
 			XFillRectangle(display, window, gc,
-			 ((orient) ? -1 : 1) + wp->triangleList[orient][0].x,
-				       wp->triangleList[orient][0].y, 1, 1);
+			((orient) ? -1 : 1) + wp->shape.triangle[orient][0].x,
+				       wp->shape.triangle[orient][0].y, 1, 1);
 		else {
 			if (orient)
-				wp->triangleList[orient][0].x += (wp->xs / 2 - 1);
+				wp->shape.triangle[orient][0].x += (wp->xs / 2 - 1);
 			else
-				wp->triangleList[orient][0].x -= (wp->xs / 2 - 1);
+				wp->shape.triangle[orient][0].x -= (wp->xs / 2 - 1);
 			if (bitmap >= KINDBITMAPS || !alive)
 				XFillPolygon(display, window, gc,
-					     wp->triangleList[orient], 3, Convex, CoordModePrevious);
+					     wp->shape.triangle[orient], 3, Convex, CoordModePrevious);
 			else {
-				XSetForeground(display, gc, MI_WIN_BLACK_PIXEL(mi));
+				XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 				XFillPolygon(display, window, gc,
-					     wp->triangleList[orient], 3, Convex, CoordModePrevious);
+					     wp->shape.triangle[orient], 3, Convex, CoordModePrevious);
 				XSetForeground(display, gc, colour);
 				XFillArc(display, window, gc,
 				     wp->xb + wp->xs * col - 4 * wp->xs / 5 +
@@ -612,13 +623,13 @@ init_wator(ModeInfo * mi)
 		wp->height = 2;
 
 	for (i = 0; i < NEIGHBORKINDS; i++) {
-		if (neighbors == initVal[i]) {
+		if (neighbors == plots[i]) {
 			wp->neighbors = neighbors;
 			break;
 		}
 		if (i == NEIGHBORKINDS - 1) {
 #if 0
-			wp->neighbors = initVal[NRAND(NEIGHBORKINDS)];
+			wp->neighbors = plots[NRAND(NEIGHBORKINDS)];
 			wp->neighbors = (LRAND() & 1) ? 4 : 8;
 #else
 			wp->neighbors = 4;
@@ -654,8 +665,8 @@ init_wator(ModeInfo * mi)
 		wp->xb = (wp->width - wp->xs * nccols) / 2 + wp->xs / 2;
 		wp->yb = (wp->height - wp->ys * (ncrows / 2) * 2) / 2 + wp->ys;
 		for (i = 0; i < 6; i++) {
-			wp->hexagonList[i].x = (wp->xs - 1) * hexagonUnit[i].x;
-			wp->hexagonList[i].y = ((wp->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
+			wp->shape.hexagon[i].x = (wp->xs - 1) * hexagonUnit[i].x;
+			wp->shape.hexagon[i].y = ((wp->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
 		}
 	} else if (wp->neighbors == 4 || wp->neighbors == 8) {
 		if (wp->width < 2)
@@ -716,9 +727,9 @@ init_wator(ModeInfo * mi)
 		wp->yb = (wp->height - wp->ys * wp->nrows) / 2 + wp->ys / 2;
 		for (orient = 0; orient < 2; orient++) {
 			for (i = 0; i < 3; i++) {
-				wp->triangleList[orient][i].x =
+				wp->shape.triangle[orient][i].x =
 					(wp->xs - 2) * triangleUnit[orient][i].x;
-				wp->triangleList[orient][i].y =
+				wp->shape.triangle[orient][i].y =
 					(wp->ys - 2) * triangleUnit[orient][i].y;
 			}
 		}
@@ -744,7 +755,8 @@ init_wator(ModeInfo * mi)
 		wp->breed[FISH] = 4;
 	wp->sstarve = 3;
 
-	XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+	MI_CLEARWINDOW(mi);
+
 	for (kind = FISH; kind <= SHARK; kind++) {
 		i = 0;
 		while (i < wp->nkind[kind]) {
