@@ -2,7 +2,7 @@
 /* worm --- draw wiggly worms */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)worm.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)worm.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -62,6 +62,15 @@ static const char sccsid[] = "@(#)worm.c	4.04 97/07/28 xlockmore";
 ModeSpecOpt worm_opts =
 {0, NULL, 0, NULL, NULL};
 
+#ifdef USE_MODULES
+ModStruct   worm_description =
+{"worm", "init_worm", "draw_worm", "release_worm",
+ "refresh_worm", "init_worm", NULL, &worm_opts,
+ 17000, -20, 10, -3, 64, 1.0, "",
+ "Shows wiggly worms", 0, NULL};
+
+#endif
+
 #define MINSIZE 1
 
 #define SEGMENTS  36
@@ -92,7 +101,7 @@ typedef struct {
 	int         nw;
 	int         circsize;
 	wormstuff  *worm;
-	XRectangle *rects;	/* [NUMCOLORS * batchcount/NUMCOLORS+1] */
+	XRectangle *rects;
 	int         maxsize;
 	int        *size;
 	unsigned int chromo;
@@ -282,8 +291,6 @@ init_worm(ModeInfo * mi)
 		wp->nc = 2;
 	else
 		wp->nc = MI_NPIXELS(mi);
-	if (wp->nc > NUMCOLORS)
-		wp->nc = NUMCOLORS;
 
 	free_worms(wp);
 	wp->nw = MI_BATCHCOUNT(mi);
@@ -295,12 +302,12 @@ init_worm(ModeInfo * mi)
 		wp->worm = (wormstuff *) malloc(wp->nw * sizeof (wormstuff));
 
 	if (!wp->size)
-		wp->size = (int *) malloc(NUMCOLORS * sizeof (int));
+		wp->size = (int *) malloc(MI_NPIXELS(mi) * sizeof (int));
 
 	wp->maxsize = (REDRAWSTEP + 1) * wp->nw;	/*  / wp->nc + 1; */
 	if (!wp->rects)
 		wp->rects =
-			(XRectangle *) malloc(wp->maxsize * NUMCOLORS * sizeof (XRectangle));
+			(XRectangle *) malloc(wp->maxsize * MI_NPIXELS(mi) * sizeof (XRectangle));
 
 
 	if (!init_table) {
@@ -354,11 +361,10 @@ init_worm(ModeInfo * mi)
 	}
 
 	if (MI_WIN_IS_INSTALL(mi) && MI_WIN_IS_USE3D(mi)) {
-		XSetForeground(MI_DISPLAY(mi), MI_GC(mi), MI_NONE_COLOR(mi));
-		XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-			       0, 0, wp->xsize, wp->ysize);
-	} else
-		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
+		MI_CLEARWINDOWCOLOR(mi, MI_NONE_COLOR(mi));
+	} else {
+		MI_CLEARWINDOW(mi);
+	}
 }
 
 void
@@ -398,7 +404,7 @@ draw_worm(ModeInfo * mi)
 			XFillRectangles(display, window, gc, &(wp->rects[i * wp->maxsize]), wp->size[i]);
 		}
 	} else {
-		XSetForeground(display, gc, MI_WIN_WHITE_PIXEL(mi));
+		XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
 		XFillRectangles(display, window, gc,
 				&(wp->rects[0]), wp->size[0]);
 	}
@@ -423,12 +429,12 @@ release_worm(ModeInfo * mi)
 void
 refresh_worm(ModeInfo * mi)
 {
-	if (MI_WIN_IS_USE3D(mi))
+	if (MI_WIN_IS_USE3D(mi)) {
 		/* The 3D code does drawing&clearing by XORing.  We do not
 		   want to go to too much trouble here to make it redraw
 		   correctly. */
-		XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi));
-	else if (worms != NULL) {
+		MI_CLEARWINDOW(mi);
+	} else if (worms != NULL) {
 		wormstruct *wp = &worms[MI_SCREEN(mi)];
 		int         i;
 
