@@ -7,7 +7,11 @@ static char *LockProcs[] =
   "bouboule",
   "bounce",
   "braid",
+  "bubble",
   "bug",
+#ifdef USE_GL
+  "cage",
+#endif
 #if defined( USE_XPM ) || defined( USE_XPMINC )
   "cartoon",
 #endif
@@ -20,9 +24,6 @@ static char *LockProcs[] =
   "demon",
   "dilemma",
   "drift",
-#ifdef USE_GL
-  "escher",
-#endif
   "eyes",
   "fadeplot",
   "flag",
@@ -54,6 +55,7 @@ static char *LockProcs[] =
   "marquee",
   "maze",
 #ifdef USE_GL
+  "moebius",
   "morph3d",
 #endif
   "mountain",
@@ -81,6 +83,7 @@ static char *LockProcs[] =
   "spline",
 #ifdef USE_GL
   "sproingies",
+  "stairs",
 #endif
   "star",
   "strange",
