@@ -83,12 +83,6 @@ extern char *ProgramName;
  *  Please send comments or fixes to jcarson@bcm.tmc.edu
  */
 
-static Display *d;
-static Atom wm_delete_window;
-static Atom wm_protocols;
-static Atom wm_state;
-static int  err_occurred = False;
-
 /*-
  *  I'm probably going to get some BadWindow errors as I kill clients
  *  which have multiple windows open and then try to kill them again
@@ -96,6 +90,8 @@ static int  err_occurred = False;
  *  The flag is set back to false in recurse_tree and kill_tree...
  */
 
+static int  err_occurred = False;
+
 static int
 err_handler(Display * display, XErrorEvent * err)
 {
@@ -108,10 +104,10 @@ err_handler(Display * display, XErrorEvent * err)
  */
 
 static int
-has_property(Window w, Atom prop)
+has_property(Display * display, Window window, Atom prop)
 {
 	int         nprops, j, retval = 0;
-	Atom       *list = XListProperties(d, w, &nprops);
+	Atom       *list = XListProperties(display, window, &nprops);
 
 	if (err_occurred)
 		return 0;
@@ -129,36 +125,25 @@ has_property(Window w, Atom prop)
 	return retval;
 }
 
-static int
-has_wm_state(Window w)
-{
-	return has_property(w, wm_state);
-}
-
-static int
-has_wm_protocols(Window w)
-{
-	return has_property(w, wm_protocols);
-}
-
 /*-
  *  Send a WM_PROTOCOLS WM_DELETE_WINDOW message to a window
  */
 
 static void
-send_delete_message(Window w)
+send_delete_message(Display * display, Window window,
+		    Atom protocols_atom, Atom delete_window_atom)
 {
 	XClientMessageEvent xclient;
 
 	xclient.type = ClientMessage;
 	xclient.send_event = True;
-	xclient.display = d;
-	xclient.window = w;
-	xclient.message_type = wm_protocols;
+	xclient.display = display;
+	xclient.window = window;
+	xclient.message_type = protocols_atom;
 	xclient.format = 32;
-	xclient.data.l[0] = wm_delete_window;
+	xclient.data.l[0] = delete_window_atom;
 
-	XSendEvent(d, w, False, 0, (XEvent *) & xclient);
+	XSendEvent(display, window, False, 0, (XEvent *) & xclient);
 }
 
 /*-
@@ -168,28 +153,30 @@ send_delete_message(Window w)
  */
 
 static void
-handle_top_level(Window w)
+handle_top_level(Display * display, Window window,
+		 Atom protocols_atom, Atom delete_window_atom)
 {
 	Atom       *prots;
 	int         nprots, j;
 
-	if (has_wm_protocols(w)) {
-		XGetWMProtocols(d, w, &prots, &nprots);
+	if (has_property(display, window, protocols_atom)) {
+		XGetWMProtocols(display, window, &prots, &nprots);
 
 		if (err_occurred)
 			return;
 
 		for (j = 0; j < nprots; j++)
-			if (prots[j] == wm_delete_window) {
-				send_delete_message(w);
+			if (prots[j] == delete_window_atom) {
+				send_delete_message(display, window,
+					 protocols_atom, delete_window_atom);
 				break;
 			}
 		if (j == nprots)	/* delete window not found */
-			XKillClient(d, w);
+			XKillClient(display, window);
 
 		XFree((caddr_t) prots);
 	} else
-		XKillClient(d, w);
+		XKillClient(display, window);
 }
 
 /*-
@@ -201,31 +188,33 @@ handle_top_level(Window w)
  */
 
 static void
-recurse_tree(Window w)
+recurse_tree(Display * display, Window window,
+	     Atom state_atom, Atom protocols_atom, Atom delete_window_atom)
 {
 	Window      root, parent, *kids;
 	unsigned int nkids;
 	int         j;
-	int         wm_state;
+	int         swm_state;
 
 	for (;;) {
-		XQueryTree(d, w, &root, &parent, &kids, &nkids);
+		XQueryTree(display, window, &root, &parent, &kids, &nkids);
 		if (err_occurred) {
 			err_occurred = False;
 			return;
 		}
 		for (j = 0; j < nkids; j++) {
-			wm_state = has_wm_state(kids[j]);
+			swm_state = has_property(display, kids[j], state_atom);
 
 			if (err_occurred)
 				break;
 
-			if (wm_state) {
-				handle_top_level(kids[j]);
+			if (swm_state) {
+				handle_top_level(display, kids[j], protocols_atom, delete_window_atom);
 				if (err_occurred)
 					break;
 			} else
-				recurse_tree(kids[j]);
+				recurse_tree(display, kids[j],
+					     state_atom, protocols_atom, delete_window_atom);
 		}
 
 		XFree((caddr_t) kids);
@@ -245,20 +234,20 @@ recurse_tree(Window w)
  */
 
 static void
-kill_tree(Window w)
+kill_tree(Display * display, Window window)
 {
 	Window      root, parent, *kids;
 	unsigned int nkids;
 	int         j;
 
 	for (;;) {
-		XQueryTree(d, w, &root, &parent, &kids, &nkids);
+		XQueryTree(display, window, &root, &parent, &kids, &nkids);
 		if (err_occurred) {
 			err_occurred = False;
 			return;
 		}
 		for (j = 0; j < nkids; j++) {
-			XKillClient(d, kids[j]);
+			XKillClient(display, kids[j]);
 			if (err_occurred)
 				break;
 		}
@@ -279,40 +268,38 @@ kill_tree(Window w)
  */
 
 static void
-closedownLogout(void)
+closedownLogout(Display * display)
 {
+	Atom        __SWM_DELETE_WINDOW = None;
+	Atom        __SWM_PROTOCOLS = None;
+	Atom        __SWM_STATE = None;
 	int         j;
-	extern Display *dsp;
-
-	/* init global variables */
-
-	d = dsp;		/* hack to allow a display from another module */
-	wm_state = XInternAtom(d, "WM_STATE", False);
-	wm_protocols = XInternAtom(d, "WM_PROTOCOLS", False);
-	wm_delete_window = XInternAtom(d, "WM_DELETE_WINDOW", False);
 
+#if 0
 	/* synchronize -- so I'm aware of errors immediately */
-
-	XSynchronize(d, True);
+	XSynchronize(display, True);
 
 	/* use my error handler from here on out */
-
 	(void) XSetErrorHandler(err_handler);
+#endif
+
+	/* init atoms */
+	__SWM_STATE = XInternAtom(display, "__SWM_STATE", False);
+	__SWM_PROTOCOLS = XInternAtom(display, "__SWM_PROTOCOLS", False);
+	__SWM_DELETE_WINDOW = XInternAtom(display, "__SWM_DELETE_WINDOW", False);
 
 	/* start looking for windows to kill -- be nice on pass 1 */
-
-	for (j = 0; j < ScreenCount(d); j++)
-		recurse_tree(RootWindow(d, j));
+	for (j = 0; j < ScreenCount(display); j++)
+		recurse_tree(display, RootWindow(display, j),
+			  __SWM_STATE, __SWM_PROTOCOLS, __SWM_DELETE_WINDOW);
 
 	/* wait for things to clean themselves up */
-
 	(void) sleep(NAP_TIME);
 
 	/* this will forcibly kill anything that's still around --
 	   this second pass may or may not be needed... */
-
-	for (j = 0; j < ScreenCount(d); j++)
-		kill_tree(RootWindow(d, j));
+	for (j = 0; j < ScreenCount(display); j++)
+		kill_tree(display, RootWindow(display, j));
 	(void) sleep(NAP_TIME);
 }
 
@@ -373,19 +360,22 @@ uglyLogout(void)
 }
 
 void
-logoutUser(void)
+logoutUser(Display * display)
 {
 
 #if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
-	extern Display *dsp;
 	extern      syslogStop(char *);
 
 	syslog(SYSLOG_INFO, "%s: expired. closing down (uid %d) on %s\n",
 	       ProgramName, getuid(), getenv("DISPLAY"));
-	syslogStop(XDisplayString(dsp));
+	syslogStop(XDisplayString(display));
 #endif
 
-	(void) finish();
+#ifdef CLOSEDOWN_LOGOUT
+	(void) finish(display, False);
+#else
+	(void) finish(display, True);
+#endif
 #ifdef VMS
 	(void) system("mcr decw$endsession -noprompt");
 #else
@@ -396,7 +386,9 @@ logoutUser(void)
 #endif
 
 #ifdef CLOSEDOWN_LOGOUT
-	closedownLogout();
+  /* Do not want to kill other user's processes like if telnetted in */
+	closedownLogout(display);
+	return;
 #endif
 #ifdef SESSION_LOGOUT
 	sessionLogout();
@@ -439,7 +431,11 @@ logoutUser(void)
 #if HAVE_LIMITS_H
 #include <limits.h>
 #endif
+#ifdef NGROUPS_MAX
 #define NGROUPS NGROUPS_MAX
+#else
+#define NGROUPS NGROUPS_MAX_DEFAULT
+#endif
 #endif
 
 int
@@ -465,7 +461,7 @@ fullLock(void)
 	/* Do not try to logout root! */
 	if (!uid)
 		return (FULL_LOCK);	/* root */
-	if (inwindow || inroot || nolock || debug)
+	if (inwindow || inroot || nolock /*|| debug */ )
 		return (FULL_LOCK);	/* (mostly) harmless user */
 
 	pwp = getpwuid(uid);
