@@ -45,7 +45,9 @@ typedef struct LockStruct_s {
 	int         def_batchcount;
 	int         def_cycles;
 	int         def_size;
+	int         def_ncolors;
 	float       def_saturation;
+	char       *def_imagefile;
 	char       *desc;	/* text description of mode */
 	unsigned int flags;	/* state flags for this mode */
 	void       *userdata;	/* for use by the mode */
@@ -53,6 +55,33 @@ typedef struct LockStruct_s {
 
 #define LS_FLAG_INITED		1
 
+typedef struct {
+	Visual     *visual;
+	int         visualclass;	/* visual class name of current window */
+	unsigned int depth;
+	unsigned long red_mask, green_mask, blue_mask;	/* masks of current window */
+	int         colormap_size;	/* colormap of current window */
+	Colormap    colormap;	/* current colormap */
+	Window      window;	/* window used to cover screen */
+	Window      icon;	/* window used during password typein */
+	Window      root;	/* convenience pointer to the root window */
+#ifdef USE_BUTTON_LOGOUT
+	Window      button;
+
+#endif
+	GC          gc;		/* graphics context for animation */
+	GC          textgc;	/* graphics context used for text rendering */
+	GC          msgtextgc;	/* graphics context for message rendering */
+	XPoint      iconpos;	/* location of top left edge of icon */
+	XPoint      msgpos;	/* location of top left edge of message */
+	int         npixels;	/* number of valid entries in pixels */
+	unsigned long *pixels;	/* pixel values in the colormap */
+	unsigned long black_pixel, white_pixel;		/* black and white pixel values */
+	unsigned long bg_pixel, fg_pixel;	/* background and foreground pixel values */
+	unsigned long right_pixel, left_pixel;	/* 3D color pixel values */
+	unsigned long none_pixel, both_pixel;
+} ScreenInfo;
+
 typedef struct {
 	Display    *display;	/* handle to X display */
 	Screen     *screenptr;	/* ptr to screen info */
@@ -61,16 +90,8 @@ typedef struct {
 	int         num_screens;	/* number screens locked */
 	int         max_screens;	/* max # active screens */
 	Window      window;	/* handle to current window */
-	int         win_width;	/* width of current window */
-	int         win_height;	/* height of current window */
-	int         win_depth;	/* depth of current window */
-	Visual     *visual;	/* visual of current window */
-	int         visualclass;	/* visual class name of current window */
-	int         colormapsize;	/* colormap of current window */
-	int         redmask, greenmask, bluemask;	/* masks of current window */
-	Colormap    colormap;	/* default colormap of current window */
-	unsigned long black_pixel;	/* pixel value for black */
-	unsigned long white_pixel;	/* pixel value for white */
+	int         width;	/* width of current window */
+	int         height;	/* height of current window */
 	unsigned int flags;	/* xlock window flags */
 	float       delta3d;
 } WindowInfo;
@@ -94,13 +115,15 @@ typedef struct {
 	long        delay;	/* inputs, current settings */
 	long        batchcount;
 	long        cycles;
+	long        ncolors;
 	long        size;
 	float       saturation;
+	char       *imagefile;
 } RunInfo;
 
 typedef struct ModeInfo_s {
 	WindowInfo  windowinfo;
-	perscreen  *screeninfo;
+	ScreenInfo *screeninfo;
 	RunInfo     runinfo;
 	struct LockStruct_s *lockstruct;
 } ModeInfo;
@@ -129,18 +152,8 @@ typedef struct ModeInfo_s {
 #define MI_NUM_SCREENS(mi)	((mi)->windowinfo.num_screens)
 #define MI_MAX_SCREENS(mi)	((mi)->windowinfo.max_screens)
 #define MI_WINDOW(mi)		((mi)->windowinfo.window)
-#define MI_WIN_WIDTH(mi)	((mi)->windowinfo.win_width)
-#define MI_WIN_HEIGHT(mi)	((mi)->windowinfo.win_height)
-#define MI_WIN_DEPTH(mi)	((mi)->windowinfo.win_depth)
-#define MI_VISUAL(mi)	((mi)->windowinfo.visual)
-#define MI_VISUALCLASS(mi)	((mi)->windowinfo.visualclass)
-#define MI_COLORMAPSIZE(mi)	((mi)->windowinfo.colormapsize)
-#define MI_REDMASK(mi)	((mi)->windowinfo.redmask)
-#define MI_GREENMASK(mi)	((mi)->windowinfo.greenmask)
-#define MI_BLUEMASK(mi)	((mi)->windowinfo.bluemask)
-#define MI_WIN_COLORMAP(mi)	((mi)->windowinfo.colormap)
-#define MI_WIN_BLACK_PIXEL(mi)	((mi)->windowinfo.black_pixel)
-#define MI_WIN_WHITE_PIXEL(mi)	((mi)->windowinfo.white_pixel)
+#define MI_WIN_WIDTH(mi)	((mi)->windowinfo.width)
+#define MI_WIN_HEIGHT(mi)	((mi)->windowinfo.height)
 #define MI_DELTA3D(mi)	((mi)->windowinfo.delta3d)
 #define MI_WIN_FLAGS(mi)	((mi)->windowinfo.flags)
 #define MI_WIN_SET_FLAG_STATE(mi,f,bool) ((mi)->windowinfo.flags = \
@@ -160,38 +173,80 @@ typedef struct ModeInfo_s {
 #define MI_WIN_IS_FULLRANDOM(mi) (MI_WIN_FLAG_IS_SET (mi, WI_FLAG_FULLRANDOM))
 #define MI_WIN_IS_WIREFRAME(mi)	(MI_WIN_FLAG_IS_SET (mi, WI_FLAG_WIREFRAME))
 
-#define MI_PERSCREEN(mi)	((mi)->screeninfo)
+#define MI_SCREENINFO(mi)	((mi)->screeninfo)
+#define MI_DEPTH(mi)	((mi)->screeninfo->depth)
+#define MI_VISUAL(mi)	((mi)->screeninfo->visual)
+#define MI_VISUALCLASS(mi)	((mi)->screeninfo->visualclass)
+#define MI_COLORMAP_SIZE(mi)	((mi)->screeninfo->colormap_size)
+#define MI_RED_MASK(mi)	((mi)->screeninfo->red_mask)
+#define MI_GREEN_MASK(mi)	((mi)->screeninfo->green_mask)
+#define MI_BLUE_MASK(mi)	((mi)->screeninfo->blue_mask)
+#define MI_COLORMAP(mi)		((mi)->screeninfo->colormap)
 #define MI_GC(mi)		((mi)->screeninfo->gc)
 #define MI_NPIXELS(mi)		((mi)->screeninfo->npixels)
-#define MI_CMAP(mi)		((mi)->screeninfo->cmap)
 #define MI_PIXELS(mi)		((mi)->screeninfo->pixels)
 #define MI_PIXEL(mi,n)		((mi)->screeninfo->pixels[n])
+#define MI_BLACK_PIXEL(mi)	((mi)->screeninfo->black_pixel)
+#define MI_WHITE_PIXEL(mi)	((mi)->screeninfo->white_pixel)
+#define MI_BG_PIXEL(mi)		((mi)->screeninfo->bg_pixel)
+#define MI_FG_PIXEL(mi)		((mi)->screeninfo->fg_pixel)
+#define MI_NONE_COLOR(mi)	((mi)->screeninfo->none_pixel)	/* -install */
+#define MI_RIGHT_COLOR(mi)	((mi)->screeninfo->right_pixel)
+#define MI_LEFT_COLOR(mi)	((mi)->screeninfo->left_pixel)
 
-/* are these of interest to modes? */
-#define MI_BG_COLOR(mi)		((mi)->screeninfo->bgcol)
-#define MI_FG_COLOR(mi)		((mi)->screeninfo->fgcol)
-#define MI_NONE_COLOR(mi)	((mi)->screeninfo->nonecol)	/* -install */
-#define MI_RIGHT_COLOR(mi)	((mi)->screeninfo->rightcol)
-#define MI_LEFT_COLOR(mi)	((mi)->screeninfo->leftcol)
-
-#define MI_PAUSE(mi)		((mi)->runinfo.pause)
 #define MI_DELAY(mi)		((mi)->runinfo.delay)
 #define MI_BATCHCOUNT(mi)	((mi)->runinfo.batchcount)
 #define MI_CYCLES(mi)		((mi)->runinfo.cycles)
 #define MI_SIZE(mi)		((mi)->runinfo.size)
+#define MI_NCOLORS(mi)		((mi)->runinfo.ncolors)
 #define MI_SATURATION(mi)	((mi)->runinfo.saturation)
+#define MI_IMAGEFILE(mi)	((mi)->runinfo.imagefile)
 
 #define MI_LOCKSTRUCT(mi)	((mi)->lockstruct)
 #define MI_DEFDELAY(mi)		((mi)->lockstruct->def_delay)
 #define MI_DEFBATCHCOUNT(mi)	((mi)->lockstruct->def_batchcount)
 #define MI_DEFCYCLES(mi)	((mi)->lockstruct->def_cycles)
 #define MI_DEFSIZE(mi)	((mi)->lockstruct->def_size)
+#define MI_DEFNCOLORS(mi)	((mi)->lockstruct->def_ncolors)
 #define MI_DEFSATURATION(mi)	((mi)->lockstruct->def_saturation)
+#define MI_DEFIMAGEFILE(mi)	((mi)->lockstruct->def_imagefile)
 
 #define MI_NAME(mi)		((mi)->lockstruct->cmdline_arg)
 #define MI_DESC(mi)		((mi)->lockstruct->desc)
 #define MI_USERDATA(mi)		((mi)->lockstruct->userdata)
 
+#ifdef SOLARIS2
+#define MI_CLEARWINDOWCOLORMAP(mi, gc, pixel) \
+ XSetForeground(MI_DISPLAY(mi), gc, pixel); \
+ if ((MI_WIN_WIDTH(mi) >= 2) || (MI_WIN_HEIGHT(mi) >= 2)) { \
+ XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc, 1, 1, \
+  (unsigned int) MI_WIN_WIDTH(mi) - 2, (unsigned int) MI_WIN_HEIGHT(mi) - 2); \
+ XDrawRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc, 0, 0, \
+  (unsigned int) MI_WIN_WIDTH(mi) - 1, (unsigned int) MI_WIN_HEIGHT(mi) - 1); \
+ } else \
+ XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc, 0, 0, \
+  (unsigned int) MI_WIN_WIDTH(mi), (unsigned int) MI_WIN_HEIGHT(mi))
+#else
+/*-
+ * It was acting weird with the first rectangle drawn after this
+ * on a Ultra2 with TrueColor Solaris2.6 (deco for example)
+ */
+#define MI_CLEARWINDOWCOLORMAP(mi, gc, pixel) \
+ XSetForeground(MI_DISPLAY(mi), gc, pixel); \
+ XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi), gc, \
+   0, 0, (unsigned int) MI_WIN_WIDTH(mi), (unsigned int) MI_WIN_HEIGHT(mi))
+#endif
+#define MI_CLEARWINDOWCOLOR(mi, pixel) \
+ MI_CLEARWINDOWCOLORMAP(mi, MI_GC(mi), pixel)
+#if 1
+/*-
+ * XClearWindow with GL or MONO does not always work.
+ */
+#define MI_CLEARWINDOW(mi) MI_CLEARWINDOWCOLOR(mi, MI_BLACK_PIXEL(mi))
+#else
+#define MI_CLEARWINDOW(mi) XClearWindow(MI_DISPLAY(mi), MI_WINDOW(mi))
+#endif
+
 /* -------------------------------------------------------------------- */
 
 extern HookProc call_init_hook;
@@ -204,6 +259,7 @@ extern void set_default_mode(LockStruct *);
 extern void release_last_mode(ModeInfo *);
 
 /* -------------------------------------------------------------------- */
+#ifndef USE_MODULES
 
 extern ModeHook init_ant;
 extern ModeHook draw_ant;
@@ -253,6 +309,21 @@ extern ModeHook release_bug;
 extern ModeHook refresh_bug;
 extern ModeSpecOpt bug_opts;
 
+extern ModeHook init_bubble;
+extern ModeHook draw_bubble;
+extern ModeHook release_bubble;
+extern ModeHook refresh_bubble;
+extern ModeSpecOpt bubble_opts;
+
+#ifdef USE_GL
+extern ModeHook init_cage;
+extern ModeHook draw_cage;
+extern ModeHook release_cage;
+extern ModeHook change_cage;
+extern ModeSpecOpt cage_opts;
+
+#endif
+
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 extern ModeHook init_cartoon;
 extern ModeHook draw_cartoon;
@@ -315,15 +386,6 @@ extern ModeHook release_drift;
 extern ModeHook refresh_drift;
 extern ModeSpecOpt drift_opts;
 
-#ifdef USE_GL
-extern ModeHook init_escher;
-extern ModeHook draw_escher;
-extern ModeHook release_escher;
-extern ModeHook change_escher;
-extern ModeSpecOpt escher_opts;
-
-#endif
-
 extern ModeHook init_eyes;
 extern ModeHook draw_eyes;
 extern ModeHook release_eyes;
@@ -352,9 +414,14 @@ extern ModeHook init_forest;
 extern ModeHook draw_forest;
 extern ModeHook release_forest;
 extern ModeHook refresh_forest;
-extern ModeHook refresh_forest;
 extern ModeSpecOpt forest_opts;
 
+extern ModeHook init_fusee;
+extern ModeHook draw_fusee;
+extern ModeHook release_fusee;
+extern ModeHook refresh_fusee;
+extern ModeSpecOpt fusee_opts;
+
 extern ModeHook init_galaxy;
 extern ModeHook draw_galaxy;
 extern ModeHook release_galaxy;
@@ -499,6 +566,12 @@ extern ModeHook refresh_maze;
 extern ModeSpecOpt maze_opts;
 
 #ifdef USE_GL
+extern ModeHook init_moebius;
+extern ModeHook draw_moebius;
+extern ModeHook release_moebius;
+extern ModeHook change_moebius;
+extern ModeSpecOpt moebius_opts;
+
 extern ModeHook init_morph3d;
 extern ModeHook draw_morph3d;
 extern ModeHook release_morph3d;
@@ -646,6 +719,13 @@ extern ModeHook release_sproingies;
 extern ModeHook refresh_sproingies;
 extern ModeSpecOpt sproingies_opts;
 
+extern ModeHook init_stairs;
+extern ModeHook draw_stairs;
+extern ModeHook release_stairs;
+extern ModeHook refresh_stairs;
+extern ModeHook change_stairs;
+extern ModeSpecOpt stairs_opts;
+
 #endif
 
 extern ModeHook init_star;
@@ -758,6 +838,17 @@ extern ModeHook change_random;
 extern ModeSpecOpt random_opts;
 
 extern LockStruct LockProcs[];
+
+#else /* #ifndef USE_MODULES */
+
+extern void LoadModules(char *);
+extern void UnloadModules();
+
+extern LockStruct *LockProcs;
+extern void **LoadedModules;	/* save handles on loaded modules for closing */
+
+#endif
+
 extern int  numprocs;
 
 /* -------------------------------------------------------------------- */
