@@ -21,15 +21,15 @@ static const char sccsid[] = "@(#)ras.c	4.00 97/01/01 xlockmore";
 #include <time.h>
 #include "ras.h"
 
-char       *imagefile;
+extern char *imagefile;
 
-#define COLORMAPSIZE 0x100
-#define RGBCOLORMAPSIZE 0x300
+#define COLORMAP_SIZE 0x100
+#define RGBCOLORMAP_SIZE 0x300
 typedef struct {
-	int         entry[COLORMAPSIZE];
+	int         entry[COLORMAP_SIZE];
 	unsigned char header[32];
-	unsigned char color[RGBCOLORMAPSIZE];
-	unsigned char red[COLORMAPSIZE], green[COLORMAPSIZE], blue[COLORMAPSIZE];
+	unsigned char color[RGBCOLORMAP_SIZE];
+	unsigned char red[COLORMAP_SIZE], green[COLORMAP_SIZE], blue[COLORMAP_SIZE];
 	unsigned char *data;
 	unsigned long sign;
 	unsigned long width, height;
@@ -59,32 +59,32 @@ convert_colors(ModeInfo * mi)
 	unsigned long black, white, fgpix, bgpix;
 	XColor      fgcol, bgcol;
 
-	black = MI_WIN_BLACK_PIXEL(mi);
-	white = MI_WIN_WHITE_PIXEL(mi);
-	fgpix = MI_FG_COLOR(mi);
-	bgpix = MI_BG_COLOR(mi);
+	black = MI_BLACK_PIXEL(mi);
+	white = MI_WHITE_PIXEL(mi);
+	fgpix = MI_FG_PIXEL(mi);
+	bgpix = MI_BG_PIXEL(mi);
 	fgcol.pixel = fgpix;
 	bgcol.pixel = bgpix;
-	XQueryColor(MI_DISPLAY(mi), MI_WIN_COLORMAP(mi), &fgcol);
-	XQueryColor(MI_DISPLAY(mi), MI_WIN_COLORMAP(mi), &bgcol);
+	XQueryColor(MI_DISPLAY(mi), MI_COLORMAP(mi), &fgcol);
+	XQueryColor(MI_DISPLAY(mi), MI_COLORMAP(mi), &bgcol);
 
 	/* Set these, if raster does not overwrite some, so much the better. */
-	if (fgpix < COLORMAPSIZE) {
+	if (fgpix < COLORMAP_SIZE) {
 		ras.red[fgpix] = fgcol.red >> 8;
 		ras.green[fgpix] = fgcol.green >> 8;
 		ras.blue[fgpix] = fgcol.blue >> 8;
 	}
-	if (bgpix < COLORMAPSIZE) {
+	if (bgpix < COLORMAP_SIZE) {
 		ras.red[bgpix] = bgcol.red >> 8;
 		ras.green[bgpix] = bgcol.green >> 8;
 		ras.blue[bgpix] = bgcol.blue >> 8;
 	}
-	if (white < COLORMAPSIZE) {
-		ras.red[white] = 255;
-		ras.green[white] = 255;
-		ras.blue[white] = 255;
+	if (white < COLORMAP_SIZE) {
+		ras.red[white] = 0xFF;
+		ras.green[white] = 0xFF;
+		ras.blue[white] = 0xFF;
 	}
-	if (black < COLORMAPSIZE) {
+	if (black < COLORMAP_SIZE) {
 		ras.red[black] = 0;
 		ras.green[black] = 0;
 		ras.blue[black] = 0;
@@ -106,9 +106,9 @@ convert_colors(ModeInfo * mi)
 static unsigned long
 find_nearest(int r, int g, int b)
 {
-	unsigned long i, minimum = RGBCOLORMAPSIZE, imin = 0, t;
+	unsigned long i, minimum = RGBCOLORMAP_SIZE, imin = 0, t;
 
-	for (i = 0; i < COLORMAPSIZE; i++) {
+	for (i = 0; i < COLORMAP_SIZE; i++) {
 		t = abs(r - (int) ras.red[i]) + abs(g - (int) ras.green[i]) +
 			abs(b - (int) ras.blue[i]);
 		if (t < minimum) {
@@ -124,15 +124,15 @@ find_nearest(int r, int g, int b)
 }
 
 unsigned long
-GetWhite(void)
+GetBlack(void)
 {
-	return ras.entry[find_nearest(0xff, 0xff, 0xff)];
+  return ras.entry[find_nearest(0, 0, 0)];
 }
 
 unsigned long
-GetBlack(void)
+GetWhite(void)
 {
-	return ras.entry[find_nearest(0, 0, 0)];
+  return ras.entry[find_nearest(0xff, 0xff, 0xff)];
 }
 
 unsigned long
@@ -141,7 +141,7 @@ GetColor(ModeInfo * mi, unsigned long pixel)
 	XColor      col;
 
 	col.pixel = pixel;
-	XQueryColor(MI_DISPLAY(mi), MI_WIN_COLORMAP(mi), &col);
+	XQueryColor(MI_DISPLAY(mi), MI_COLORMAP(mi), &col);
 	return ras.entry[find_nearest(col.red >> 8, col.green >> 8, col.blue >> 8)];
 }
 
@@ -205,10 +205,10 @@ RasterFileToImage(ModeInfo * mi, char *filename, XImage ** image)
 void
 SetImageColors(Display * display, Colormap cmap)
 {
-	XColor      xcolor[COLORMAPSIZE];
+	XColor      xcolor[COLORMAP_SIZE];
 	int         i;
 
-	for (i = 0; i < COLORMAPSIZE; i++) {
+	for (i = 0; i < COLORMAP_SIZE; i++) {
 		xcolor[i].flags = DoRed | DoGreen | DoBlue;
 		xcolor[i].red = ras.red[i] << 8;
 		xcolor[i].green = ras.green[i] << 8;
