@@ -1,5 +1,5 @@
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)resource.c	4.04 97/07/28 xlockmore";
+static const char sccsid[] = "@(#)resource.c	4.07 97/11/24 xlockmore";
 
 #endif
 
@@ -86,6 +86,12 @@ extern int  fullLock();
 
 extern char *getenv(const char *);
 
+#ifdef USE_MODULES
+#ifndef DEF_MODULEPATH
+#define DEF_MODULEPATH "/usr/lib/X11/xlock_modules"
+#endif
+#endif
+
 #ifndef DEF_FILESEARCHPATH
 #ifdef VMS
 #include <descrip>
@@ -133,6 +139,7 @@ static int  descr();
 #define DEF_BATCHCOUNT	"100"	/* vectors (or whatever) per batch */
 #define DEF_CYCLES	"1000"	/* timeout in cycles for a batch */
 #define DEF_SIZE	"0"	/* size, default if 0 */
+#define DEF_NCOLORS	"64"	/* maximum number of colors */
 #define DEF_SATURATION	"1.0"	/* color ramp saturation 0->1 */
 #define DEF_NICE	"10"	/* xlock process nicelevel */
 #define DEF_LOCKDELAY	"0"	/* secs until lock */
@@ -145,15 +152,26 @@ static int  descr();
 #endif /* !AIXV3 */
 #endif
 #define DEF_MSGFONT     "-adobe-courier-medium-r-*-*-14-*-*-*-m-*-iso8859-1"
+#ifdef USE_MB
+# define DEF_FONTSET	DEF_FONT ## ",-*-24-*"
+#endif
 #define DEF_BG		"White"
 #define DEF_FG		"Black"
 #define DEF_NAME	"Name: "
 #define DEF_PASS	"Password: "
+#ifdef SAFEWORD
+#define DEF_DPASS "Dynamic password: "
+#define DEF_FPASS "Fixed Password: "
+#define DEF_CHALL "Challenge: "
+#endif
 #define DEF_INFO	"Enter password to unlock; select icon to lock."
 #define DEF_VALID	"Validating login..."
 #define DEF_INVALID	"Invalid login."
 #define DEF_GEOMETRY	""
 #define DEF_ICONGEOMETRY	""
+#ifdef FX
+#define DEF_GLGEOMETRY ""
+#endif
 #define DEF_DELTA3D	"1.5"	/* space between things in 3d mode relative to their size */
 #ifndef DEF_MESSAGESFILE
 #define DEF_MESSAGESFILE	""
@@ -238,8 +256,8 @@ extern char *messagefile;
 extern char *message;
 extern char *mfont;
 
-/* For modes with images, images and puzzle support ras and xpm */
-extern char *imagefile;
+/* For modes with images: xbm, xpm, and ras */
+char       *imagefile;
 
 /* For automata modes */
 int         neighbors;
@@ -299,6 +317,8 @@ static XrmOptionDescRec genTable[] =
 	{"+grabserver", ".grabserver", XrmoptionNoArg, (caddr_t) "off"},
 	{"-install", ".install", XrmoptionNoArg, (caddr_t) "on"},
 	{"+install", ".install", XrmoptionNoArg, (caddr_t) "off"},
+	{"-mousemotion", ".mousemotion", XrmoptionNoArg, (caddr_t) "on"},
+	{"+mousemotion", ".mousemotion", XrmoptionNoArg, (caddr_t) "off"},
 	{"-sound", ".sound", XrmoptionNoArg, (caddr_t) "on"},
 	{"+sound", ".sound", XrmoptionNoArg, (caddr_t) "off"},
 	{"-timeelapsed", ".timeelapsed", XrmoptionNoArg, (caddr_t) "on"},
@@ -312,6 +332,9 @@ static XrmOptionDescRec genTable[] =
 	{"-timeout", ".timeout", XrmoptionSepArg, (caddr_t) NULL},
 	{"-font", ".font", XrmoptionSepArg, (caddr_t) NULL},
 	{"-msgfont", ".msgfont", XrmoptionSepArg, (caddr_t) NULL},
+#ifdef USE_MB
+	{"-fontset", ".fontset", XrmoptionSepArg, (caddr_t) NULL},
+#endif
 	{"-bg", ".background", XrmoptionSepArg, (caddr_t) NULL},
 	{"-fg", ".foreground", XrmoptionSepArg, (caddr_t) NULL},
 	{"-background", ".background", XrmoptionSepArg, (caddr_t) NULL},
@@ -323,6 +346,9 @@ static XrmOptionDescRec genTable[] =
 	{"-invalid", ".invalid", XrmoptionSepArg, (caddr_t) NULL},
 	{"-geometry", ".geometry", XrmoptionSepArg, (caddr_t) NULL},
 	{"-icongeometry", ".icongeometry", XrmoptionSepArg, (caddr_t) NULL},
+#ifdef FX
+	{"-glgeometry", ".glgeometry", XrmoptionSepArg, (caddr_t) NULL},
+#endif
 
 	{"-wireframe", ".wireframe", XrmoptionNoArg, (caddr_t) "on"},
 	{"+wireframe", ".wireframe", XrmoptionNoArg, (caddr_t) "off"},
@@ -341,15 +367,13 @@ static XrmOptionDescRec genTable[] =
 	{"-messagefile", ".messagefile", XrmoptionSepArg, (caddr_t) NULL},
 	{"-message", ".message", XrmoptionSepArg, (caddr_t) NULL},
 	{"-mfont", ".mfont", XrmoptionSepArg, (caddr_t) NULL},
-    /* For modes with images, images and puzzle support ras and xpm */
-	{"-imagefile", ".imagefile", XrmoptionSepArg, (caddr_t) NULL},
     /* For automata modes */
 	{"-neighbors", ".neighbors", XrmoptionSepArg, (caddr_t) NULL},
     /* For eyes and julia modes */
 	{"-mouse", ".mouse", XrmoptionNoArg, (caddr_t) "on"},
 	{"+mouse", ".mouse", XrmoptionNoArg, (caddr_t) "off"},
 
-#ifdef USE_XLOCKRC
+#if defined( USE_XLOCKRC ) || defined( FALLBACK_XLOCKRC )
 	{"-cpasswd", ".cpasswd", XrmoptionSepArg, (caddr_t) NULL},
 #endif
 #ifdef USE_AUTO_LOGOUT
@@ -382,7 +406,10 @@ static XrmOptionDescRec modeTable[] =
 	{"-batchcount", "*batchcount", XrmoptionSepArg, (caddr_t) NULL},
 	{"-cycles", "*cycles", XrmoptionSepArg, (caddr_t) NULL},
 	{"-size", "*size", XrmoptionSepArg, (caddr_t) NULL},
+	{"-ncolors", "*ncolors", XrmoptionSepArg, (caddr_t) NULL},
 	{"-saturation", "*saturation", XrmoptionSepArg, (caddr_t) NULL},
+    /* For modes with images, xbm, xpm, & ras */
+	{"-imagefile", "*imagefile", XrmoptionSepArg, (caddr_t) NULL},
 };
 
 #define modeEntries (sizeof modeTable / sizeof modeTable[0])
@@ -414,12 +441,23 @@ static XrmOptionDescRec nameTable[] =
 	{"-name", ".name", XrmoptionSepArg, (caddr_t) NULL},
 };
 
+#ifdef USE_MODULES
+static XrmOptionDescRec modulepathTable[] =
+{
+	{"-modulepath", ".modulepath", XrmoptionSepArg, (caddr_t) NULL},
+};
+
+#endif
+
 static OptionStruct opDesc[] =
 {
 	{"-help", "print out this message to standard output"},
 	{"-version", "print version number (if >= 4.00) to standard output"},
 	{"-resources", "print default resource file to standard output"},
 	{"-display displayname", "X server to contact"},
+#ifdef USE_MODULES
+	{"-modulepath", "directory where screensaver modules are stored"},
+#endif
 	{"-visual visualname", "X visual to use"},
 	{"-parent", "parent window id (for inwindow)"},
 {"-name resourcename", "class name to use for resources (default is XLock)"},
@@ -427,7 +465,10 @@ static OptionStruct opDesc[] =
 	{"-batchcount num", "number of things per batch"},
 	{"-cycles num", "number of cycles per batch"},
 	{"-size num", "size of a unit in a mode, default is 0"},
+	{"-ncolors num", "maximum number of colors, default is 64"},
 	{"-saturation value", "saturation of color ramp"},
+  /* For modes with images, xbm, xpm, & ras */
+	{"-imagefile filename", "image file"},
 	{"-/+nolock", "turn on/off no password required"},
 	{"-/+inwindow", "turn on/off making xlock run in a window"},
 	{"-/+inroot", "turn on/off making xlock run in the root window"},
@@ -447,6 +488,7 @@ static OptionStruct opDesc[] =
 	{"-/+grabserver", "turn on/off grabbing of server"},
 	{"-/+install", "whether to use private colormap if needed (yes/no)"},
 	{"-/+fullrandom", "turn on/off full random choice of mode-options"},
+	{"-/+mousemotion", "turn on/off sensitivity to mouse"},
 	{"-/+sound", "whether to use sound if configured for it (yes/no)"},
 	{"-/+timeelapsed", "turn on/off clock"},
 	{"-/+usefirst", "turn on/off using the first char typed in password"},
@@ -456,6 +498,9 @@ static OptionStruct opDesc[] =
 	{"-timeout seconds", "number of seconds before password times out"},
 	{"-font fontname", "font to use for password prompt"},
 	{"-msgfont fontname", "font to use for message"},
+#ifdef USE_MB
+	{"-fontset fontsetname", "fontset to use for Xmb..."},
+#endif
 	{"-bg color", "background color to use for password prompt"},
 	{"-fg color", "foreground color to use for password prompt"},
 	{"-background color", "background color to use for password prompt"},
@@ -467,7 +512,9 @@ static OptionStruct opDesc[] =
       {"-invalid string", "text string to use for invalid password message"},
 	{"-geometry geom", "geometry for non-full screen lock"},
    {"-icongeometry geom", "geometry for password window (location ignored)"},
-
+#ifdef FX
+	{"-glgeometry geom", "geometry for gl modes (location ignored)"},
+#endif
 	{"-/+wireframe", "turn on/off wireframe"},
 
 	{"-/+use3d", "turn on/off 3d view"},
@@ -483,14 +530,12 @@ static OptionStruct opDesc[] =
 	{"-messagefile filename", "file message to say"},
 	{"-message string", "message to say"},
 	{"-mfont mode-fontname", "font for a specific mode"},
-    /* For modes with images, images and puzzle support ras and xpm */
-	{"-imagefile filename", "image file"},
     /* For automata modes */
       {"-neighbors num", "squares 4 or 8, hexagons 6, triangles 3, 9 or 12"},
     /* For eyes and julia modes */
 	{"-/+mouse", "turn on/off the grabbing the mouse"},
 
-#ifdef USE_XLOCKRC
+#if defined( USE_XLOCKRC ) || defined( FALLBACK_XLOCKRC )
 	{"-cpasswd crypted-password", "text string of encrypted password"},
 #endif
 #ifdef USE_AUTO_LOGOUT
@@ -519,8 +564,14 @@ int         delay;
 int         batchcount;
 int         cycles;
 int         size;
+int         ncolors;
 float       saturation;
 
+#ifdef USE_MODULES
+char       *modulepath;
+
+#endif
+
 Bool        nolock;
 Bool        inwindow;
 Bool        inroot;
@@ -543,6 +594,7 @@ Bool        fullrandom;
 Bool        grabmouse;
 Bool        grabserver;
 Bool        install;
+Bool        mousemotion;
 Bool        sound;
 Bool        timeelapsed;
 Bool        usefirst;
@@ -558,16 +610,31 @@ int         timeout;
 
 char       *fontname;
 char       *messagefont;
+#ifdef USE_MB
+char	   *fontsetname;
+#endif
 char       *background;
 char       *foreground;
 char       *text_user;
 char       *text_pass;
+
+#ifdef SAFEWORD
+char       *text_dpass;
+char       *text_fpass;
+char       *text_chall;
+
+#endif
 char       *text_info;
 char       *text_valid;
 char       *text_invalid;
 char       *geometry;
 char       *icongeometry;
 
+#ifdef FX
+char       *glgeometry;
+
+#endif
+
 Bool        wireframe;
 
 Bool        use3d;
@@ -577,7 +644,7 @@ char       *right3d;
 char       *left3d;
 char       *both3d;
 
-#ifdef USE_XLOCKRC
+#if defined( USE_XLOCKRC ) || defined( FALLBACK_XLOCKRC)
 char       *cpasswd;
 
 #endif
@@ -615,15 +682,16 @@ static argtype genvars[] =
 	{(caddr_t *) & debug, "debug", "Debug", "off", t_Bool},
 	{(caddr_t *) & echokeys, "echokeys", "EchoKeys", "off", t_Bool},
     {(caddr_t *) & enablesaver, "enablesaver", "EnableSaver", "off", t_Bool},
-    {(caddr_t *) & resetsaver, "resetsaver", "ResetSaver", "on", t_Bool},
+	{(caddr_t *) & resetsaver, "resetsaver", "ResetSaver", "on", t_Bool},
 	{(caddr_t *) & fullrandom, "fullrandom", "FullRandom", "off", t_Bool},
 	{(caddr_t *) & grabmouse, "grabmouse", "GrabMouse", "on", t_Bool},
 	{(caddr_t *) & grabserver, "grabserver", "GrabServer", "off", t_Bool},
 	{(caddr_t *) & install, "install", "Install", "on", t_Bool},
+    {(caddr_t *) & mousemotion, "mousemotion", "MouseMotion", "off", t_Bool},
 	{(caddr_t *) & mono, "mono", "Mono", "off", t_Bool},
 	{(caddr_t *) & sound, "sound", "Sound", "off", t_Bool},
     {(caddr_t *) & timeelapsed, "timeelapsed", "TimeElapsed", "off", t_Bool},
-	{(caddr_t *) & usefirst, "usefirst", "Usefirst", "off", t_Bool},
+	{(caddr_t *) & usefirst, "usefirst", "UseFirst", "on", t_Bool},
 	{(caddr_t *) & verbose, "verbose", "Verbose", "off", t_Bool},
 	{(caddr_t *) & visualname, "visual", "Visual", "", t_String},
 
@@ -632,16 +700,26 @@ static argtype genvars[] =
 	{(caddr_t *) & timeout, "timeout", "Timeout", DEF_TIMEOUT, t_Int},
 	{(caddr_t *) & fontname, "font", "Font", DEF_FONT, t_String},
     {(caddr_t *) & messagefont, "msgfont", "MsgFont", DEF_MSGFONT, t_String},
+#ifdef USE_MB
+	{(caddr_t *) & fontsetname, "fontset", "FontSet", DEF_FONTSET, t_String},
+#endif
     {(caddr_t *) & background, "background", "Background", DEF_BG, t_String},
     {(caddr_t *) & foreground, "foreground", "Foreground", DEF_FG, t_String},
 	{(caddr_t *) & text_user, "username", "Username", DEF_NAME, t_String},
 	{(caddr_t *) & text_pass, "password", "Password", DEF_PASS, t_String},
+#ifdef SAFEWORD
+	{(caddr_t *) & text_dpass, "dynpass", "Dynpass", DEF_DPASS, t_String},
+	{(caddr_t *) & text_fpass, "fixpass", "Fixpass", DEF_FPASS, t_String},
+   {(caddr_t *) & text_chall, "challenge", "Challenge", DEF_CHALL, t_String},
+#endif
 	{(caddr_t *) & text_info, "info", "Info", DEF_INFO, t_String},
      {(caddr_t *) & text_valid, "validate", "Validate", DEF_VALID, t_String},
    {(caddr_t *) & text_invalid, "invalid", "Invalid", DEF_INVALID, t_String},
     {(caddr_t *) & geometry, "geometry", "Geometry", DEF_GEOMETRY, t_String},
 	{(caddr_t *) & icongeometry, "icongeometry", "IconGeometry", DEF_ICONGEOMETRY, t_String},
-
+#ifdef FX
+	{(caddr_t *) & glgeometry, "glgeometry", "GLGeometry", DEF_GLGEOMETRY, t_String},
+#endif
 	{(caddr_t *) & wireframe, "wireframe", "WireFrame", "off", t_Bool},
 
 	{(caddr_t *) & use3d, "use3d", "Use3D", "off", t_Bool},
@@ -656,11 +734,10 @@ static argtype genvars[] =
 	{(caddr_t *) & messagefile, "messagefile", "Messagefile", DEF_MESSAGEFILE, t_String},
 	{(caddr_t *) & message, "message", "Message", DEF_MESSAGE, t_String},
 	{(caddr_t *) & mfont, "mfont", "MFont", DEF_MFONT, t_String},
-{(caddr_t *) & imagefile, "imagefile", "Imagefile", DEF_IMAGEFILE, t_String},
    {(caddr_t *) & neighbors, "neighbors", "Neighbors", DEF_NEIGHBORS, t_Int},
 	{(caddr_t *) & mouse, "mouse", "Mouse", DEF_MOUSE, t_Bool},
 
-#ifdef USE_XLOCKRC
+#if defined( USE_XLOCKRC ) || defined( FALLBACK_XLOCKRC )
 	{(caddr_t *) & cpasswd, "cpasswd", "cpasswd", "", t_String},
 #endif
 #ifdef USE_AUTO_LOGOUT
@@ -702,7 +779,9 @@ static argtype modevars[] =
 {(caddr_t *) & batchcount, "batchcount", "BatchCount", DEF_BATCHCOUNT, t_Int},
 	{(caddr_t *) & cycles, "cycles", "Cycles", DEF_CYCLES, t_Int},
 	{(caddr_t *) & size, "size", "Size", DEF_SIZE, t_Int},
+	{(caddr_t *) & ncolors, "ncolors", "NColors", DEF_NCOLORS, t_Int},
 	{(caddr_t *) & saturation, "saturation", "Saturation", DEF_SATURATION, t_Float},
+ {(caddr_t *) & imagefile, "imagefile", "Imagefile", DEF_IMAGEFILE, t_String}
 };
 
 #define NMODEARGS (sizeof modevars / sizeof modevars[0])
@@ -713,7 +792,9 @@ static int  modevaroffs[NMODEARGS] =
 	offsetof(LockStruct, def_batchcount),
 	offsetof(LockStruct, def_cycles),
 	offsetof(LockStruct, def_size),
-	offsetof(LockStruct, def_saturation)
+	offsetof(LockStruct, def_ncolors),
+	offsetof(LockStruct, def_saturation),
+	offsetof(LockStruct, def_imagefile)
 };
 
 #ifdef VMS
@@ -831,8 +912,12 @@ DumpResources(void)
 			      "cycles", LockProcs[i].def_cycles);
 		(void) printf("%s.%s.%s: %d\n", classname, LockProcs[i].cmdline_arg,
 			      "size", LockProcs[i].def_size);
+		(void) printf("%s.%s.%s: %d\n", classname, LockProcs[i].cmdline_arg,
+			      "ncolors", LockProcs[i].def_ncolors);
 		(void) printf("%s.%s.%s: %g\n", classname, LockProcs[i].cmdline_arg,
 			      "saturation", LockProcs[i].def_saturation);
+		(void) printf("%s.%s.%s: %s\n", classname, LockProcs[i].cmdline_arg,
+			      "imagefile", LockProcs[i].def_imagefile);
 		for (j = 0; j < LockProcs[i].msopt->numvarsdesc; j++)
 			(void) printf("%s.%s.%s: %s\n", classname, LockProcs[i].cmdline_arg,
 				      LockProcs[i].msopt->vars[j].name,
@@ -847,7 +932,7 @@ LowerString(char *s)
 {
 
 	while (*s) {
-		if (isupper(*s))
+		if (isupper((int) *s))
 			*s += ('a' - 'A');
 		s++;
 	}
@@ -909,6 +994,12 @@ GetResource(XrmDatabase database, char *parentname, char *parentclassname,
 				*((char **) valuep) = s;
 			}
 			break;
+		case t_Float:
+			*((float *) valuep) = (float) atof(buffer);
+			break;
+		case t_Int:
+			*((int *) valuep) = atoi(buffer);
+			break;
 		case t_Bool:
 			LowerString(buffer);
 			*((int *) valuep) = (!strcmp(buffer, "true") ||
@@ -916,12 +1007,6 @@ GetResource(XrmDatabase database, char *parentname, char *parentclassname,
 					     !strcmp(buffer, "enabled") ||
 				      !strcmp(buffer, "yes")) ? True : False;
 			break;
-		case t_Int:
-			*((int *) valuep) = atoi(buffer);
-			break;
-		case t_Float:
-			*((float *) valuep) = (float) atof(buffer);
-			break;
 	}
 }
 
@@ -1112,6 +1197,17 @@ open_display(void)
 	char       *buf;
 	struct hostent *host;
 
+#if defined(__cplusplus) || defined(c_plusplus)		/* !__bsdi__ */
+#if HAVE_GETHOSTNAME
+	extern int  gethostname(char *, size_t);
+	extern struct hostent *gethostbyname(const char *);
+
+#else
+#define gethostname(name,namelen) sysinfo(SI_HOSTNAME,name,namelen)
+#endif
+#endif
+
+
 	if (!(dsp = XOpenDisplay(displayname))) {
 		buf = (char *) malloc(strlen(ProgramName) + strlen(displayname) + 80);
 		(void) sprintf(buf,
@@ -1121,15 +1217,12 @@ open_display(void)
 	}
 	displayname = DisplayString(dsp);
 
-
-#if defined(__cplusplus) || defined(c_plusplus)		/* !__bsdi__ */
-#if HAVE_GETHOSTNAME
-	extern int  gethostname(char *, size_t);
-
-#else
-#define gethostname(name,namelen) sysinfo(SI_HOSTNAME,name,namelen)
-#endif
-#endif
+	/*
+	 * only restrict access to other displays if we are locking and if the
+	 * Remote resource is not set.
+	 */
+	if (nolock || inwindow || inroot)
+		remote = True;
 
 	if (gethostname(hostname, MAXHOSTNAMELEN)) {
 		buf = (char *) malloc(strlen(ProgramName) + 80);
@@ -1138,21 +1231,6 @@ open_display(void)
 		error(buf);
 		(void) free((void *) buf);	/* Should never get here */
 	}
-	if (!(host = (struct hostent *) gethostbyname(hostname))) {
-		/* Something wrong with /etc/hosts? */
-		buf = (char *) malloc(strlen(ProgramName) + 80);
-		(void) sprintf(buf,
-			       "%s: Can not get hostbyname.\n", ProgramName);
-		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
-	}
-	/*
-	 * only restrict access to other displays if we are locking and if the
-	 * Remote resource is not set.
-	 */
-	if (nolock || inwindow || inroot)
-		remote = True;
-
 #ifdef VMS
 	if (!remote && ((displayname[0] == '_') |
 			((displayname[0] == 'W') &
@@ -1175,8 +1253,8 @@ open_display(void)
 				exit(status);
 
 			if (strcmp(server_transport, "LOCAL")) {
-				strcat(displayname, "'s display via ");
-				strncat(displayname, server_transport, len);
+				(void) strcat(displayname, "'s display via ");
+				(void) strncat(displayname, server_transport, len);
 				buf = (char *) malloc(strlen(ProgramName) + strlen(displayname) + 80);
 				(void) sprintf(buf,
 					       "%s: can not lock %s\n", ProgramName, displayname);
@@ -1206,7 +1284,14 @@ open_display(void)
 				int         badhost = 1;
 				char      **hp;
 
-				if (strncmp(displayname, host->h_name, n)) {
+				if (!(host = (struct hostent *) gethostbyname(hostname))) {
+					if (debug || verbose) {
+						(void) fprintf(stderr, "%s: Can not get hostbyname.\n",
+							       ProgramName);
+						(void) fprintf(stderr,
+							       "Contact your administrator to fix /etc/hosts.\n");
+					}
+				} else if (strncmp(displayname, host->h_name, n)) {
 					for (hp = host->h_aliases; *hp; hp++) {
 						if (!strncmp(displayname, *hp, n)) {
 							badhost = 0;
@@ -1218,7 +1303,8 @@ open_display(void)
 						buf = (char *) malloc(strlen(ProgramName) +
 						   strlen(displayname) + 80);
 						(void) sprintf(buf,
-							       "%s: can not lock %s's display\n", ProgramName, displayname);
+							       "%s: can not lock %s's display\n",
+						   ProgramName, displayname);
 						error(buf);
 						(void) free((void *) buf);	/* Should never get here */
 					}
@@ -1236,29 +1322,33 @@ printvar(char *classname, argtype var)
 	switch (var.type) {
 		case t_String:
 			(void) fprintf(stderr, "%s.%s: %s\n",
-				  classname, var.name, *((char **) var.var));
-			break;
-		case t_Bool:
-			(void) fprintf(stderr, "%s.%s: %s\n",
-				       classname, var.name, *((int *) var.var)
-				       ? "True" : "False");
-			break;
-		case t_Int:
-			(void) fprintf(stderr, "%s.%s: %d\n",
-				    classname, var.name, *((int *) var.var));
+				       classname, var.name,
+			 (*((char **) var.var)) ? *((char **) var.var) : "");
 			break;
 		case t_Float:
 			(void) fprintf(stderr, "%s.%s: %g\n",
 				  classname, var.name, *((float *) var.var));
 			break;
+		case t_Int:
+			(void) fprintf(stderr, "%s.%s: %d\n",
+				    classname, var.name, *((int *) var.var));
+			break;
+		case t_Bool:
+			(void) fprintf(stderr, "%s.%s: %s\n",
+				       classname, var.name, *((int *) var.var) ? "True" : "False");
+			break;
 	}
 }
 
-
 void
 getResources(int argc, char **argv)
 {
 	XrmDatabase RDB = NULL;
+
+#ifdef USE_MODULES
+	XrmDatabase modulepathDB = NULL;
+
+#endif
 	XrmDatabase nameDB = NULL;
 	XrmDatabase modeDB = NULL;
 	XrmDatabase cmdlineDB = NULL;
@@ -1278,13 +1368,6 @@ getResources(int argc, char **argv)
 
 	XrmInitialize();
 
-	for (i = 0; i < argc; i++) {
-		if (!strncmp(argv[i], "-help", strlen(argv[i])))
-			Help();
-		if (!strncmp(argv[i], "-version", strlen(argv[i])))
-			Version();
-	}
-
 	/*
 	 * get -name arg from command line so you can have different resource
 	 * files for different configurations/machines etc...
@@ -1390,6 +1473,34 @@ getResources(int argc, char **argv)
 		(void) free((void *) buf);
 	}
 
+#ifdef USE_MODULES
+	/*
+	 * Now that all the resource files have been loaded, check for the
+	 * modules directory, so we'll know what modes are available.
+	 */
+	XrmParseCommand(&modulepathDB, modulepathTable, 1, ProgramName,
+			&argc, argv);
+	(void) XrmMergeDatabases(modulepathDB, &RDB);
+	GetResource(RDB, ProgramName, classname, "modulepath", "Modulepath",
+		    t_String, DEF_MODULEPATH, &modulepath);
+
+	/* read modules from modules directory */
+	atexit(UnloadModules);	/* make sure modules get unloaded */
+	LoadModules(modulepath);
+#endif
+
+	/*
+	 * I moved the search for help to here because now the modules
+	 * have been loaded so they can be listed by help.
+	 */
+
+	for (i = 0; i < argc; i++) {
+		if (!strncmp(argv[i], "-help", strlen(argv[i])))
+			Help();
+		if (!strncmp(argv[i], "-version", strlen(argv[i])))
+			Version();
+	}
+
 	XrmParseCommand(&generalDB, genTable, genEntries, ProgramName, &argc, argv);
 /* PURIFY 4.0.1 on Solaris 2 reports an uninitialized memory read on the next
    * line.  PURIFY 4.0.1 on SunOS4 does not report this error. */
@@ -1437,26 +1548,28 @@ getResources(int argc, char **argv)
 
 		(void) sprintf(modename, "%s.%s", ProgramName, LockProcs[i].cmdline_arg);
 		(void) sprintf(modeclassname, "%s.%s", classname, LockProcs[i].cmdline_arg);
-
 		for (j = 0; j < (int) NMODEARGS; j++) {
-			char        buf[16];
+			char       *buf = NULL;
 			void       *p = (void *) ((char *) (&LockProcs[i]) + modevaroffs[j]);
 
- switch (modevars[j].type) {
-               case t_Int:
-                       (void) sprintf(buf, "%d", *((int *) p));
-                       break;
-               case t_Float:
-                       (void) sprintf(buf, "%g", *((float *) p));
-                       break;
-               case t_String:
-                       (void) sprintf(buf, "%s", *((char **) p));
-                       break;
-               case t_Bool:
-                       (void) sprintf(buf, "%s", *((int *) p)
-                                      ? "True" : "False");
-                       break;
-       }
+			switch (modevars[j].type) {
+				case t_String:
+					buf = (char *) malloc(strlen(*((char **) p)) + 1);
+					(void) sprintf(buf, "%s", *((char **) p));
+					break;
+				case t_Float:
+					buf = (char *) malloc(16);
+					(void) sprintf(buf, "%g", *((float *) p));
+					break;
+				case t_Int:
+					buf = (char *) malloc(16);
+					(void) sprintf(buf, "%d", *((int *) p));
+					break;
+				case t_Bool:
+					buf = (char *) malloc(6);
+					(void) sprintf(buf, "%s", *((int *) p) ? "True" : "False");
+					break;
+			}
 			GetResource(RDB, modename, modeclassname,
 				    modevars[j].name, modevars[j].classname,
 				    modevars[j].type, buf, (caddr_t *) p);
@@ -1465,6 +1578,10 @@ getResources(int argc, char **argv)
 				     modevars[j].name, modevars[j].classname,
 				     modevars[j].type, buf, modevars[j].var);
 			}
+			if (buf) {
+				(void) free((void *) buf);
+				buf = NULL;
+			}
 		}
 		if (!ms->numvarsdesc)
 			continue;
@@ -1547,7 +1664,7 @@ getResources(int argc, char **argv)
 		for (i = 0; i < (int) NMODEARGS; i++)
 			printvar(modename, modevars[i]);
 	}
-	if (*visualname == '\0') {
+	if (!strcmp(visualname, "default") || (*visualname == '\0')) {
 		VisualClassWanted = -1;
 		if (verbose)
 			(void) fprintf(stderr, "Using default visual class\n");
@@ -1559,7 +1676,7 @@ getResources(int argc, char **argv)
 	}
 #if HAVE_DIRENT_H
 	/* Evaluate imagefile */
-	if (strcmp(imagefile, DEF_IMAGEFILE)) {
+	if (imagefile && strcmp(imagefile, DEF_IMAGEFILE)) {
 		extern void get_dir(char *fullpath, char *dir, char *filename);
 		extern int  sel_image(struct dirent *name);
 		extern int  scan_dir(const char *directoryname, struct dirent ***namelist,
@@ -1575,8 +1692,9 @@ getResources(int argc, char **argv)
 			for (i = 0; i < num_list; i++)
 				(void) printf("File number %d: %s\n", i, image_list[i]->d_name);
 
-	} else
+	} else {
 		num_list = 0;
+	}
 #endif
 	(void) free((void *) modename);
 	(void) free((void *) modeclassname);
@@ -1589,16 +1707,14 @@ checkResources(void)
 {
 	int         i;
 
+	/* batchcount and size we allow negative
+	   to mean randomize up to that number */
 	if (delay < 0)
 		Syntax("-delay argument must not be negative.");
 	if (cycles < 0)
 		Syntax("-cycles argument must not be negative.");
-#if 0
-	if (batchcount < 0)
-		Syntax("-batchcount argument must not be negative.");
-	if (size < 0)
-		Syntax("-size argument must not be negative.");
-#endif
+	if (ncolors < 2 || ncolors > 200)
+		Syntax("-ncolors argument must be between 2 and 200.");
 	if (saturation < 0.0 || saturation > 1.0)
 		Syntax("-saturation argument must be between 0.0 and 1.0.");
 	if (delta3d < 0.0 || delta3d > 20.0)
