@@ -12,8 +12,8 @@ static const char sccsid[] = "@(#)utils.c	4.04 97/07/28 xlockmore";
  *
  * Changes maintained by David Bagley <bagleyd@bigfoot.com>
  * 08-Jul-96: Bug in strcat_firstword fixed thanks to
-              <Jeffrey_Doggett@caradon.com>.  Fix for ":not found" text
-              that appears after about 40 minutes.
+ *            <Jeffrey_Doggett@caradon.com>.  Fix for ":not found" text
+ *            that appears after about 40 minutes.
  * 04-Apr-96: Added procedures to handle wildcards on filenames
  *            J. Jansen <joukj@crys.chem.uva.nl>
  * 15-May-95: random number generator added, moved hsbramp.c to utils.c .
@@ -61,6 +61,21 @@ XPoint      triangleUnit[2][3] =
 	}
 };
 
+unsigned char stipples[NUMSTIPPLES][STIPPLESIZE] =
+{
+	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},	/* white */
+	{0x11, 0x22, 0x11, 0x22, 0x11, 0x22, 0x11, 0x22},	/* grey+white | stripe */
+	{0x00, 0x66, 0x66, 0x00, 0x00, 0x66, 0x66, 0x00},	/* spots */
+	{0x88, 0x44, 0x22, 0x11, 0x88, 0x44, 0x22, 0x11},	/* lt. / stripe */
+	{0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66},	/* | bars */
+	{0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa},	/* 50% grey */
+	{0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00},	/* - bars */
+	{0xee, 0xdd, 0xbb, 0x77, 0xee, 0xdd, 0xbb, 0x77},	/* dark \ stripe */
+	{0xff, 0x99, 0x99, 0xff, 0xff, 0x99, 0x99, 0xff},	/* spots */
+	{0xaa, 0xff, 0xff, 0x55, 0xaa, 0xff, 0xff, 0x55},	/* black+grey - stripe */
+	{0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff}	/* black */
+};
+
 #include <sys/stat.h>
 
 #ifdef USE_OLD_EVENT_LOOP
@@ -229,7 +244,7 @@ strcat_firstword(char *fword, char *words)
 {
 	while (*fword)
 		fword++;
-	while (*words && !(isspace(*words)))
+	while (*words && !(isspace((int) *words)))
 		*fword++ = *words++;
 	*fword = '\0';
 }
@@ -241,7 +256,7 @@ isRibbon(void)
 }
 
 char       *
-getWords(int screen)
+getWords(int screen, int screens)
 {
 	FILE       *pp;
 	static char *buf = NULL, progerr[BUFSIZ], progrun[BUFSIZ];
@@ -250,7 +265,7 @@ getWords(int screen)
 	int         i;
 
 	if (!buf) {
-		buf = (char *) calloc(MAXSCREENS * BUFSIZ, sizeof (char));
+		buf = (char *) calloc(screens * BUFSIZ, sizeof (char));
 
 		if (!buf)
 			return NULL;
@@ -451,26 +466,19 @@ getFont(Display * display)
 }
 
 #if HAVE_DIRENT_H
-static char *
-randomImage(char *im_file)
+static void
+randomFileFromList(char *directory,
+		   struct dirent **filelist, int numfiles, char *file_local)
 {
-	extern char directory_r[DIRBUF];
-	extern struct dirent **image_list;
-	extern int  num_list;
-	char       *newimf;
 	int         num;
 
-	if (num_list > 0) {
-		num = NRAND(num_list);
-		if ((newimf = malloc(strlen(directory_r)
-			      + strlen(image_list[num]->d_name) + 1)) != 0) {
-			(void) sprintf(newimf, "%s%s",
-				       directory_r, image_list[num]->d_name);
-			free(im_file);
-			im_file = newimf;
+	if (numfiles > 0) {
+		num = NRAND(numfiles);
+		if (strlen(directory) + strlen(filelist[num]->d_name) + 1 < 256) {
+			(void) sprintf(file_local, "%s%s",
+				       directory, filelist[num]->d_name);
 		}
 	}
-	return im_file;
 }
 
 #endif
@@ -569,7 +577,7 @@ sel_image(struct dirent *name)
 	extern char filename_r[MAXNAMLEN];
 	char       *name_tmp = name->d_name;
 	char       *filename_tmp = filename_r;
-	static int  numfrag = -1;
+	int         numfrag = -1;
 	static char *frags[64];
 	int         ip, i;
 
@@ -653,6 +661,31 @@ scan_dir(const char *directoryname, struct dirent ***namelist,
 
 #endif
 
+#if HAVE_DIRENT_H
+static void
+getRandomImageFile(char *randomfile, char *randomfile_local)
+{
+	extern char directory_r[DIRBUF];
+	struct dirent ***images_list;
+	extern struct dirent **image_list;
+	extern int  num_list;
+	extern char filename_r[MAXNAMLEN];
+	extern void get_dir(char *fullpath, char *dir, char *filename);
+	extern int  sel_image(struct dirent *name);
+	extern int  scan_dir(const char *directoryname, struct dirent ***namelist,
+			     int         (*select) (struct dirent *),
+			int         (*compare) (const void *, const void *));
+
+	get_dir(randomfile, directory_r, filename_r);
+	images_list = (struct dirent ***) malloc(sizeof (struct dirent **));
+
+	num_list = scan_dir(directory_r, images_list, sel_image, NULL);
+	image_list = *images_list;
+	randomFileFromList(directory_r, image_list, num_list, randomfile_local);
+}
+
+#endif
+
 extern int  XbmReadFileToImage(char *filename,
 			       int *width, int *height, unsigned char **bits);
 
@@ -675,87 +708,104 @@ static XImage blogo =
 
 void
 getImage(ModeInfo * mi, XImage ** logo,
-	 int width, int height, unsigned char *bits,
+	 int default_width, int default_height, unsigned char *default_bits,
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 	 int default_xpm, char **name,
 #endif
 	 int *graphics_format, Colormap * ncm,
-	 unsigned long *blackpix, unsigned long *whitepix)
+	 unsigned long *black)
 {
-	extern char *imagefile;
 	Display    *display = MI_DISPLAY(mi);
-	Window      window = MI_WINDOW(mi);
+	extern char *imagefile;
+	static char *imagefile_local = NULL;
 
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 	XpmAttributes attrib;
 
 #endif
-
+#if 0
+	/* This probably works best in most cases but for random mode used
+	   with random selection of a file it will fail often. */
+	*ncm = None;
+#else
 	if (!fixedColors(mi))
-		*ncm = XCreateColormap(display, window, MI_VISUAL(mi), AllocNone);
+		*ncm = XCreateColormap(display, MI_WINDOW(mi), MI_VISUAL(mi), AllocNone);
 	else
 		*ncm = None;
-
+#endif
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 	attrib.visual = MI_VISUAL(mi);
-	if (*ncm == None)
-		attrib.colormap = MI_WIN_COLORMAP(mi);
-	else
+	if (*ncm == None) {
+		attrib.colormap = MI_COLORMAP(mi);
+	} else {
 		attrib.colormap = *ncm;
-	attrib.depth = MI_WIN_DEPTH(mi);
+	}
+	attrib.depth = MI_DEPTH(mi);
 	attrib.valuemask = XpmVisual | XpmColormap | XpmDepth;
 #endif
 
 	*graphics_format = 0;
 
+	if (imagefile_local) {
+		(void) free((void *) imagefile_local);
+		imagefile_local = NULL;
+	}
+	if (imagefile && strlen(imagefile)) {
+		imagefile_local = malloc(256);
+		(void) strncpy(imagefile_local, imagefile, sizeof (imagefile_local));
 #if HAVE_DIRENT_H
-	imagefile = randomImage(imagefile);
+		getRandomImageFile(imagefile, imagefile_local);
 #endif
-
-	if (strlen(imagefile))
-		if (readable(imagefile)) {
+	}
+	if (imagefile_local && strlen(imagefile_local)) {
+		if (readable(imagefile_local)) {
 			if (MI_NPIXELS(mi) > 2) {
-				if (RasterSuccess == RasterFileToImage(mi, imagefile, logo)) {
+				if (RasterSuccess == RasterFileToImage(mi, imagefile_local, logo)) {
 					*graphics_format = IS_RASTERFILE;
 					if (!fixedColors(mi))
 						SetImageColors(display, *ncm);
-					*blackpix = GetBlack();
-					*whitepix = GetWhite();
+					*black = GetColor(mi, MI_BLACK_PIXEL(mi));
+					(void) GetColor(mi, MI_WHITE_PIXEL(mi));
+					(void) GetColor(mi, MI_BG_PIXEL(mi));
+					(void) GetColor(mi, MI_FG_PIXEL(mi));
 				}
 			}
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-#ifndef USE_MONOXPM
-			if (MI_NPIXELS(mi) > 2)
-#endif
-			{
-				if (*graphics_format <= 0) {
-					if (*ncm != None)
-						reserveColors(mi, *ncm, blackpix, whitepix);
-					if (XpmSuccess == XpmReadFileToImage(display, imagefile, logo,
-						  (XImage **) NULL, &attrib))
-						*graphics_format = IS_XPMFILE;
-				}
-			}
-#endif
-			if (*graphics_format <= 0)
-				if (!blogo.data) {
-					if (BitmapSuccess == XbmReadFileToImage(imagefile,
-										&blogo.width, &blogo.height, (unsigned char **) &blogo.data)) {
-						blogo.bytes_per_line = (blogo.width + 7) / 8;
-						*graphics_format = IS_XBMFILE;
-						*logo = &blogo;
-					}
-				} else {
-					*graphics_format = IS_XBMDONE;
-					*logo = &blogo;
-				}
-			if (*graphics_format <= 0 && MI_WIN_IS_VERBOSE(mi))
-				(void) fprintf(stderr,
-					       "\"%s\" is in an unrecognized format or not compatible with screen\n",
-					       imagefile);
 		} else
 			(void) fprintf(stderr,
-				  "could not read file \"%s\"\n", imagefile);
+			    "could not read file \"%s\"\n", imagefile_local);
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifndef USE_MONOXPM
+		if (MI_NPIXELS(mi) > 2)
+#endif
+		{
+			if (*graphics_format <= 0) {
+				if (*ncm != None)
+					reserveColors(mi, *ncm, black);
+				if (XpmSuccess == XpmReadFileToImage(display, imagefile_local, logo,
+						  (XImage **) NULL, &attrib))
+					*graphics_format = IS_XPMFILE;
+			}
+		}
+#endif
+		if (*graphics_format <= 0) {
+			if (!blogo.data) {
+				if (BitmapSuccess == XbmReadFileToImage(imagefile_local,
+						 &blogo.width, &blogo.height,
+					   (unsigned char **) &blogo.data)) {
+					blogo.bytes_per_line = (blogo.width + 7) / 8;
+					*graphics_format = IS_XBMFILE;
+					*logo = &blogo;
+				}
+			} else {
+				*graphics_format = IS_XBMDONE;
+				*logo = &blogo;
+			}
+		}
+		if (*graphics_format <= 0 && MI_WIN_IS_VERBOSE(mi))
+			(void) fprintf(stderr,
+				       "\"%s\" is in an unrecognized format or not compatible with screen\n",
+				       imagefile_local);
+	}
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 	if (*graphics_format <= 0 && default_xpm)
 #ifndef USE_MONOXPM
@@ -767,9 +817,9 @@ getImage(ModeInfo * mi, XImage ** logo,
 #endif
 	if (*graphics_format <= 0) {
 		if (!blogo.data) {
-			blogo.data = (char *) bits;
-			blogo.width = width;
-			blogo.height = height;
+			blogo.data = (char *) default_bits;
+			blogo.width = default_width;
+			blogo.height = default_height;
 			blogo.bytes_per_line = (blogo.width + 7) / 8;
 			*graphics_format = IS_XBM;
 		} else
@@ -800,76 +850,59 @@ destroyImage(XImage ** logo, int *graphics_format)
 		case IS_XPMFILE:
 		case IS_RASTERFILE:
 			if (logo)
-				XDestroyImage(*logo);
+				(void) XDestroyImage(*logo);
 			break;
 	}
 	*graphics_format = -1;
 	*logo = NULL;
 }
 
-#if 0
 void
-getBitmap(XImage ** logo, int width, int height, unsigned char *bits,
+getPixmap(Display * display, Drawable drawable,
+	  int default_width, int default_height, unsigned char *default_bits,
+	  int *width, int *height, Pixmap * pixmap,
 	  int *graphics_format)
 {
 	extern char *imagefile;
+	static char *imagefile_local = NULL;
+	int         x_hot, y_hot;	/* dummy */
 
+	if (imagefile_local) {
+		(void) free((void *) imagefile_local);
+		imagefile_local = NULL;
+	}
+	if (imagefile && strlen(imagefile)) {
+		imagefile_local = malloc(256);
+		(void) strncpy(imagefile_local, imagefile, sizeof (imagefile_local));
 #if HAVE_DIRENT_H
-	imagefile = randomImage(imagefile);
+		getRandomImageFile(imagefile, imagefile_local);
 #endif
-
-	if (strlen(imagefile))
-		if (readable(imagefile)) {
-			if (!blogo.data) {
-				if (BitmapSuccess == XbmReadFileToImage(imagefile,
-									&blogo.width, &blogo.height, (unsigned char **) &blogo.data)) {
-					blogo.bytes_per_line = (blogo.width + 7) / 8;
-					*graphics_format = IS_XBMFILE;
-					*logo = &blogo;
-				}
-			} else {
-				*graphics_format = IS_XBMDONE;
-				*logo = &blogo;
+	}
+	if (imagefile_local && strlen(imagefile_local)) {
+		if (readable(imagefile_local)) {
+			if (BitmapSuccess == XReadBitmapFile(display, drawable,
+							     imagefile_local,
+			     (unsigned int *) width, (unsigned int *) height,
+						   pixmap, &x_hot, &y_hot)) {
+				*graphics_format = IS_XBMFILE;
 			}
 			if (*graphics_format <= 0)
 				(void) fprintf(stderr,
-				       "\"%s\" not xbm format\n", imagefile);
+				 "\"%s\" not xbm format\n", imagefile_local);
 		} else
 			(void) fprintf(stderr,
-				  "could not read file \"%s\"\n", imagefile);
+			    "could not read file \"%s\"\n", imagefile_local);
+	}
 	if (*graphics_format <= 0) {
-		if (!blogo.data) {
-			blogo.data = (char *) bits;
-			blogo.width = width;
-			blogo.height = height;
-			blogo.bytes_per_line = (blogo.width + 7) / 8;
-			*graphics_format = IS_XBM;
-		} else
-			*graphics_format = IS_XBMDONE;
-		*logo = &blogo;
+		*width = default_width;
+		*height = default_height;
+		*graphics_format = IS_XBM;
+		*pixmap = XCreateBitmapFromData(display, drawable,
+				     (char *) default_bits, *width, *height);
+
 	}
 }
 
-void
-destroyBitmap(XImage ** logo, int *graphics_format)
-{
-	switch (*graphics_format) {
-		case IS_XBM:
-			blogo.data = NULL;
-			break;
-		case IS_XBMFILE:
-			if (blogo.data) {
-				(void) free((void *) blogo.data);
-				blogo.data = NULL;
-			}
-			break;
-	}
-	*graphics_format = -1;
-	*logo = NULL;
-}
-
-#endif
-
 #ifdef USE_MATHERR
 /* Handle certain math exception errors */
 int
