@@ -21,8 +21,14 @@ static LockStruct LockProcs[] =
 	 "Shows bouncing footballs"},
 	{"braid",
 	 "Shows random braids and knots"},
+	{"bubble",
+	 "Shows popping bubbles"},
 	{"bug",
 	 "Shows Palmiter's bug evolution and garden of Eden"},
+#ifdef USE_GL
+	{"cage",
+	 "Shows the Impossible Cage, an Escher-like GL scene"},
+#endif
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 	{"cartoon",
 	 "Shows bouncing cartoons"},
@@ -45,10 +51,6 @@ static LockStruct LockProcs[] =
 	 "Shows Lloyd's Prisoner's Dilemma simulation"},
 	{"drift",
 	 "Shows cosmic drifting flame fractals"},
-#ifdef USE_GL
-	{"escher",
-	 "Shows some Escher like scenes"},
-#endif
 	{"eyes",
 	 "Shows eyes following a bouncing grelb"},
 	{"fadeplot",
@@ -108,6 +110,8 @@ static LockStruct LockProcs[] =
 	{"maze",
 	 "Shows a random maze and a depth first search solution"},
 #ifdef USE_GL
+	{"moebius",
+	 "Shows Moebius Strip II, an Escher-like GL scene with ants"},
 	{"morph3d",
 	 "Shows GL morphing polyhedra"},
 #endif
@@ -156,6 +160,8 @@ static LockStruct LockProcs[] =
 #ifdef USE_GL
 	{"sproingies",
 	 "Shows Sproingies!  Nontoxic.  Safe for pets and small children"},
+	{"stairs",
+	 "Shows Infinite Stairs, an Escher-like GL scene"},
 #endif
 	{"star",
 	 "Shows a star field with a twist"},
@@ -193,10 +199,10 @@ static LockStruct LockProcs[] =
 	{"bomb",
 	 "Shows a bomb and will autologout after a time"},
 	{"random",
-	 "Shows a random mode from above except blank and bomb"}
+	 "Shows a random mode (except blank and bomb)"}
 #else
 	{"random",
-	 "Shows a random mode from above except blank"}
+	 "Shows a random mode (except blank)"}
 #endif
 };
 
