@@ -1,5 +1,5 @@
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)xmlock.c	4.00 97/01/01 xlockmore";
+static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
 
 #endif
 
@@ -24,6 +24,16 @@ static const char sccsid[] = "@(#)xmlock.c	4.00 97/01/01 xlockmore";
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <sys/signal.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#else
+#define HAVE_UNISTD_H 1
+#endif /* HAVE_CONFIG_H */
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #ifdef VMS
 #include <descrip.h>
 #include <lib$routines.h>
@@ -38,10 +48,6 @@ static const char sccsid[] = "@(#)xmlock.c	4.00 97/01/01 xlockmore";
 #include <Xm/Form.h>
 #include <Xm/Label.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
-
 #if USE_XMU
 #include <X11/Xmu/Editres.h>
 #endif
@@ -59,14 +65,22 @@ static const char sccsid[] = "@(#)xmlock.c	4.00 97/01/01 xlockmore";
 #define  TOGGLES  9
 #define  OPTIONS  8
 
+#define XLOCK "xlock"
+#define WINDOW_WIDTH 160
+#define WINDOW_HEIGHT 100
+#define WINDOW_GEOMETRY "160x100"
+
 /* extern variable */
 extern char *c_Options[OPTIONS];
 extern Widget Menuoption;
 
+static int         numberprocess = -1;	/* PID of xlock */
+
 extern void Setup_Option(Widget MenuBar);
 
 /* Widget */
 Widget      toplevel;
+static Widget      Labelxlock;
 
 static Widget ScrolledListModes, PushButtons[PUSHBUTTONS], Toggles[TOGGLES];
 
@@ -114,7 +128,7 @@ static int  numberinlist = 0;
 static void
 f_PushButtons(Widget w, XtPointer client_data, XtPointer call_data)
 {
-	int         i;
+	int         i, n;
 	char        command[500];
 
 #ifdef VMS
@@ -123,7 +137,7 @@ f_PushButtons(Widget w, XtPointer client_data, XtPointer call_data)
 
 #endif
 
-	(void) strcpy(command, "xlock ");
+	(void) sprintf(command, "%s ", XLOCK);
 
 /* booleans (+/-) options */
 
@@ -153,6 +167,10 @@ f_PushButtons(Widget w, XtPointer client_data, XtPointer call_data)
 			(void) strcat(command, "-inroot ");
 			break;
 		case EXIT:
+			if (numberprocess != -1) {
+				(void) kill(numberprocess, SIGKILL);
+				(void) wait(&n);
+			}
 			exit(0);
 			break;
 	}
@@ -175,7 +193,29 @@ f_PushButtons(Widget w, XtPointer client_data, XtPointer call_data)
 static void
 f_ScrolledListModes(Widget w, XtPointer client_data, XtPointer call_data)
 {
+	char        numberwidget[50];
+	char        str[50];
+	int         n;
+
 	numberinlist = ((XmListCallbackStruct *) call_data)->item_position - 1;
+	(void) sprintf(numberwidget, "%ld", XtWindow(Labelxlock));
+	(void) sprintf(str, "%s", LockProcs[numberinlist].cmdline_arg);
+	if (numberprocess != -1) {
+		(void) kill(numberprocess, SIGKILL);
+		(void) wait(&n);
+	}
+#ifdef VMS
+	if ((numberprocess = vfork()) == -1)
+		/* This may also be a goodie for older Unix systems. J.J. */
+#else
+	if ((numberprocess = fork()) == -1)
+#endif
+		(void) fprintf(stderr, "Fork error\n");
+	else if (numberprocess == 0) {
+		(void) execlp(XLOCK, XLOCK, "-parent", numberwidget,
+		       "-mode", str, "-geometry", WINDOW_GEOMETRY, "-delay", "100000",
+           "-nolock", "-inwindow", 0);
+	}
 }
 
 /* Setup Widget */
@@ -184,28 +224,44 @@ Setup_Widget(Widget father)
 {
 	Arg         args[15];
 	int         i, ac = 0;
-	Widget      Row, PushButtonRow, TogglesRow, Label;
+	Widget      Row, PushButtonRow, TogglesRow, ModeLabel, SwitchLabel;
 	char        string[160];
 	XmString    label_str;
 
 #define NUMPROCS 100		/* Greater than or equal to numprocs */
 	XmString    TabXmStr[NUMPROCS];
 
-/* two labels in the top */
 	ac = 0;
 	XtSetArg(args[ac], XmNrightAttachment, XmATTACH_FORM);
 	ac++;
 	XtSetArg(args[ac], XmNtopAttachment, XmATTACH_FORM);
 	ac++;
-	Label = XmCreateLabel(father, "Switches", args, ac);
-	XtManageChild(Label);
+	XtSetArg(args[ac], XmNheight, WINDOW_HEIGHT);
+	ac++;
+	SwitchLabel = XmCreateLabel(father, "Switches", args, ac);
+	XtManageChild(SwitchLabel);
 	ac = 0;
-	XtSetArg(args[ac], XmNleftAttachment, XmATTACH_FORM);
+	XtSetArg(args[ac], XmNheight, WINDOW_HEIGHT);
 	ac++;
 	XtSetArg(args[ac], XmNtopAttachment, XmATTACH_FORM);
 	ac++;
-	Label = XmCreateLabel(father, "Modes", args, ac);
-	XtManageChild(Label);
+	XtSetArg(args[ac], XmNleftAttachment, XmATTACH_FORM);
+	ac++;
+	ModeLabel = XmCreateLabel(father, "Modes", args, ac);
+	XtManageChild(ModeLabel);
+	ac = 0;
+	XtSetArg(args[ac], XmNwidth, WINDOW_WIDTH);
+	ac++;
+	XtSetArg(args[ac], XmNheight, WINDOW_HEIGHT);
+	ac++;
+	XtSetArg(args[ac], XmNtopAttachment, XmATTACH_FORM);
+	ac++;
+	XtSetArg(args[ac], XmNleftAttachment, XmATTACH_WIDGET);
+	ac++;
+	XtSetArg(args[ac], XmNleftWidget, ModeLabel);
+	ac++;
+	Labelxlock = XmCreateLabel(father, "Window", args, ac);
+	XtManageChild(Labelxlock);
 
 /* buttons in the bottom */
 	ac = 0;
@@ -240,7 +296,7 @@ Setup_Widget(Widget father)
 	ac = 0;
 	XtSetArg(args[ac], XmNtopAttachment, XmATTACH_WIDGET);
 	ac++;
-	XtSetArg(args[ac], XmNtopWidget, Label);
+	XtSetArg(args[ac], XmNtopWidget, Labelxlock);
 	ac++;
 	XtSetArg(args[ac], XmNrightAttachment, XmATTACH_FORM);
 	ac++;
@@ -255,9 +311,8 @@ Setup_Widget(Widget father)
 	Row = XmCreateRowColumn(father, "Row", args, ac);
 
 	if (numprocs > NUMPROCS) {
-    (void) printf("NUMPROCS too small should be at least %d\n", numprocs);
-  }
-
+		(void) printf("NUMPROCS too small should be at least %d\n", numprocs);
+	}
 	for (i = 0; i < numprocs; i++) {
 		(void) sprintf(string, "%-14s%s", LockProcs[i].cmdline_arg,
 			       LockProcs[i].desc);
